/*---------------------------------------------------------------------------*/

/*!  \file  bonds.c
     \brief  manipulations of transient bonds.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007,2008,2018,2019 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/

/*  DO NOT EDIT THIS FILE. This file is generated from bonds.c.m4.  */

#include "common.h"
#include "bonds.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#ifdef HAVE_MATH_H
#include <math.h>
#endif

#include "vector.h"
#include "particles.h"
#include "celllist.h"

#define SQR(x) ((x)*(x))

#define INDEX(i,n) (((i) == -1) ? ((n) - 1) : (((i) == (n)) ? (0) : (i)))
#define LATTICE_VECTOR(i,n,c) (((i) == -1) ? (c) : (((i) == (n)) ? (-(c)) : (0)))
#define IS_LOWER_BOUNDARY(i1,i2,n) (((i1) == 0) && ((i2) == -1))
#define IS_UPPER_BOUNDARY(i1,i2,n) (((i1) == ((n) - 1)) && ((i2) == (n)))


/*---------------------------------------------------------------------------*/
static void create_bonds_equilibrium1(vector1 *r,linked_cell_list *celllist,box_geometry *geometry,
                  transient_bonds *bonds,rng_state *rng,double xi);
static void create_bonds_equilibrium1_in_same_cell(vector1 *r,linked_cell_list *celllist,int k,
                               transient_bonds *bonds,rng_state *rng,double xi);
static void create_bonds_equilibrium1_in_different_cells(vector1 *r,linked_cell_list *celllist,
                                     int k1,int k2,vector1 c,
                                     transient_bonds *bonds,rng_state *rng,double xi);

static void create_bonds_equilibrium2(vector2 *r,linked_cell_list *celllist,box_geometry *geometry,
                  transient_bonds *bonds,rng_state *rng,double xi);
static void create_bonds_equilibrium2_in_same_cell(vector2 *r,linked_cell_list *celllist,int k,
                               transient_bonds *bonds,rng_state *rng,double xi);
static void create_bonds_equilibrium2_in_different_cells(vector2 *r,linked_cell_list *celllist,
                                     int k1,int k2,vector2 c,
                                     transient_bonds *bonds,rng_state *rng,double xi);

static void create_bonds_equilibrium3(vector3 *r,linked_cell_list *celllist,box_geometry *geometry,
                  transient_bonds *bonds,rng_state *rng,double xi);
static void create_bonds_equilibrium3_in_same_cell(vector3 *r,linked_cell_list *celllist,int k,
                               transient_bonds *bonds,rng_state *rng,double xi);
static void create_bonds_equilibrium3_in_different_cells(vector3 *r,linked_cell_list *celllist,
                                     int k1,int k2,vector3 c,
                                     transient_bonds *bonds,rng_state *rng,double xi);

static void construct_bonds_by_transition1(vector1 *r,linked_cell_list *celllist,box_geometry *geometry,
                  transient_bonds *bonds,rng_state *rng,double xi,double dt);
static void construct_bonds_by_transition1_in_same_cell(vector1 *r,linked_cell_list *celllist,int k,
                               transient_bonds *bonds,rng_state *rng,double xi,double dt);
static void construct_bonds_by_transition1_in_different_cells(vector1 *r,linked_cell_list *celllist,
                                     int k1,int k2,vector1 c,
                                     transient_bonds *bonds,rng_state *rng,double xi,double dt);

static void construct_bonds_by_transition2(vector2 *r,linked_cell_list *celllist,box_geometry *geometry,
                  transient_bonds *bonds,rng_state *rng,double xi,double dt);
static void construct_bonds_by_transition2_in_same_cell(vector2 *r,linked_cell_list *celllist,int k,
                               transient_bonds *bonds,rng_state *rng,double xi,double dt);
static void construct_bonds_by_transition2_in_different_cells(vector2 *r,linked_cell_list *celllist,
                                     int k1,int k2,vector2 c,
                                     transient_bonds *bonds,rng_state *rng,double xi,double dt);

static void construct_bonds_by_transition3(vector3 *r,linked_cell_list *celllist,box_geometry *geometry,
                  transient_bonds *bonds,rng_state *rng,double xi,double dt);
static void construct_bonds_by_transition3_in_same_cell(vector3 *r,linked_cell_list *celllist,int k,
                               transient_bonds *bonds,rng_state *rng,double xi,double dt);
static void construct_bonds_by_transition3_in_different_cells(vector3 *r,linked_cell_list *celllist,
                                     int k1,int k2,vector3 c,
                                     transient_bonds *bonds,rng_state *rng,double xi,double dt);

static void destroy_bonds_by_transition1(vector1 *r,transient_bonds *bonds,box_geometry *geometry,
           rng_state *rng,double dt);

static void destroy_bonds_by_transition2(vector2 *r,transient_bonds *bonds,box_geometry *geometry,
           rng_state *rng,double dt);

static void destroy_bonds_by_transition3(vector3 *r,transient_bonds *bonds,box_geometry *geometry,
           rng_state *rng,double dt);

/*---------------------------------------------------------------------------*/
void add_bond(transient_bonds *bonds,int i,int j)
{
    int head_new;

    if(bonds->head == INACTIVE_BOND)
    {
        fprintf(stderr,"warning: cannot add a new bond\n");
        return;
    }

    head_new = bonds->s[bonds->head][1];

    /*printf("%d -> %d: %d %d\n",bonds->head,head_new,i,j);*/

    bonds->s[bonds->head][0] = i;
    bonds->s[bonds->head][1] = j;

    bonds->head  = head_new;
}

/*---------------------------------------------------------------------------*/
void remove_bond(transient_bonds *bonds,int k)
{
    bonds->s[k][0] = INACTIVE_BOND;
    bonds->s[k][1] = bonds->head;

    bonds->head = k;
}

/*---------------------------------------------------------------------------*/
void clear_bonds(transient_bonds *bonds)
{
    int i;

    bonds->head = 0;

    for(i = 0;i < bonds->m_max;i++)
    {
        bonds->s[i][0] = -1;
        bonds->s[i][1] = i + 1;
    }

    bonds->s[bonds->m_max - 1][1] = -1;
}

/*---------------------------------------------------------------------------*/
int total_number_of_bonds(transient_bonds *bonds)
{
    int i,m;

    m = 0;
    for(i = 0;i < bonds->m_max;i++)
    {
        if(IS_ACTIVE_BOND(bonds->s[i]))
            m++;
    }

    return m;
}

/*---------------------------------------------------------------------------*/
void create_bonds_equilibrium(bonded_particles *particles,
                              linked_cell_list *celllist,
                              box_geometry *geometry,
                              rng_state *rng,double xi)
{
    construct_linked_cell_list(celllist,particles,geometry);

        switch(particles->dimension)
    {
        case 1:
            create_bonds_equilibrium1(particles->particles1.r,
                                                celllist,geometry,
                                                &(particles->bonds),rng,xi);
            break;

        case 2:
            create_bonds_equilibrium2(particles->particles2.r,
                                                celllist,geometry,
                                                &(particles->bonds),rng,xi);
            break;

        case 3:
            create_bonds_equilibrium3(particles->particles3.r,
                                                celllist,geometry,
                                                &(particles->bonds),rng,xi);
            break;

        default:
            break;
    }
}

/*---------------------------------------------------------------------------*/
void reconstruct_bonds_by_transition(simulation_condition *condition,
                                     bonded_particles *particles,
                                     linked_cell_list *celllist,
                                     box_geometry *geometry,
                                     rng_state *rng)
{
        switch(particles->dimension)
    {
        case 1:
            destroy_bonds_by_transition1(particles->particles1.r,
                                                     &(particles->bonds),geometry,
                                                     rng,
                                                     (condition->dt / condition->tau));
                    construct_bonds_by_transition1(particles->particles1.r,
                                                   celllist,geometry,
                                                   &(particles->bonds),rng,
                                                   (condition->xi),
                                                   (condition->dt / condition->tau));
            break;

        case 2:
            destroy_bonds_by_transition2(particles->particles2.r,
                                                     &(particles->bonds),geometry,
                                                     rng,
                                                     (condition->dt / condition->tau));
                    construct_bonds_by_transition2(particles->particles2.r,
                                                   celllist,geometry,
                                                   &(particles->bonds),rng,
                                                   (condition->xi),
                                                   (condition->dt / condition->tau));
            break;

        case 3:
            destroy_bonds_by_transition3(particles->particles3.r,
                                                     &(particles->bonds),geometry,
                                                     rng,
                                                     (condition->dt / condition->tau));
                    construct_bonds_by_transition3(particles->particles3.r,
                                                   celllist,geometry,
                                                   &(particles->bonds),rng,
                                                   (condition->xi),
                                                   (condition->dt / condition->tau));
            break;

        default:
            break;
    }
}

/*---------------------------------------------------------------------------*/
void create_bonds_equilibrium1(vector1 *r,linked_cell_list *celllist,box_geometry *geometry,
           transient_bonds *bonds,rng_state *rng,double xi)
{
    int i1,i2,i2_i;
    int cell_index1,cell_index2;
    vector1 c;
    int i2_min,i2_max;

    for(i1 = 0;i1 < celllist->nx;i1++)
    {
        cell_index1 = i1;

        for(i2 = i1 - 1;i2 <= i1 + 1;i2++)
        {
            i2_i = INDEX(i2,celllist->nx);
            cell_index2 = i2_i;
            c[0] = LATTICE_VECTOR(i2,celllist->nx,geometry->l[0]);

            if(cell_index1 == cell_index2)
            {
                create_bonds_equilibrium1_in_same_cell(r,celllist,cell_index1,
                                  bonds,rng,xi);
            }
            else if(cell_index1 < cell_index2)
            {
                create_bonds_equilibrium1_in_different_cells(r,celllist,cell_index1,
                                         cell_index2,c,
                                         bonds,rng,xi);
            }
        }
    }

}

/*---------------------------------------------------------------------------*/
void create_bonds_equilibrium1_in_same_cell(vector1 *r,linked_cell_list *celllist,int k,
                        transient_bonds *bonds,rng_state *rng,double xi)
{
    int i,j;
    double dr2;
    vector1 dr;
        double m_bar;
    int l,m;

    i = celllist->head[k];

    while(!(i < 0))
    {
        j = celllist->head[k];
        while(!(j < 0))
        {
            if(i < j)
            {
                dr[0] = r[i][0] - r[j][0];

                dr2 = SQR(dr[0]);
                if(dr2 < celllist->rc2)
                {
                        m_bar = xi * exp(-0.5 * 1 * dr2);
    m = prand(rng,m_bar);
    for(l = 0;l < m;l++)
        add_bond(bonds,i,j);
                }
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/
void create_bonds_equilibrium1_in_different_cells(vector1 *r,linked_cell_list *celllist,
                              int k1,int k2,vector1 c,
                              transient_bonds *bonds,rng_state *rng,double xi)
{
    int i,j;
    double dr2;
    vector1 dr;
        double m_bar;
    int l,m;

    i = celllist->head[k1];
    while(!(i < 0))
    {
        j = celllist->head[k2];
        while(!(j < 0))
        {
            dr[0] = r[i][0] - r[j][0] + c[0];

            dr2 = SQR(dr[0]);
            if(dr2 < celllist->rc2)
            {
                    m_bar = xi * exp(-0.5 * 1 * dr2);
    m = prand(rng,m_bar);
    for(l = 0;l < m;l++)
        add_bond(bonds,i,j);
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
void create_bonds_equilibrium2(vector2 *r,linked_cell_list *celllist,box_geometry *geometry,
           transient_bonds *bonds,rng_state *rng,double xi)
{
    int i1,j1,i2,j2,i2_i,j2_i;
    int cell_index1,cell_index2;
    vector2 c;
    int i2_min,i2_max;

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        
        for(j1 = 0;j1 < celllist->ny;j1++)
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = j1 * celllist->nx + i1;

                
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    for(i2 = i1 - 1;i2 <= i1 + 1;i2++)
                    {
                        i2_i = INDEX(i2,celllist->nx);

                        cell_index2 = j2_i * celllist->nx + i2_i;

                        c[0] = LATTICE_VECTOR(i2,celllist->nx,geometry->l[0]);
                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        


                        if(cell_index1 == cell_index2)
                        {
                            create_bonds_equilibrium2_in_same_cell(r,celllist,cell_index1,
                                               bonds,rng,xi);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            create_bonds_equilibrium2_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     bonds,rng,xi);
                        }
                    }
                }
                

            }
        }
        
    }
    else
    {
            /*  box deformation  */
        

            /*  boundary cells (j = 0, ny - 1)  */
        for(j1 = 0;j1 < celllist->ny;j1 += (celllist->ny - 1))
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = j1 * celllist->nx + i1;

                
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    if(IS_LOWER_BOUNDARY(j1,j2,celllist->ny))
                    {
                        i2_min = i1 + (int)floor(-1.0 + (geometry->delta_l * celllist->nx) / geometry->l[0]);
                        i2_max = i1 + (int)ceil(1.0 + (geometry->delta_l * celllist->nx) / geometry->l[0]);
                    }
                    else if(IS_UPPER_BOUNDARY(j1,j2,celllist->ny))
                    {
                        i2_min = i1 + (int)floor(-1.0 - (geometry->delta_l * celllist->nx) / geometry->l[0]);
                        i2_max = i1 + (int)ceil(1.0 - (geometry->delta_l * celllist->nx) / geometry->l[0]);
                    }
                    else
                    {
                        i2_min = i1 - 1;
                        i2_max = i1 + 1;
                    }

                    for(i2 = i2_min;i2 <= i2_max;i2++)
                    {
                        i2_i = i2;
                        c[0] = 0.0;

                        while(i2_i < 0)
                        {
                            i2_i += celllist->nx;
                            c[0] += geometry->l[0];
                        }
                        while(i2_i >= celllist->nx)
                        {
                            i2_i -= celllist->nx;
                            c[0] -= geometry->l[0];
                        }

                        if(IS_LOWER_BOUNDARY(j1,j2,celllist->ny))
                        {
                            c[0] += geometry->delta_l;
                        }
                        else if(IS_UPPER_BOUNDARY(j1,j2,celllist->ny))
                        {
                            c[0] -= geometry->delta_l;
                        }


                        cell_index2 = j2_i * celllist->nx + i2_i;

                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        


                        if(cell_index1 == cell_index2)
                        {
                            create_bonds_equilibrium2_in_same_cell(r,celllist,cell_index1,
                                               bonds,rng,xi);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            create_bonds_equilibrium2_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     bonds,rng,xi);
                        }
                    }
                }
                
            }
        }


            /*  non-boundary cells  */
        for(j1 = 1;j1 < celllist->ny - 1;j1++)
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = j1 * celllist->nx + i1;

                
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    for(i2 = i1 - 1;i2 <= i1 + 1;i2++)
                    {
                        i2_i = INDEX(i2,celllist->nx);

                        cell_index2 = j2_i * celllist->nx + i2_i;

                        c[0] = LATTICE_VECTOR(i2,celllist->nx,geometry->l[0]);
                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        


                        if(cell_index1 == cell_index2)
                        {
                            create_bonds_equilibrium2_in_same_cell(r,celllist,cell_index1,
                                               bonds,rng,xi);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            create_bonds_equilibrium2_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     bonds,rng,xi);
                        }
                    }
                }
                

            }
        }
        
    }

}

/*---------------------------------------------------------------------------*/
void create_bonds_equilibrium2_in_same_cell(vector2 *r,linked_cell_list *celllist,int k,
                        transient_bonds *bonds,rng_state *rng,double xi)
{
    int i,j;
    double dr2;
    vector2 dr;
        double m_bar;
    int l,m;

    i = celllist->head[k];

    while(!(i < 0))
    {
        j = celllist->head[k];
        while(!(j < 0))
        {
            if(i < j)
            {
                dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];

                dr2 = SQR(dr[0]) + SQR(dr[1]);
                if(dr2 < celllist->rc2)
                {
                        m_bar = xi * exp(-0.5 * 2 * dr2);
    m = prand(rng,m_bar);
    for(l = 0;l < m;l++)
        add_bond(bonds,i,j);
                }
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/
void create_bonds_equilibrium2_in_different_cells(vector2 *r,linked_cell_list *celllist,
                              int k1,int k2,vector2 c,
                              transient_bonds *bonds,rng_state *rng,double xi)
{
    int i,j;
    double dr2;
    vector2 dr;
        double m_bar;
    int l,m;

    i = celllist->head[k1];
    while(!(i < 0))
    {
        j = celllist->head[k2];
        while(!(j < 0))
        {
            dr[0] = r[i][0] - r[j][0] + c[0];
dr[1] = r[i][1] - r[j][1] + c[1];

            dr2 = SQR(dr[0]) + SQR(dr[1]);
            if(dr2 < celllist->rc2)
            {
                    m_bar = xi * exp(-0.5 * 2 * dr2);
    m = prand(rng,m_bar);
    for(l = 0;l < m;l++)
        add_bond(bonds,i,j);
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
void create_bonds_equilibrium3(vector3 *r,linked_cell_list *celllist,box_geometry *geometry,
           transient_bonds *bonds,rng_state *rng,double xi)
{
    int i1,j1,k1,i2,j2,k2,i2_i,j2_i,k2_i;
    int cell_index1,cell_index2;
    vector3 c;
    int i2_min,i2_max;

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(k1 = 0;k1 < celllist->nz;k1++)
               {
        for(j1 = 0;j1 < celllist->ny;j1++)
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = (k1 * celllist->ny + j1) * celllist->nx + i1;

                for(k2 = k1 - 1;k2 <= k1 + 1;k2++)
                       {
                           k2_i = INDEX(k2,celllist->nz);
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    for(i2 = i1 - 1;i2 <= i1 + 1;i2++)
                    {
                        i2_i = INDEX(i2,celllist->nx);

                        cell_index2 = (k2_i * celllist->ny + j2_i) * celllist->nx + i2_i;

                        c[0] = LATTICE_VECTOR(i2,celllist->nx,geometry->l[0]);
                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        c[2] = LATTICE_VECTOR(k2,celllist->nz,geometry->l[2]);


                        if(cell_index1 == cell_index2)
                        {
                            create_bonds_equilibrium3_in_same_cell(r,celllist,cell_index1,
                                               bonds,rng,xi);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            create_bonds_equilibrium3_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     bonds,rng,xi);
                        }
                    }
                }
                }

            }
        }
        }
    }
    else
    {
            /*  box deformation  */
        for(k1 = 0;k1 < celllist->nz;k1++)
               {

            /*  boundary cells (j = 0, ny - 1)  */
        for(j1 = 0;j1 < celllist->ny;j1 += (celllist->ny - 1))
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = (k1 * celllist->ny + j1) * celllist->nx + i1;

                for(k2 = k1 - 1;k2 <= k1 + 1;k2++)
                       {
                           k2_i = INDEX(k2,celllist->nz);
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    if(IS_LOWER_BOUNDARY(j1,j2,celllist->ny))
                    {
                        i2_min = i1 + (int)floor(-1.0 + (geometry->delta_l * celllist->nx) / geometry->l[0]);
                        i2_max = i1 + (int)ceil(1.0 + (geometry->delta_l * celllist->nx) / geometry->l[0]);
                    }
                    else if(IS_UPPER_BOUNDARY(j1,j2,celllist->ny))
                    {
                        i2_min = i1 + (int)floor(-1.0 - (geometry->delta_l * celllist->nx) / geometry->l[0]);
                        i2_max = i1 + (int)ceil(1.0 - (geometry->delta_l * celllist->nx) / geometry->l[0]);
                    }
                    else
                    {
                        i2_min = i1 - 1;
                        i2_max = i1 + 1;
                    }

                    for(i2 = i2_min;i2 <= i2_max;i2++)
                    {
                        i2_i = i2;
                        c[0] = 0.0;

                        while(i2_i < 0)
                        {
                            i2_i += celllist->nx;
                            c[0] += geometry->l[0];
                        }
                        while(i2_i >= celllist->nx)
                        {
                            i2_i -= celllist->nx;
                            c[0] -= geometry->l[0];
                        }

                        if(IS_LOWER_BOUNDARY(j1,j2,celllist->ny))
                        {
                            c[0] += geometry->delta_l;
                        }
                        else if(IS_UPPER_BOUNDARY(j1,j2,celllist->ny))
                        {
                            c[0] -= geometry->delta_l;
                        }


                        cell_index2 = (k2_i * celllist->ny + j2_i) * celllist->nx + i2_i;

                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        c[2] = LATTICE_VECTOR(k2,celllist->nz,geometry->l[2]);


                        if(cell_index1 == cell_index2)
                        {
                            create_bonds_equilibrium3_in_same_cell(r,celllist,cell_index1,
                                               bonds,rng,xi);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            create_bonds_equilibrium3_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     bonds,rng,xi);
                        }
                    }
                }
                }
            }
        }


            /*  non-boundary cells  */
        for(j1 = 1;j1 < celllist->ny - 1;j1++)
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = (k1 * celllist->ny + j1) * celllist->nx + i1;

                for(k2 = k1 - 1;k2 <= k1 + 1;k2++)
                       {
                           k2_i = INDEX(k2,celllist->nz);
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    for(i2 = i1 - 1;i2 <= i1 + 1;i2++)
                    {
                        i2_i = INDEX(i2,celllist->nx);

                        cell_index2 = (k2_i * celllist->ny + j2_i) * celllist->nx + i2_i;

                        c[0] = LATTICE_VECTOR(i2,celllist->nx,geometry->l[0]);
                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        c[2] = LATTICE_VECTOR(k2,celllist->nz,geometry->l[2]);


                        if(cell_index1 == cell_index2)
                        {
                            create_bonds_equilibrium3_in_same_cell(r,celllist,cell_index1,
                                               bonds,rng,xi);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            create_bonds_equilibrium3_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     bonds,rng,xi);
                        }
                    }
                }
                }

            }
        }
        }
    }

}

/*---------------------------------------------------------------------------*/
void create_bonds_equilibrium3_in_same_cell(vector3 *r,linked_cell_list *celllist,int k,
                        transient_bonds *bonds,rng_state *rng,double xi)
{
    int i,j;
    double dr2;
    vector3 dr;
        double m_bar;
    int l,m;

    i = celllist->head[k];

    while(!(i < 0))
    {
        j = celllist->head[k];
        while(!(j < 0))
        {
            if(i < j)
            {
                dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];
dr[2] = r[i][2] - r[j][2];

                dr2 = SQR(dr[0]) + SQR(dr[1]) + SQR(dr[2]);
                if(dr2 < celllist->rc2)
                {
                        m_bar = xi * exp(-0.5 * 3 * dr2);
    m = prand(rng,m_bar);
    for(l = 0;l < m;l++)
        add_bond(bonds,i,j);
                }
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/
void create_bonds_equilibrium3_in_different_cells(vector3 *r,linked_cell_list *celllist,
                              int k1,int k2,vector3 c,
                              transient_bonds *bonds,rng_state *rng,double xi)
{
    int i,j;
    double dr2;
    vector3 dr;
        double m_bar;
    int l,m;

    i = celllist->head[k1];
    while(!(i < 0))
    {
        j = celllist->head[k2];
        while(!(j < 0))
        {
            dr[0] = r[i][0] - r[j][0] + c[0];
dr[1] = r[i][1] - r[j][1] + c[1];
dr[2] = r[i][2] - r[j][2] + c[2];

            dr2 = SQR(dr[0]) + SQR(dr[1]) + SQR(dr[2]);
            if(dr2 < celllist->rc2)
            {
                    m_bar = xi * exp(-0.5 * 3 * dr2);
    m = prand(rng,m_bar);
    for(l = 0;l < m;l++)
        add_bond(bonds,i,j);
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/


void construct_bonds_by_transition1(vector1 *r,linked_cell_list *celllist,box_geometry *geometry,
           transient_bonds *bonds,rng_state *rng,double xi,double dt)
{
    int i1,i2,i2_i;
    int cell_index1,cell_index2;
    vector1 c;
    int i2_min,i2_max;

    for(i1 = 0;i1 < celllist->nx;i1++)
    {
        cell_index1 = i1;

        for(i2 = i1 - 1;i2 <= i1 + 1;i2++)
        {
            i2_i = INDEX(i2,celllist->nx);
            cell_index2 = i2_i;
            c[0] = LATTICE_VECTOR(i2,celllist->nx,geometry->l[0]);

            if(cell_index1 == cell_index2)
            {
                construct_bonds_by_transition1_in_same_cell(r,celllist,cell_index1,
                                  bonds,rng,xi,dt);
            }
            else if(cell_index1 < cell_index2)
            {
                construct_bonds_by_transition1_in_different_cells(r,celllist,cell_index1,
                                         cell_index2,c,
                                         bonds,rng,xi,dt);
            }
        }
    }

}

/*---------------------------------------------------------------------------*/
void construct_bonds_by_transition1_in_same_cell(vector1 *r,linked_cell_list *celllist,int k,
                        transient_bonds *bonds,rng_state *rng,double xi,double dt)
{
    int i,j;
    double dr2;
    vector1 dr;
    double p_plus;

    i = celllist->head[k];

    while(!(i < 0))
    {
        j = celllist->head[k];
        while(!(j < 0))
        {
            if(i < j)
            {
                dr[0] = r[i][0] - r[j][0];

                dr2 = SQR(dr[0]);
                if(dr2 < celllist->rc2)
                {
                        p_plus = - expm1(-dt * xi * exp(-0.5 * 1 * dr2));
    if(urand(rng) < p_plus)
        add_bond(bonds,i,j);
                }
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/
void construct_bonds_by_transition1_in_different_cells(vector1 *r,linked_cell_list *celllist,
                              int k1,int k2,vector1 c,
                              transient_bonds *bonds,rng_state *rng,double xi,double dt)
{
    int i,j;
    double dr2;
    vector1 dr;
    double p_plus;

    i = celllist->head[k1];
    while(!(i < 0))
    {
        j = celllist->head[k2];
        while(!(j < 0))
        {
            dr[0] = r[i][0] - r[j][0] + c[0];

            dr2 = SQR(dr[0]);
            if(dr2 < celllist->rc2)
            {
                    p_plus = - expm1(-dt * xi * exp(-0.5 * 1 * dr2));
    if(urand(rng) < p_plus)
        add_bond(bonds,i,j);
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
void construct_bonds_by_transition2(vector2 *r,linked_cell_list *celllist,box_geometry *geometry,
           transient_bonds *bonds,rng_state *rng,double xi,double dt)
{
    int i1,j1,i2,j2,i2_i,j2_i;
    int cell_index1,cell_index2;
    vector2 c;
    int i2_min,i2_max;

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        
        for(j1 = 0;j1 < celllist->ny;j1++)
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = j1 * celllist->nx + i1;

                
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    for(i2 = i1 - 1;i2 <= i1 + 1;i2++)
                    {
                        i2_i = INDEX(i2,celllist->nx);

                        cell_index2 = j2_i * celllist->nx + i2_i;

                        c[0] = LATTICE_VECTOR(i2,celllist->nx,geometry->l[0]);
                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        


                        if(cell_index1 == cell_index2)
                        {
                            construct_bonds_by_transition2_in_same_cell(r,celllist,cell_index1,
                                               bonds,rng,xi,dt);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            construct_bonds_by_transition2_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     bonds,rng,xi,dt);
                        }
                    }
                }
                

            }
        }
        
    }
    else
    {
            /*  box deformation  */
        

            /*  boundary cells (j = 0, ny - 1)  */
        for(j1 = 0;j1 < celllist->ny;j1 += (celllist->ny - 1))
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = j1 * celllist->nx + i1;

                
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    if(IS_LOWER_BOUNDARY(j1,j2,celllist->ny))
                    {
                        i2_min = i1 + (int)floor(-1.0 + (geometry->delta_l * celllist->nx) / geometry->l[0]);
                        i2_max = i1 + (int)ceil(1.0 + (geometry->delta_l * celllist->nx) / geometry->l[0]);
                    }
                    else if(IS_UPPER_BOUNDARY(j1,j2,celllist->ny))
                    {
                        i2_min = i1 + (int)floor(-1.0 - (geometry->delta_l * celllist->nx) / geometry->l[0]);
                        i2_max = i1 + (int)ceil(1.0 - (geometry->delta_l * celllist->nx) / geometry->l[0]);
                    }
                    else
                    {
                        i2_min = i1 - 1;
                        i2_max = i1 + 1;
                    }

                    for(i2 = i2_min;i2 <= i2_max;i2++)
                    {
                        i2_i = i2;
                        c[0] = 0.0;

                        while(i2_i < 0)
                        {
                            i2_i += celllist->nx;
                            c[0] += geometry->l[0];
                        }
                        while(i2_i >= celllist->nx)
                        {
                            i2_i -= celllist->nx;
                            c[0] -= geometry->l[0];
                        }

                        if(IS_LOWER_BOUNDARY(j1,j2,celllist->ny))
                        {
                            c[0] += geometry->delta_l;
                        }
                        else if(IS_UPPER_BOUNDARY(j1,j2,celllist->ny))
                        {
                            c[0] -= geometry->delta_l;
                        }


                        cell_index2 = j2_i * celllist->nx + i2_i;

                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        


                        if(cell_index1 == cell_index2)
                        {
                            construct_bonds_by_transition2_in_same_cell(r,celllist,cell_index1,
                                               bonds,rng,xi,dt);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            construct_bonds_by_transition2_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     bonds,rng,xi,dt);
                        }
                    }
                }
                
            }
        }


            /*  non-boundary cells  */
        for(j1 = 1;j1 < celllist->ny - 1;j1++)
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = j1 * celllist->nx + i1;

                
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    for(i2 = i1 - 1;i2 <= i1 + 1;i2++)
                    {
                        i2_i = INDEX(i2,celllist->nx);

                        cell_index2 = j2_i * celllist->nx + i2_i;

                        c[0] = LATTICE_VECTOR(i2,celllist->nx,geometry->l[0]);
                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        


                        if(cell_index1 == cell_index2)
                        {
                            construct_bonds_by_transition2_in_same_cell(r,celllist,cell_index1,
                                               bonds,rng,xi,dt);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            construct_bonds_by_transition2_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     bonds,rng,xi,dt);
                        }
                    }
                }
                

            }
        }
        
    }

}

/*---------------------------------------------------------------------------*/
void construct_bonds_by_transition2_in_same_cell(vector2 *r,linked_cell_list *celllist,int k,
                        transient_bonds *bonds,rng_state *rng,double xi,double dt)
{
    int i,j;
    double dr2;
    vector2 dr;
    double p_plus;

    i = celllist->head[k];

    while(!(i < 0))
    {
        j = celllist->head[k];
        while(!(j < 0))
        {
            if(i < j)
            {
                dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];

                dr2 = SQR(dr[0]) + SQR(dr[1]);
                if(dr2 < celllist->rc2)
                {
                        p_plus = - expm1(-dt * xi * exp(-0.5 * 2 * dr2));
    if(urand(rng) < p_plus)
        add_bond(bonds,i,j);
                }
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/
void construct_bonds_by_transition2_in_different_cells(vector2 *r,linked_cell_list *celllist,
                              int k1,int k2,vector2 c,
                              transient_bonds *bonds,rng_state *rng,double xi,double dt)
{
    int i,j;
    double dr2;
    vector2 dr;
    double p_plus;

    i = celllist->head[k1];
    while(!(i < 0))
    {
        j = celllist->head[k2];
        while(!(j < 0))
        {
            dr[0] = r[i][0] - r[j][0] + c[0];
dr[1] = r[i][1] - r[j][1] + c[1];

            dr2 = SQR(dr[0]) + SQR(dr[1]);
            if(dr2 < celllist->rc2)
            {
                    p_plus = - expm1(-dt * xi * exp(-0.5 * 2 * dr2));
    if(urand(rng) < p_plus)
        add_bond(bonds,i,j);
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
void construct_bonds_by_transition3(vector3 *r,linked_cell_list *celllist,box_geometry *geometry,
           transient_bonds *bonds,rng_state *rng,double xi,double dt)
{
    int i1,j1,k1,i2,j2,k2,i2_i,j2_i,k2_i;
    int cell_index1,cell_index2;
    vector3 c;
    int i2_min,i2_max;

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(k1 = 0;k1 < celllist->nz;k1++)
               {
        for(j1 = 0;j1 < celllist->ny;j1++)
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = (k1 * celllist->ny + j1) * celllist->nx + i1;

                for(k2 = k1 - 1;k2 <= k1 + 1;k2++)
                       {
                           k2_i = INDEX(k2,celllist->nz);
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    for(i2 = i1 - 1;i2 <= i1 + 1;i2++)
                    {
                        i2_i = INDEX(i2,celllist->nx);

                        cell_index2 = (k2_i * celllist->ny + j2_i) * celllist->nx + i2_i;

                        c[0] = LATTICE_VECTOR(i2,celllist->nx,geometry->l[0]);
                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        c[2] = LATTICE_VECTOR(k2,celllist->nz,geometry->l[2]);


                        if(cell_index1 == cell_index2)
                        {
                            construct_bonds_by_transition3_in_same_cell(r,celllist,cell_index1,
                                               bonds,rng,xi,dt);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            construct_bonds_by_transition3_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     bonds,rng,xi,dt);
                        }
                    }
                }
                }

            }
        }
        }
    }
    else
    {
            /*  box deformation  */
        for(k1 = 0;k1 < celllist->nz;k1++)
               {

            /*  boundary cells (j = 0, ny - 1)  */
        for(j1 = 0;j1 < celllist->ny;j1 += (celllist->ny - 1))
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = (k1 * celllist->ny + j1) * celllist->nx + i1;

                for(k2 = k1 - 1;k2 <= k1 + 1;k2++)
                       {
                           k2_i = INDEX(k2,celllist->nz);
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    if(IS_LOWER_BOUNDARY(j1,j2,celllist->ny))
                    {
                        i2_min = i1 + (int)floor(-1.0 + (geometry->delta_l * celllist->nx) / geometry->l[0]);
                        i2_max = i1 + (int)ceil(1.0 + (geometry->delta_l * celllist->nx) / geometry->l[0]);
                    }
                    else if(IS_UPPER_BOUNDARY(j1,j2,celllist->ny))
                    {
                        i2_min = i1 + (int)floor(-1.0 - (geometry->delta_l * celllist->nx) / geometry->l[0]);
                        i2_max = i1 + (int)ceil(1.0 - (geometry->delta_l * celllist->nx) / geometry->l[0]);
                    }
                    else
                    {
                        i2_min = i1 - 1;
                        i2_max = i1 + 1;
                    }

                    for(i2 = i2_min;i2 <= i2_max;i2++)
                    {
                        i2_i = i2;
                        c[0] = 0.0;

                        while(i2_i < 0)
                        {
                            i2_i += celllist->nx;
                            c[0] += geometry->l[0];
                        }
                        while(i2_i >= celllist->nx)
                        {
                            i2_i -= celllist->nx;
                            c[0] -= geometry->l[0];
                        }

                        if(IS_LOWER_BOUNDARY(j1,j2,celllist->ny))
                        {
                            c[0] += geometry->delta_l;
                        }
                        else if(IS_UPPER_BOUNDARY(j1,j2,celllist->ny))
                        {
                            c[0] -= geometry->delta_l;
                        }


                        cell_index2 = (k2_i * celllist->ny + j2_i) * celllist->nx + i2_i;

                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        c[2] = LATTICE_VECTOR(k2,celllist->nz,geometry->l[2]);


                        if(cell_index1 == cell_index2)
                        {
                            construct_bonds_by_transition3_in_same_cell(r,celllist,cell_index1,
                                               bonds,rng,xi,dt);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            construct_bonds_by_transition3_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     bonds,rng,xi,dt);
                        }
                    }
                }
                }
            }
        }


            /*  non-boundary cells  */
        for(j1 = 1;j1 < celllist->ny - 1;j1++)
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = (k1 * celllist->ny + j1) * celllist->nx + i1;

                for(k2 = k1 - 1;k2 <= k1 + 1;k2++)
                       {
                           k2_i = INDEX(k2,celllist->nz);
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    for(i2 = i1 - 1;i2 <= i1 + 1;i2++)
                    {
                        i2_i = INDEX(i2,celllist->nx);

                        cell_index2 = (k2_i * celllist->ny + j2_i) * celllist->nx + i2_i;

                        c[0] = LATTICE_VECTOR(i2,celllist->nx,geometry->l[0]);
                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        c[2] = LATTICE_VECTOR(k2,celllist->nz,geometry->l[2]);


                        if(cell_index1 == cell_index2)
                        {
                            construct_bonds_by_transition3_in_same_cell(r,celllist,cell_index1,
                                               bonds,rng,xi,dt);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            construct_bonds_by_transition3_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     bonds,rng,xi,dt);
                        }
                    }
                }
                }

            }
        }
        }
    }

}

/*---------------------------------------------------------------------------*/
void construct_bonds_by_transition3_in_same_cell(vector3 *r,linked_cell_list *celllist,int k,
                        transient_bonds *bonds,rng_state *rng,double xi,double dt)
{
    int i,j;
    double dr2;
    vector3 dr;
    double p_plus;

    i = celllist->head[k];

    while(!(i < 0))
    {
        j = celllist->head[k];
        while(!(j < 0))
        {
            if(i < j)
            {
                dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];
dr[2] = r[i][2] - r[j][2];

                dr2 = SQR(dr[0]) + SQR(dr[1]) + SQR(dr[2]);
                if(dr2 < celllist->rc2)
                {
                        p_plus = - expm1(-dt * xi * exp(-0.5 * 3 * dr2));
    if(urand(rng) < p_plus)
        add_bond(bonds,i,j);
                }
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/
void construct_bonds_by_transition3_in_different_cells(vector3 *r,linked_cell_list *celllist,
                              int k1,int k2,vector3 c,
                              transient_bonds *bonds,rng_state *rng,double xi,double dt)
{
    int i,j;
    double dr2;
    vector3 dr;
    double p_plus;

    i = celllist->head[k1];
    while(!(i < 0))
    {
        j = celllist->head[k2];
        while(!(j < 0))
        {
            dr[0] = r[i][0] - r[j][0] + c[0];
dr[1] = r[i][1] - r[j][1] + c[1];
dr[2] = r[i][2] - r[j][2] + c[2];

            dr2 = SQR(dr[0]) + SQR(dr[1]) + SQR(dr[2]);
            if(dr2 < celllist->rc2)
            {
                    p_plus = - expm1(-dt * xi * exp(-0.5 * 3 * dr2));
    if(urand(rng) < p_plus)
        add_bond(bonds,i,j);
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/


void destroy_bonds_by_transition1(vector1 *r,transient_bonds *bonds,box_geometry *geometry,
           rng_state *rng,double dt)
{
    int i,j,k;
    vector1 dr;
    double p_minus;

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            dr[0] = r[i][0] - r[j][0];

            if(dr[0] >= 0.5 * geometry->l[0])
                      {
                          dr[0] -= geometry->l[0];
                      }
                      else if(dr[0] < - 0.5 * geometry->l[0])
                      {
                          dr[0] += geometry->l[0];
                      }

                p_minus = - expm1(-dt);
    if(urand(rng) < p_minus)
        remove_bond(bonds,k);
        }
    }
    else
    {
            /*  box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            /*  calculate the bond force.  */
            dr[0] = r[i][0] - r[j][0];

            if(dr[1] >= 0.5 * geometry->l[1])
            {
                dr[1] -= geometry->l[1];
                dr[0] -= geometry->delta_l;
            }
            else if(dr[1] < - 0.5 * geometry->l[1])
            {
                dr[1] += geometry->l[1];
                dr[0] += geometry->delta_l;
            }
            while(dr[0] >= 0.5 * geometry->l[0])
            {
                dr[0] -= geometry->l[0];
            }
            while(dr[0] < - 0.5 * geometry->l[0])
            {
                dr[0] += geometry->l[0];
            }
            

                p_minus = - expm1(-dt);
    if(urand(rng) < p_minus)
        remove_bond(bonds,k);
        }
    }
}

/*---------------------------------------------------------------------------*/
void destroy_bonds_by_transition2(vector2 *r,transient_bonds *bonds,box_geometry *geometry,
           rng_state *rng,double dt)
{
    int i,j,k;
    vector2 dr;
    double p_minus;

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];

            if(dr[0] >= 0.5 * geometry->l[0])
                      {
                          dr[0] -= geometry->l[0];
                      }
                      else if(dr[0] < - 0.5 * geometry->l[0])
                      {
                          dr[0] += geometry->l[0];
                      }
if(dr[1] >= 0.5 * geometry->l[1])
                      {
                          dr[1] -= geometry->l[1];
                      }
                      else if(dr[1] < - 0.5 * geometry->l[1])
                      {
                          dr[1] += geometry->l[1];
                      }

                p_minus = - expm1(-dt);
    if(urand(rng) < p_minus)
        remove_bond(bonds,k);
        }
    }
    else
    {
            /*  box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            /*  calculate the bond force.  */
            dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];

            if(dr[1] >= 0.5 * geometry->l[1])
            {
                dr[1] -= geometry->l[1];
                dr[0] -= geometry->delta_l;
            }
            else if(dr[1] < - 0.5 * geometry->l[1])
            {
                dr[1] += geometry->l[1];
                dr[0] += geometry->delta_l;
            }
            while(dr[0] >= 0.5 * geometry->l[0])
            {
                dr[0] -= geometry->l[0];
            }
            while(dr[0] < - 0.5 * geometry->l[0])
            {
                dr[0] += geometry->l[0];
            }
            

                p_minus = - expm1(-dt);
    if(urand(rng) < p_minus)
        remove_bond(bonds,k);
        }
    }
}

/*---------------------------------------------------------------------------*/
void destroy_bonds_by_transition3(vector3 *r,transient_bonds *bonds,box_geometry *geometry,
           rng_state *rng,double dt)
{
    int i,j,k;
    vector3 dr;
    double p_minus;

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];
dr[2] = r[i][2] - r[j][2];

            if(dr[0] >= 0.5 * geometry->l[0])
                      {
                          dr[0] -= geometry->l[0];
                      }
                      else if(dr[0] < - 0.5 * geometry->l[0])
                      {
                          dr[0] += geometry->l[0];
                      }
if(dr[1] >= 0.5 * geometry->l[1])
                      {
                          dr[1] -= geometry->l[1];
                      }
                      else if(dr[1] < - 0.5 * geometry->l[1])
                      {
                          dr[1] += geometry->l[1];
                      }
if(dr[2] >= 0.5 * geometry->l[2])
                      {
                          dr[2] -= geometry->l[2];
                      }
                      else if(dr[2] < - 0.5 * geometry->l[2])
                      {
                          dr[2] += geometry->l[2];
                      }

                p_minus = - expm1(-dt);
    if(urand(rng) < p_minus)
        remove_bond(bonds,k);
        }
    }
    else
    {
            /*  box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            /*  calculate the bond force.  */
            dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];
dr[2] = r[i][2] - r[j][2];

            if(dr[1] >= 0.5 * geometry->l[1])
            {
                dr[1] -= geometry->l[1];
                dr[0] -= geometry->delta_l;
            }
            else if(dr[1] < - 0.5 * geometry->l[1])
            {
                dr[1] += geometry->l[1];
                dr[0] += geometry->delta_l;
            }
            while(dr[0] >= 0.5 * geometry->l[0])
            {
                dr[0] -= geometry->l[0];
            }
            while(dr[0] < - 0.5 * geometry->l[0])
            {
                dr[0] += geometry->l[0];
            }
            if(dr[2] >= 0.5 * geometry->l[2])
                   {
                       dr[2] -= geometry->l[2];
                   }
                   else if(dr[2] < - 0.5 * geometry->l[2])
                   {
                       dr[2] += geometry->l[2];
                   }

                p_minus = - expm1(-dt);
    if(urand(rng) < p_minus)
        remove_bond(bonds,k);
        }
    }
}

/*---------------------------------------------------------------------------*/
