/*---------------------------------------------------------------------------*/

/*!  \file  celllist.c
     \brief  create or update the linked cell list.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007,2008,2018,2019 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/

/*  DO NOT EDIT THIS FILE. This file is generated from celllist.c.m4.  */

#include "common.h"

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif
#ifdef HAVE_MATH_H
#include <math.h>
#endif

#include "xmalloc.h"
#include "vector.h"
#include "celllist.h"

    /*  the threshold value for the cutoff of site potential and
        bond creation probability.  */
#define CUTOFF_THRESHOLD (1.0e-4)
    /*  the minimum value for the squared cutoff radius for
        the construction of a cell list.  */
#define RC2_MIN (1.0)

/*---------------------------------------------------------------------------*/
static linked_cell_list *allocate_linked_cell_list1(bonded_particles1 *particles,
                                                      double rc2,
                                                    box_geometry *geometry);
static void clear_linked_cell_list1(linked_cell_list *celllist,
                                    bonded_particles1 *particles);
static void construct_linked_cell_list1(linked_cell_list *celllist,
                                        vector1 *r,
                                        bonded_particles1 *particles,
                                        box_geometry *geometry);

static linked_cell_list *allocate_linked_cell_list2(bonded_particles2 *particles,
                                                      double rc2,
                                                    box_geometry *geometry);
static void clear_linked_cell_list2(linked_cell_list *celllist,
                                    bonded_particles2 *particles);
static void construct_linked_cell_list2(linked_cell_list *celllist,
                                        vector2 *r,
                                        bonded_particles2 *particles,
                                        box_geometry *geometry);

static linked_cell_list *allocate_linked_cell_list3(bonded_particles3 *particles,
                                                      double rc2,
                                                    box_geometry *geometry);
static void clear_linked_cell_list3(linked_cell_list *celllist,
                                    bonded_particles3 *particles);
static void construct_linked_cell_list3(linked_cell_list *celllist,
                                        vector3 *r,
                                        bonded_particles3 *particles,
                                        box_geometry *geometry);

/*---------------------------------------------------------------------------*/
double estimate_squared_cutoff_radius(int dimension,double xi)
{
    double rc2;

    if(xi <= CUTOFF_THRESHOLD)
    {
        rc2 = 0.0;
    }
    else
    {
        rc2 = (- log(CUTOFF_THRESHOLD) + log(xi)) * 2.0 / dimension;
    }

    return rc2;
}

/*---------------------------------------------------------------------------*/
linked_cell_list *allocate_linked_cell_list(bonded_particles *particles,
                                            double rc2,
                                            box_geometry *geometry)
{
    linked_cell_list *celllist;

        switch(particles->dimension)
    {
        case 1:
            celllist = allocate_linked_cell_list1(&(particles->particles1),rc2,geometry);
            break;

        case 2:
            celllist = allocate_linked_cell_list2(&(particles->particles2),rc2,geometry);
            break;

        case 3:
            celllist = allocate_linked_cell_list3(&(particles->particles3),rc2,geometry);
            break;

        default:
            celllist = NULL;
            break;
    }

    return celllist;
}

/*---------------------------------------------------------------------------*/
void free_linked_cell_list(linked_cell_list *celllist)
{
    if(celllist == NULL)
        return;

    free(celllist->head);
    free(celllist->list);
    free(celllist);
}

/*---------------------------------------------------------------------------*/
void construct_linked_cell_list(linked_cell_list *celllist,
                                bonded_particles *particles,
                                box_geometry *geometry)
{
        switch(particles->dimension)
    {
        case 1:
            construct_linked_cell_list1(celllist,particles->particles1.r,
                                                  &(particles->particles1),geometry);
            break;

        case 2:
            construct_linked_cell_list2(celllist,particles->particles2.r,
                                                  &(particles->particles2),geometry);
            break;

        case 3:
            construct_linked_cell_list3(celllist,particles->particles3.r,
                                                  &(particles->particles3),geometry);
            break;

        default:
            break;
    }
}

/*---------------------------------------------------------------------------*/
void construct_linked_cell_list_star(linked_cell_list *celllist,
                                     bonded_particles *particles,
                                     box_geometry *geometry)
{
        switch(particles->dimension)
    {
        case 1:
            construct_linked_cell_list1(celllist,particles->particles1.r_star,
                                                  &(particles->particles1),geometry);
            break;

        case 2:
            construct_linked_cell_list2(celllist,particles->particles2.r_star,
                                                  &(particles->particles2),geometry);
            break;

        case 3:
            construct_linked_cell_list3(celllist,particles->particles3.r_star,
                                                  &(particles->particles3),geometry);
            break;

        default:
            break;
    }
}

/*---------------------------------------------------------------------------*/
linked_cell_list *allocate_linked_cell_list1(bonded_particles1 *particles,
                                               double rc2,
                                               box_geometry *geometry)
{
    linked_cell_list *celllist;
    int nc;

    celllist = xmalloc(sizeof(linked_cell_list));

    celllist->rc2 = rc2;
    if(rc2 < RC2_MIN)
    {
        /*  this cutoff radius is used to construct the cell list
            instead of the true radius, to avoid the increase of cells  */
        rc2 = RC2_MIN;
    }

        /*  the number of division in each direction should be
            at least 3, to perform the scan for all pairs correctly.  */
    celllist->dimension = 1;
    celllist->nx = (int)floor(geometry->l[0] / sqrt(rc2));
    if(celllist->nx < 3)
        celllist->nx = 3;
    celllist->ny = 1;
    celllist->nz = 1;

    nc = celllist->nx;

    celllist->head = xmalloc(sizeof(int) * nc);
    celllist->list = xmalloc(sizeof(int) * particles->n);

    clear_linked_cell_list1(celllist,particles);

    return celllist;
}

/*---------------------------------------------------------------------------*/
void clear_linked_cell_list1(linked_cell_list *celllist,
                               bonded_particles1 *particles)
{
    int i,nc;

    nc = celllist->nx;

    for(i = 0;i < nc;i++)
    {
        celllist->head[i] = -1;
    }
    for(i = 0;i < particles->n;i++)
    {
        celllist->list[i] = -1;
    }
}

/*---------------------------------------------------------------------------*/
void construct_linked_cell_list1(linked_cell_list *celllist,
                                 vector1 *r,
                                 bonded_particles1 *particles,
                                 box_geometry *geometry)
{
    int i,nc;
    int index[1];
    int cell_index;
    vector1 c;

    c[0] = celllist->nx / geometry->l[0];
    nc = celllist->nx;

    clear_linked_cell_list1(celllist,particles);

    for(i = 0;i < particles->n;i++)
    {
            /*  calculate the index for the cell.  */
        index[0] = (int)(r[i][0] * c[0]);

        cell_index = index[0];

            /*  add the particle index into the list.  */
        celllist->list[i] = celllist->head[cell_index];
        celllist->head[cell_index] = i;
    }
}

/*---------------------------------------------------------------------------*/
linked_cell_list *allocate_linked_cell_list2(bonded_particles2 *particles,
                                               double rc2,
                                               box_geometry *geometry)
{
    linked_cell_list *celllist;
    int nc;

    celllist = xmalloc(sizeof(linked_cell_list));

    celllist->rc2 = rc2;
    if(rc2 < RC2_MIN)
    {
        /*  this cutoff radius is used to construct the cell list
            instead of the true radius, to avoid the increase of cells  */
        rc2 = RC2_MIN;
    }

        /*  the number of division in each direction should be
            at least 3, to perform the scan for all pairs correctly.  */
    celllist->dimension = 2;
    celllist->nx = (int)floor(geometry->l[0] / sqrt(rc2));
    if(celllist->nx < 3)
        celllist->nx = 3;
    celllist->ny = (int)floor(geometry->l[1] / sqrt(rc2));
    if(celllist->ny < 3)
        celllist->ny = 3;
    celllist->nz = 1;

    nc = celllist->nx * celllist->ny;

    celllist->head = xmalloc(sizeof(int) * nc);
    celllist->list = xmalloc(sizeof(int) * particles->n);

    clear_linked_cell_list2(celllist,particles);

    return celllist;
}

/*---------------------------------------------------------------------------*/
void clear_linked_cell_list2(linked_cell_list *celllist,
                               bonded_particles2 *particles)
{
    int i,nc;

    nc = celllist->nx * celllist->ny;

    for(i = 0;i < nc;i++)
    {
        celllist->head[i] = -1;
    }
    for(i = 0;i < particles->n;i++)
    {
        celllist->list[i] = -1;
    }
}

/*---------------------------------------------------------------------------*/
void construct_linked_cell_list2(linked_cell_list *celllist,
                                 vector2 *r,
                                 bonded_particles2 *particles,
                                 box_geometry *geometry)
{
    int i,nc;
    int index[2];
    int cell_index;
    vector2 c;

    c[0] = celllist->nx / geometry->l[0];
    c[1] = celllist->ny / geometry->l[1];
           nc = celllist->nx * celllist->ny;

    clear_linked_cell_list2(celllist,particles);

    for(i = 0;i < particles->n;i++)
    {
            /*  calculate the index for the cell.  */
        index[0] = (int)(r[i][0] * c[0]);
index[1] = (int)(r[i][1] * c[1]);

        cell_index = index[1] * celllist->nx + index[0];

            /*  add the particle index into the list.  */
        celllist->list[i] = celllist->head[cell_index];
        celllist->head[cell_index] = i;
    }
}

/*---------------------------------------------------------------------------*/
linked_cell_list *allocate_linked_cell_list3(bonded_particles3 *particles,
                                               double rc2,
                                               box_geometry *geometry)
{
    linked_cell_list *celllist;
    int nc;

    celllist = xmalloc(sizeof(linked_cell_list));

    celllist->rc2 = rc2;
    if(rc2 < RC2_MIN)
    {
        /*  this cutoff radius is used to construct the cell list
            instead of the true radius, to avoid the increase of cells  */
        rc2 = RC2_MIN;
    }

        /*  the number of division in each direction should be
            at least 3, to perform the scan for all pairs correctly.  */
    celllist->dimension = 3;
    celllist->nx = (int)floor(geometry->l[0] / sqrt(rc2));
    if(celllist->nx < 3)
        celllist->nx = 3;
    celllist->ny = (int)floor(geometry->l[1] / sqrt(rc2));
    if(celllist->ny < 3)
        celllist->ny = 3;
    celllist->nz = (int)floor(geometry->l[2] / sqrt(rc2));
           if(celllist->nz < 3)
               celllist->nz = 3;

    nc = celllist->nx * celllist->ny * celllist->nz;

    celllist->head = xmalloc(sizeof(int) * nc);
    celllist->list = xmalloc(sizeof(int) * particles->n);

    clear_linked_cell_list3(celllist,particles);

    return celllist;
}

/*---------------------------------------------------------------------------*/
void clear_linked_cell_list3(linked_cell_list *celllist,
                               bonded_particles3 *particles)
{
    int i,nc;

    nc = celllist->nx * celllist->ny * celllist->nz;

    for(i = 0;i < nc;i++)
    {
        celllist->head[i] = -1;
    }
    for(i = 0;i < particles->n;i++)
    {
        celllist->list[i] = -1;
    }
}

/*---------------------------------------------------------------------------*/
void construct_linked_cell_list3(linked_cell_list *celllist,
                                 vector3 *r,
                                 bonded_particles3 *particles,
                                 box_geometry *geometry)
{
    int i,nc;
    int index[3];
    int cell_index;
    vector3 c;

    c[0] = celllist->nx / geometry->l[0];
    c[1] = celllist->ny / geometry->l[1];
           c[2] = celllist->nz / geometry->l[2];
           nc = celllist->nx * celllist->ny * celllist->nz;

    clear_linked_cell_list3(celllist,particles);

    for(i = 0;i < particles->n;i++)
    {
            /*  calculate the index for the cell.  */
        index[0] = (int)(r[i][0] * c[0]);
index[1] = (int)(r[i][1] * c[1]);
index[2] = (int)(r[i][2] * c[2]);

        cell_index = (index[2] * celllist->ny + index[1]) * celllist->nx + index[0];

            /*  add the particle index into the list.  */
        celllist->list[i] = celllist->head[cell_index];
        celllist->head[cell_index] = i;
    }
}

/*---------------------------------------------------------------------------*/
