dnl ---------------------------------------------------------------------------
dnl
dnl  celllist.c.m4
dnl
dnl ---------------------------------------------------------------------------
dnl
dnl  Copyright (C) 2007,2018,2019 Takashi Uneyama
dnl
dnl  This program is free software; you can redistribute it and/or modify
dnl  it under the terms of the GNU General Public License as published by
dnl  the Free Software Foundation; either version 2, or (at your option)
dnl  any later version.
dnl
dnl  This program is distributed in the hope that it will be useful,
dnl  but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl  GNU General Public License for more details.
dnl
dnl  You should have received a copy of the GNU General Public License
dnl  along with this program; if not, write to the Free Software
dnl  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
dnl  USA.
dnl
dnl ---------------------------------------------------------------------------
include(`dimension.m4')`'dnl
GENERATE_HEADER(celllist.c,
                create or update the linked cell list.)

#include "common.h"

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif
#ifdef HAVE_MATH_H
#include <math.h>
#endif

#include "xmalloc.h"
#include "vector.h"
#include "celllist.h"

    /*  the threshold value for the cutoff of site potential and
        bond creation probability.  */
#define CUTOFF_THRESHOLD (1.0e-4)
    /*  the minimum value for the squared cutoff radius for
        the construction of a cell list.  */
#define RC2_MIN (1.0)

/*---------------------------------------------------------------------------*/
PROTOTYPE_FUNCTIONS(`dnl
static linked_cell_list *allocate_linked_cell_list@D@(bonded_particles@D@ *particles,
                                                      double rc2,
                                                    box_geometry *geometry);
static void clear_linked_cell_list@D@(linked_cell_list *celllist,
                                    bonded_particles@D@ *particles);
static void construct_linked_cell_list@D@(linked_cell_list *celllist,
                                        vector@D@ *r,
                                        bonded_particles@D@ *particles,
                                        box_geometry *geometry);'`')

/*---------------------------------------------------------------------------*/
double estimate_squared_cutoff_radius(int dimension,double xi)
{
    double rc2;

    if(xi <= CUTOFF_THRESHOLD)
    {
        rc2 = 0.0;
    }
    else
    {
        rc2 = (- log(CUTOFF_THRESHOLD) + log(xi)) * 2.0 / dimension;
    }

    return rc2;
}

/*---------------------------------------------------------------------------*/
linked_cell_list *allocate_linked_cell_list(bonded_particles *particles,
                                            double rc2,
                                            box_geometry *geometry)
{
    linked_cell_list *celllist;

    SWITCH_FUNCTION(particles->dimension,
                    celllist = allocate_linked_cell_list@D@(&(particles->particles@D@),rc2,geometry);,
                    celllist = NULL;)

    return celllist;
}

/*---------------------------------------------------------------------------*/
void free_linked_cell_list(linked_cell_list *celllist)
{
    if(celllist == NULL)
        return;

    free(celllist->head);
    free(celllist->list);
    free(celllist);
}

/*---------------------------------------------------------------------------*/
void construct_linked_cell_list(linked_cell_list *celllist,
                                bonded_particles *particles,
                                box_geometry *geometry)
{
    SWITCH_FUNCTION(particles->dimension,
                    construct_linked_cell_list@D@(celllist,particles->particles@D@.r,
                                                  &(particles->particles@D@),geometry);)
}

/*---------------------------------------------------------------------------*/
void construct_linked_cell_list_star(linked_cell_list *celllist,
                                     bonded_particles *particles,
                                     box_geometry *geometry)
{
    SWITCH_FUNCTION(particles->dimension,
                    construct_linked_cell_list@D@(celllist,particles->particles@D@.r_star,
                                                  &(particles->particles@D@),geometry);)
}

/*---------------------------------------------------------------------------*/
EXPAND_FUNCTIONS(`dnl
linked_cell_list *allocate_linked_cell_list@D@(bonded_particles@D@ *particles,
                                               double rc2,
                                               box_geometry *geometry)
{
    linked_cell_list *celllist;
    int nc;

    celllist = xmalloc(sizeof(linked_cell_list));

    celllist->rc2 = rc2;
    if(rc2 < RC2_MIN)
    {
        /*  this cutoff radius is used to construct the cell list
            instead of the true radius, to avoid the increase of cells  */
        rc2 = RC2_MIN;
    }

        /*  the number of division in each direction should be
            at least 3, to perform the scan for all pairs correctly.  */
    celllist->dimension = @D@;
    celllist->nx = (int)floor(geometry->l[0] / sqrt(rc2));
    if(celllist->nx < 3)
        celllist->nx = 3;
    ifelse(@D@,1,
    celllist->ny = 1;,
    celllist->ny = (int)floor(geometry->l[1] / sqrt(rc2));
    if(celllist->ny < 3)
        celllist->ny = 3;)
    ifelse(@D@,3,
           celllist->nz = (int)floor(geometry->l[2] / sqrt(rc2));
           if(celllist->nz < 3)
               celllist->nz = 3;,
           celllist->nz = 1;)

    ifelse(@D@,1,
           nc = celllist->nx;,
           @D@,2,
           nc = celllist->nx * celllist->ny;,
           @D@,3,
           nc = celllist->nx * celllist->ny * celllist->nz;)

    celllist->head = xmalloc(sizeof(int) * nc);
    celllist->list = xmalloc(sizeof(int) * particles->n);

    clear_linked_cell_list@D@(celllist,particles);

    return celllist;
}

/*---------------------------------------------------------------------------*/
void clear_linked_cell_list@D@(linked_cell_list *celllist,
                               bonded_particles@D@ *particles)
{
    int i,nc;

    ifelse(@D@,1,
           nc = celllist->nx;,
           @D@,2,
           nc = celllist->nx * celllist->ny;,
           @D@,3,
           nc = celllist->nx * celllist->ny * celllist->nz;)

    for(i = 0;i < nc;i++)
    {
        celllist->head[i] = -1;
    }
    for(i = 0;i < particles->n;i++)
    {
        celllist->list[i] = -1;
    }
}

/*---------------------------------------------------------------------------*/
void construct_linked_cell_list@D@(linked_cell_list *celllist,
                                 vector@D@ *r,
                                 bonded_particles@D@ *particles,
                                 box_geometry *geometry)
{
    int i,nc;
    int index[@D@];
    int cell_index;
    vector@D@ c;

    c[0] = celllist->nx / geometry->l[0];
    ifelse(@D@,1,
           nc = celllist->nx;,
           @D@,2,
           c[1] = celllist->ny / geometry->l[1];
           nc = celllist->nx * celllist->ny;,
           @D@,3,
           c[1] = celllist->ny / geometry->l[1];
           c[2] = celllist->nz / geometry->l[2];
           nc = celllist->nx * celllist->ny * celllist->nz;)

    clear_linked_cell_list@D@(celllist,particles);

    for(i = 0;i < particles->n;i++)
    {
            /*  calculate the index for the cell.  */
        VECTOR@D@(index[@@] = (int)(r[i][@@] * c[@@]);)

        ifelse(@D@,1,
               cell_index = index[0];,
               @D@,2,
               cell_index = index[1] * celllist->nx + index[0];,
               @D@,3,
               cell_index = (index[2] * celllist->ny + index[1]) * celllist->nx + index[0];)

            /*  add the particle index into the list.  */
        celllist->list[i] = celllist->head[cell_index];
        celllist->head[cell_index] = i;
    }
}'`')

/*---------------------------------------------------------------------------*/
