/*---------------------------------------------------------------------------*/

/*!  \file  dxwrite.c
     \brief  output the position data as OpenDX format.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007,2008,2018,2019 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/

/*  DO NOT EDIT THIS FILE. This file is generated from dxwrite.c.m4.  */

#include "common.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include "xmalloc.h"
#include "dxwrite.h"

#define MAX(x,y) (((x) > (y)) ? (x) : (y))

/*---------------------------------------------------------------------------*/
static void write_positions1_dx_binary(char *filename,
                                         bonded_particles1 *particles,
                                         transient_bonds *bonds,
                                         box_geometry *geometry);
static void write_box1_dx(FILE *fp,box_geometry *geometry);
static double calculate_bond_scale_factor1(vector1 r,vector1 q,double *l);

static void write_positions2_dx_binary(char *filename,
                                         bonded_particles2 *particles,
                                         transient_bonds *bonds,
                                         box_geometry *geometry);
static void write_box2_dx(FILE *fp,box_geometry *geometry);
static double calculate_bond_scale_factor2(vector2 r,vector2 q,double *l);

static void write_positions3_dx_binary(char *filename,
                                         bonded_particles3 *particles,
                                         transient_bonds *bonds,
                                         box_geometry *geometry);
static void write_box3_dx(FILE *fp,box_geometry *geometry);
static double calculate_bond_scale_factor3(vector3 r,vector3 q,double *l);

static void count_number_of_bonds_for_particles1(vector1 *r,transient_bonds *bonds,box_geometry *geometry,
           int *m);

static void count_number_of_bonds_for_particles2(vector2 *r,transient_bonds *bonds,box_geometry *geometry,
           int *m);

static void count_number_of_bonds_for_particles3(vector3 *r,transient_bonds *bonds,box_geometry *geometry,
           int *m);

static void extract_active_bonds_in_box1(vector1 *r,transient_bonds *bonds,box_geometry *geometry,
           float *r_bonds, int *m_eff);

static void extract_active_bonds_in_box2(vector2 *r,transient_bonds *bonds,box_geometry *geometry,
           float *r_bonds, int *m_eff);

static void extract_active_bonds_in_box3(vector3 *r,transient_bonds *bonds,box_geometry *geometry,
           float *r_bonds, int *m_eff);

/*---------------------------------------------------------------------------*/
void write_positions_dx(char *filename,bonded_particles *particles,
                        box_geometry *geometry)
{
#if ((SIZEOF_FLOAT == 4) && (SIZEOF_INT == 4))
    int *m;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

        /*  do not generate data in 1d  */
    if(particles->dimension == 1)
        return;

        /*  IEEE floating number format (binary)  */
        switch(particles->dimension)
    {
        case 1:
            write_positions1_dx_binary(filename,&(particles->particles1),
                                                 &(particles->bonds),geometry);
            break;

        case 2:
            write_positions2_dx_binary(filename,&(particles->particles2),
                                                 &(particles->bonds),geometry);
            break;

        case 3:
            write_positions3_dx_binary(filename,&(particles->particles3),
                                                 &(particles->bonds),geometry);
            break;

        default:
            break;
    }
#endif
}

/*---------------------------------------------------------------------------*/
void write_positions1_dx_binary(char *filename,bonded_particles1 *particles,
                                  transient_bonds *bonds,box_geometry *geometry)
{
    FILE *fp;
    int i,j;
    int n;
    float *r_particles,*r_bonds;
    int *id,*m,*s_eff;
    int offset,m_eff;
    char *endian;

#ifdef WORDS_BIGENDIAN
    endian = "msb";
#else
    endian = "lsb";
#endif

    n = particles->n;
    r_particles = xmalloc(sizeof(float) * 1 * n);
    r_bonds = xmalloc(sizeof(float) * 1 * 2 * bonds->m_max);
    id = xmalloc(sizeof(int) * n);
    m = xmalloc(sizeof(int) * n);
    s_eff = xmalloc(sizeof(int) * 2 * bonds->m_max);


        /*  prepare the binary data  */
    for(i = 0;i < particles->n;i++)
    {
        id[i] = i;
        m[i] = 0;
        r_particles[1 * i + 0] = particles->r[i][0];
    }
    count_number_of_bonds_for_particles1(particles->r,bonds,
                                           geometry,m);

    m_eff = 0;
    extract_active_bonds_in_box1(particles->r,bonds,
                                   geometry,r_bonds,&m_eff);
    if(m_eff == 0)
    {
        /*  no bond in this system, add a dummy bond  */
        m_eff = 1;
        r_bonds[1 * 0 + 0] = 0.0f;
        r_bonds[1 * 1 + 0] = 0.0f;
    }
    for(i = 0;i < 2 * m_eff;i++)
    {
        s_eff[i] = i;
    }


    fp = fopen(filename,"wb");


    write_box1_dx(fp,geometry);

        /*  write the id, position, and bond data of bonded particles.  */
    fprintf(fp,"object \"particle_id\" class array type int rank 0 items %d %s binary data 0\n",
            particles->n,endian);
    fprintf(fp,"attribute \"dep\" string \"positions\"\n");
    fprintf(fp,"\n");
    offset = sizeof(int) * particles->n;

    fprintf(fp,"object \"particle_bond_number\" class array type int rank 0 items %d %s binary data %d\n",
            particles->n,endian,offset);
    fprintf(fp,"attribute \"dep\" string \"positions\"\n");
    fprintf(fp,"\n");
    offset += sizeof(int) * particles->n;

    fprintf(fp,"object \"particle_positions\" class array type float rank 1 shape 1 items %d %s ieee data %d\n",
            particles->n,endian,offset);
    fprintf(fp,"attribute \"dep\" string \"positions\"\n");
    fprintf(fp,"\n");
    offset += sizeof(float) * 1 * particles->n;

    if(m_eff > 0)
    {
        fprintf(fp,"object \"bond_positions\" class array type float rank 1 shape 1 items %d %s ieee data %d\n",
                2 * m_eff,endian,offset);
        fprintf(fp,"attribute \"dep\" string \"positions\"\n");
        fprintf(fp,"\n");
        offset += sizeof(float) * 1 * 2 * m_eff;

        fprintf(fp,"object \"bond_connections\" class array type int rank 1 shape 2 items %d %s binary data %d\n",
                m_eff,endian,offset);
        fprintf(fp,"attribute \"element type\" string \"lines\"\n");
        fprintf(fp,"attribute \"dep\" string \"connections\"\n");
        fprintf(fp,"attribute \"ref\" string \"positions\"\n");
        fprintf(fp,"\n");
    }

    fprintf(fp,"object \"particles\" class field\n");
    fprintf(fp,"component \"id\" value \"particle_id\"\n");
    fprintf(fp,"component \"data\" value \"particle_bond_number\"\n");
    fprintf(fp,"component \"positions\" value \"particle_positions\"\n");
    fprintf(fp,"\n");
    if(m_eff > 0)
    {
        fprintf(fp,"object \"bonds\" class field\n");
        fprintf(fp,"component \"positions\" value \"bond_positions\"\n");
        fprintf(fp,"component \"connections\" value \"bond_connections\"\n");
        fprintf(fp,"\n");
    }

    fprintf(fp,"end\n");

        /*  write the binary data  */
    fwrite(id,sizeof(int),n,fp);
    fwrite(m,sizeof(int),n,fp);
    fwrite(r_particles,sizeof(float),1 * n,fp);
    if(m_eff > 0)
    {
        fwrite(r_bonds,sizeof(float),1 * 2 * m_eff,fp);
        fwrite(s_eff,sizeof(int),2 * m_eff,fp);
    }


    fclose(fp);

    free(r_particles);
    free(r_bonds);
    free(id);
    free(m);
    free(s_eff);
}

/*---------------------------------------------------------------------------*/
void write_box1_dx(FILE *fp,box_geometry *geometry)
{
                /*  write the box data  */
    
    fprintf(fp,"attribute \"dep\" string \"positions\"\n");
    fprintf(fp,"\n");
    fprintf(fp,"object \"box\" class field\n");
    fprintf(fp,"component \"positions\" value \"data_box\"\n");
    fprintf(fp,"\n");

    fprintf(fp,"object \"delta_l\" class array type float rank 0 items 1 data follows\n");
    fprintf(fp,"%f\n",geometry->delta_l);
    fprintf(fp,"\n");
}

/*---------------------------------------------------------------------------*/
double calculate_bond_scale_factor1(vector1 r,vector1 q,double *l)
{
    double c;

    c = 1.0;

    if(q[0] + r[0] < 0.0)
    {
        c = - r[0] / q[0];
    }
    if(q[0] + r[0] >= l[0])
    {
        c = (l[0] - r[0]) / q[0];
    }

    if(q[1] + r[1] < 0.0)
    {
        if(c > - r[1] / q[1])
        {
            c = - r[1] / q[1];
        }
    }
    if(q[1] + r[1] >= l[1])
    {
        if(c > (l[1] - r[1]) / q[1])
        {
            c = (l[1] - r[1]) / q[1];
        }
    }

    

    return c;
}

/*---------------------------------------------------------------------------*/
void write_positions2_dx_binary(char *filename,bonded_particles2 *particles,
                                  transient_bonds *bonds,box_geometry *geometry)
{
    FILE *fp;
    int i,j;
    int n;
    float *r_particles,*r_bonds;
    int *id,*m,*s_eff;
    int offset,m_eff;
    char *endian;

#ifdef WORDS_BIGENDIAN
    endian = "msb";
#else
    endian = "lsb";
#endif

    n = particles->n;
    r_particles = xmalloc(sizeof(float) * 2 * n);
    r_bonds = xmalloc(sizeof(float) * 2 * 2 * bonds->m_max);
    id = xmalloc(sizeof(int) * n);
    m = xmalloc(sizeof(int) * n);
    s_eff = xmalloc(sizeof(int) * 2 * bonds->m_max);


        /*  prepare the binary data  */
    for(i = 0;i < particles->n;i++)
    {
        id[i] = i;
        m[i] = 0;
        r_particles[2 * i + 0] = particles->r[i][0];
r_particles[2 * i + 1] = particles->r[i][1];
    }
    count_number_of_bonds_for_particles2(particles->r,bonds,
                                           geometry,m);

    m_eff = 0;
    extract_active_bonds_in_box2(particles->r,bonds,
                                   geometry,r_bonds,&m_eff);
    if(m_eff == 0)
    {
        /*  no bond in this system, add a dummy bond  */
        m_eff = 1;
        r_bonds[2 * 0 + 0] = 0.0f;
r_bonds[2 * 0 + 1] = 0.0f;
        r_bonds[2 * 1 + 0] = 0.0f;
r_bonds[2 * 1 + 1] = 0.0f;
    }
    for(i = 0;i < 2 * m_eff;i++)
    {
        s_eff[i] = i;
    }


    fp = fopen(filename,"wb");


    write_box2_dx(fp,geometry);

        /*  write the id, position, and bond data of bonded particles.  */
    fprintf(fp,"object \"particle_id\" class array type int rank 0 items %d %s binary data 0\n",
            particles->n,endian);
    fprintf(fp,"attribute \"dep\" string \"positions\"\n");
    fprintf(fp,"\n");
    offset = sizeof(int) * particles->n;

    fprintf(fp,"object \"particle_bond_number\" class array type int rank 0 items %d %s binary data %d\n",
            particles->n,endian,offset);
    fprintf(fp,"attribute \"dep\" string \"positions\"\n");
    fprintf(fp,"\n");
    offset += sizeof(int) * particles->n;

    fprintf(fp,"object \"particle_positions\" class array type float rank 1 shape 2 items %d %s ieee data %d\n",
            particles->n,endian,offset);
    fprintf(fp,"attribute \"dep\" string \"positions\"\n");
    fprintf(fp,"\n");
    offset += sizeof(float) * 2 * particles->n;

    if(m_eff > 0)
    {
        fprintf(fp,"object \"bond_positions\" class array type float rank 1 shape 2 items %d %s ieee data %d\n",
                2 * m_eff,endian,offset);
        fprintf(fp,"attribute \"dep\" string \"positions\"\n");
        fprintf(fp,"\n");
        offset += sizeof(float) * 2 * 2 * m_eff;

        fprintf(fp,"object \"bond_connections\" class array type int rank 1 shape 2 items %d %s binary data %d\n",
                m_eff,endian,offset);
        fprintf(fp,"attribute \"element type\" string \"lines\"\n");
        fprintf(fp,"attribute \"dep\" string \"connections\"\n");
        fprintf(fp,"attribute \"ref\" string \"positions\"\n");
        fprintf(fp,"\n");
    }

    fprintf(fp,"object \"particles\" class field\n");
    fprintf(fp,"component \"id\" value \"particle_id\"\n");
    fprintf(fp,"component \"data\" value \"particle_bond_number\"\n");
    fprintf(fp,"component \"positions\" value \"particle_positions\"\n");
    fprintf(fp,"\n");
    if(m_eff > 0)
    {
        fprintf(fp,"object \"bonds\" class field\n");
        fprintf(fp,"component \"positions\" value \"bond_positions\"\n");
        fprintf(fp,"component \"connections\" value \"bond_connections\"\n");
        fprintf(fp,"\n");
    }

    fprintf(fp,"end\n");

        /*  write the binary data  */
    fwrite(id,sizeof(int),n,fp);
    fwrite(m,sizeof(int),n,fp);
    fwrite(r_particles,sizeof(float),2 * n,fp);
    if(m_eff > 0)
    {
        fwrite(r_bonds,sizeof(float),2 * 2 * m_eff,fp);
        fwrite(s_eff,sizeof(int),2 * m_eff,fp);
    }


    fclose(fp);

    free(r_particles);
    free(r_bonds);
    free(id);
    free(m);
    free(s_eff);
}

/*---------------------------------------------------------------------------*/
void write_box2_dx(FILE *fp,box_geometry *geometry)
{
                /*  write the box data  */
    fprintf(fp,"object \"data_box\" class gridpositions counts 2 2\n");
           fprintf(fp,"origin 0.0 0.0\n");
           fprintf(fp,"delta %f 0.0\n",geometry->l[0]);
           fprintf(fp,"delta 0.0 %f\n",geometry->l[1]);
    fprintf(fp,"attribute \"dep\" string \"positions\"\n");
    fprintf(fp,"\n");
    fprintf(fp,"object \"box\" class field\n");
    fprintf(fp,"component \"positions\" value \"data_box\"\n");
    fprintf(fp,"\n");

    fprintf(fp,"object \"delta_l\" class array type float rank 0 items 1 data follows\n");
    fprintf(fp,"%f\n",geometry->delta_l);
    fprintf(fp,"\n");
}

/*---------------------------------------------------------------------------*/
double calculate_bond_scale_factor2(vector2 r,vector2 q,double *l)
{
    double c;

    c = 1.0;

    if(q[0] + r[0] < 0.0)
    {
        c = - r[0] / q[0];
    }
    if(q[0] + r[0] >= l[0])
    {
        c = (l[0] - r[0]) / q[0];
    }

    if(q[1] + r[1] < 0.0)
    {
        if(c > - r[1] / q[1])
        {
            c = - r[1] / q[1];
        }
    }
    if(q[1] + r[1] >= l[1])
    {
        if(c > (l[1] - r[1]) / q[1])
        {
            c = (l[1] - r[1]) / q[1];
        }
    }

    

    return c;
}

/*---------------------------------------------------------------------------*/
void write_positions3_dx_binary(char *filename,bonded_particles3 *particles,
                                  transient_bonds *bonds,box_geometry *geometry)
{
    FILE *fp;
    int i,j;
    int n;
    float *r_particles,*r_bonds;
    int *id,*m,*s_eff;
    int offset,m_eff;
    char *endian;

#ifdef WORDS_BIGENDIAN
    endian = "msb";
#else
    endian = "lsb";
#endif

    n = particles->n;
    r_particles = xmalloc(sizeof(float) * 3 * n);
    r_bonds = xmalloc(sizeof(float) * 3 * 2 * bonds->m_max);
    id = xmalloc(sizeof(int) * n);
    m = xmalloc(sizeof(int) * n);
    s_eff = xmalloc(sizeof(int) * 2 * bonds->m_max);


        /*  prepare the binary data  */
    for(i = 0;i < particles->n;i++)
    {
        id[i] = i;
        m[i] = 0;
        r_particles[3 * i + 0] = particles->r[i][0];
r_particles[3 * i + 1] = particles->r[i][1];
r_particles[3 * i + 2] = particles->r[i][2];
    }
    count_number_of_bonds_for_particles3(particles->r,bonds,
                                           geometry,m);

    m_eff = 0;
    extract_active_bonds_in_box3(particles->r,bonds,
                                   geometry,r_bonds,&m_eff);
    if(m_eff == 0)
    {
        /*  no bond in this system, add a dummy bond  */
        m_eff = 1;
        r_bonds[3 * 0 + 0] = 0.0f;
r_bonds[3 * 0 + 1] = 0.0f;
r_bonds[3 * 0 + 2] = 0.0f;
        r_bonds[3 * 1 + 0] = 0.0f;
r_bonds[3 * 1 + 1] = 0.0f;
r_bonds[3 * 1 + 2] = 0.0f;
    }
    for(i = 0;i < 2 * m_eff;i++)
    {
        s_eff[i] = i;
    }


    fp = fopen(filename,"wb");


    write_box3_dx(fp,geometry);

        /*  write the id, position, and bond data of bonded particles.  */
    fprintf(fp,"object \"particle_id\" class array type int rank 0 items %d %s binary data 0\n",
            particles->n,endian);
    fprintf(fp,"attribute \"dep\" string \"positions\"\n");
    fprintf(fp,"\n");
    offset = sizeof(int) * particles->n;

    fprintf(fp,"object \"particle_bond_number\" class array type int rank 0 items %d %s binary data %d\n",
            particles->n,endian,offset);
    fprintf(fp,"attribute \"dep\" string \"positions\"\n");
    fprintf(fp,"\n");
    offset += sizeof(int) * particles->n;

    fprintf(fp,"object \"particle_positions\" class array type float rank 1 shape 3 items %d %s ieee data %d\n",
            particles->n,endian,offset);
    fprintf(fp,"attribute \"dep\" string \"positions\"\n");
    fprintf(fp,"\n");
    offset += sizeof(float) * 3 * particles->n;

    if(m_eff > 0)
    {
        fprintf(fp,"object \"bond_positions\" class array type float rank 1 shape 3 items %d %s ieee data %d\n",
                2 * m_eff,endian,offset);
        fprintf(fp,"attribute \"dep\" string \"positions\"\n");
        fprintf(fp,"\n");
        offset += sizeof(float) * 3 * 2 * m_eff;

        fprintf(fp,"object \"bond_connections\" class array type int rank 1 shape 2 items %d %s binary data %d\n",
                m_eff,endian,offset);
        fprintf(fp,"attribute \"element type\" string \"lines\"\n");
        fprintf(fp,"attribute \"dep\" string \"connections\"\n");
        fprintf(fp,"attribute \"ref\" string \"positions\"\n");
        fprintf(fp,"\n");
    }

    fprintf(fp,"object \"particles\" class field\n");
    fprintf(fp,"component \"id\" value \"particle_id\"\n");
    fprintf(fp,"component \"data\" value \"particle_bond_number\"\n");
    fprintf(fp,"component \"positions\" value \"particle_positions\"\n");
    fprintf(fp,"\n");
    if(m_eff > 0)
    {
        fprintf(fp,"object \"bonds\" class field\n");
        fprintf(fp,"component \"positions\" value \"bond_positions\"\n");
        fprintf(fp,"component \"connections\" value \"bond_connections\"\n");
        fprintf(fp,"\n");
    }

    fprintf(fp,"end\n");

        /*  write the binary data  */
    fwrite(id,sizeof(int),n,fp);
    fwrite(m,sizeof(int),n,fp);
    fwrite(r_particles,sizeof(float),3 * n,fp);
    if(m_eff > 0)
    {
        fwrite(r_bonds,sizeof(float),3 * 2 * m_eff,fp);
        fwrite(s_eff,sizeof(int),2 * m_eff,fp);
    }


    fclose(fp);

    free(r_particles);
    free(r_bonds);
    free(id);
    free(m);
    free(s_eff);
}

/*---------------------------------------------------------------------------*/
void write_box3_dx(FILE *fp,box_geometry *geometry)
{
                /*  write the box data  */
    fprintf(fp,"object \"data_box\" class gridpositions counts 2 2 2\n");
           fprintf(fp,"origin 0.0 0.0 0.0\n");
           fprintf(fp,"delta %f 0.0 0.0\n",geometry->l[0]);
           fprintf(fp,"delta 0.0 %f 0.0\n",geometry->l[1]);
           fprintf(fp,"delta 0.0 0.0 %f\n",geometry->l[2]);
    fprintf(fp,"attribute \"dep\" string \"positions\"\n");
    fprintf(fp,"\n");
    fprintf(fp,"object \"box\" class field\n");
    fprintf(fp,"component \"positions\" value \"data_box\"\n");
    fprintf(fp,"\n");

    fprintf(fp,"object \"delta_l\" class array type float rank 0 items 1 data follows\n");
    fprintf(fp,"%f\n",geometry->delta_l);
    fprintf(fp,"\n");
}

/*---------------------------------------------------------------------------*/
double calculate_bond_scale_factor3(vector3 r,vector3 q,double *l)
{
    double c;

    c = 1.0;

    if(q[0] + r[0] < 0.0)
    {
        c = - r[0] / q[0];
    }
    if(q[0] + r[0] >= l[0])
    {
        c = (l[0] - r[0]) / q[0];
    }

    if(q[1] + r[1] < 0.0)
    {
        if(c > - r[1] / q[1])
        {
            c = - r[1] / q[1];
        }
    }
    if(q[1] + r[1] >= l[1])
    {
        if(c > (l[1] - r[1]) / q[1])
        {
            c = (l[1] - r[1]) / q[1];
        }
    }

        if(q[2] + r[2] < 0.0)
    {
        if(c > - r[2] / q[2])
        {
            c = - r[2] / q[2];
        }
    }
    if(q[2] + r[2] >= l[2])
    {
        if(c > (l[2] - r[2]) / q[2])
        {
            c = (l[2] - r[2]) / q[2];
        }
    }
    

    return c;
}

/*---------------------------------------------------------------------------*/
void count_number_of_bonds_for_particles1(vector1 *r,transient_bonds *bonds,box_geometry *geometry,
           int *m)
{
    int i,j,k;
    vector1 dr;
    

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            dr[0] = r[i][0] - r[j][0];

            if(dr[0] >= 0.5 * geometry->l[0])
                      {
                          dr[0] -= geometry->l[0];
                      }
                      else if(dr[0] < - 0.5 * geometry->l[0])
                      {
                          dr[0] += geometry->l[0];
                      }

            m[bonds->s[k][0]]++;
m[bonds->s[k][1]]++;
        }
    }
    else
    {
            /*  box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            /*  calculate the bond force.  */
            dr[0] = r[i][0] - r[j][0];

            if(dr[1] >= 0.5 * geometry->l[1])
            {
                dr[1] -= geometry->l[1];
                dr[0] -= geometry->delta_l;
            }
            else if(dr[1] < - 0.5 * geometry->l[1])
            {
                dr[1] += geometry->l[1];
                dr[0] += geometry->delta_l;
            }
            while(dr[0] >= 0.5 * geometry->l[0])
            {
                dr[0] -= geometry->l[0];
            }
            while(dr[0] < - 0.5 * geometry->l[0])
            {
                dr[0] += geometry->l[0];
            }
            

            m[bonds->s[k][0]]++;
m[bonds->s[k][1]]++;
        }
    }
}

/*---------------------------------------------------------------------------*/
void count_number_of_bonds_for_particles2(vector2 *r,transient_bonds *bonds,box_geometry *geometry,
           int *m)
{
    int i,j,k;
    vector2 dr;
    

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];

            if(dr[0] >= 0.5 * geometry->l[0])
                      {
                          dr[0] -= geometry->l[0];
                      }
                      else if(dr[0] < - 0.5 * geometry->l[0])
                      {
                          dr[0] += geometry->l[0];
                      }
if(dr[1] >= 0.5 * geometry->l[1])
                      {
                          dr[1] -= geometry->l[1];
                      }
                      else if(dr[1] < - 0.5 * geometry->l[1])
                      {
                          dr[1] += geometry->l[1];
                      }

            m[bonds->s[k][0]]++;
m[bonds->s[k][1]]++;
        }
    }
    else
    {
            /*  box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            /*  calculate the bond force.  */
            dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];

            if(dr[1] >= 0.5 * geometry->l[1])
            {
                dr[1] -= geometry->l[1];
                dr[0] -= geometry->delta_l;
            }
            else if(dr[1] < - 0.5 * geometry->l[1])
            {
                dr[1] += geometry->l[1];
                dr[0] += geometry->delta_l;
            }
            while(dr[0] >= 0.5 * geometry->l[0])
            {
                dr[0] -= geometry->l[0];
            }
            while(dr[0] < - 0.5 * geometry->l[0])
            {
                dr[0] += geometry->l[0];
            }
            

            m[bonds->s[k][0]]++;
m[bonds->s[k][1]]++;
        }
    }
}

/*---------------------------------------------------------------------------*/
void count_number_of_bonds_for_particles3(vector3 *r,transient_bonds *bonds,box_geometry *geometry,
           int *m)
{
    int i,j,k;
    vector3 dr;
    

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];
dr[2] = r[i][2] - r[j][2];

            if(dr[0] >= 0.5 * geometry->l[0])
                      {
                          dr[0] -= geometry->l[0];
                      }
                      else if(dr[0] < - 0.5 * geometry->l[0])
                      {
                          dr[0] += geometry->l[0];
                      }
if(dr[1] >= 0.5 * geometry->l[1])
                      {
                          dr[1] -= geometry->l[1];
                      }
                      else if(dr[1] < - 0.5 * geometry->l[1])
                      {
                          dr[1] += geometry->l[1];
                      }
if(dr[2] >= 0.5 * geometry->l[2])
                      {
                          dr[2] -= geometry->l[2];
                      }
                      else if(dr[2] < - 0.5 * geometry->l[2])
                      {
                          dr[2] += geometry->l[2];
                      }

            m[bonds->s[k][0]]++;
m[bonds->s[k][1]]++;
        }
    }
    else
    {
            /*  box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            /*  calculate the bond force.  */
            dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];
dr[2] = r[i][2] - r[j][2];

            if(dr[1] >= 0.5 * geometry->l[1])
            {
                dr[1] -= geometry->l[1];
                dr[0] -= geometry->delta_l;
            }
            else if(dr[1] < - 0.5 * geometry->l[1])
            {
                dr[1] += geometry->l[1];
                dr[0] += geometry->delta_l;
            }
            while(dr[0] >= 0.5 * geometry->l[0])
            {
                dr[0] -= geometry->l[0];
            }
            while(dr[0] < - 0.5 * geometry->l[0])
            {
                dr[0] += geometry->l[0];
            }
            if(dr[2] >= 0.5 * geometry->l[2])
                   {
                       dr[2] -= geometry->l[2];
                   }
                   else if(dr[2] < - 0.5 * geometry->l[2])
                   {
                       dr[2] += geometry->l[2];
                   }

            m[bonds->s[k][0]]++;
m[bonds->s[k][1]]++;
        }
    }
}

/*---------------------------------------------------------------------------*/
void extract_active_bonds_in_box1(vector1 *r,transient_bonds *bonds,box_geometry *geometry,
           float *r_bonds, int *m_eff)
{
    int i,j,k;
    vector1 dr;
    vector1 q;
double c;

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            dr[0] = r[i][0] - r[j][0];

            if(dr[0] >= 0.5 * geometry->l[0])
                      {
                          dr[0] -= geometry->l[0];
                      }
                      else if(dr[0] < - 0.5 * geometry->l[0])
                      {
                          dr[0] += geometry->l[0];
                      }

            if(((dr[0] + r[j][0]) < 0.0) || ((dr[0] + r[j][0]) >= geometry->l[0])
   || ((dr[1] + r[j][1]) < 0.0) || ((dr[1] + r[j][1]) >= geometry->l[1])
  )
{
    /*  outside, split to two bonds  */
    q[0] = - dr[0];
    c = calculate_bond_scale_factor1(r[i],q,geometry->l);
    r_bonds[*m_eff * 2 * 1 + 0] = r[i][0];
    r_bonds[*m_eff * 2 * 1 + 1 + 0] = r[i][0] - c * dr[0];
        
    (*m_eff)++;
    if(*m_eff == bonds->m_max)
        return;

    c = calculate_bond_scale_factor1(r[j],dr,geometry->l);
    r_bonds[*m_eff * 2 * 1 + 0] = r[j][0] + c * dr[0];
    r_bonds[*m_eff * 2 * 1 + 1 + 0] = r[j][0];
        
    (*m_eff)++;
    if(*m_eff == bonds->m_max)
        return;
}
else
{
    /*  inside  */
    r_bonds[*m_eff * 2 * 1 + 0] = r[i][0];
    r_bonds[*m_eff * 2 * 1 + 1 + 0] = r[j][0];
        
    (*m_eff)++;
    if(*m_eff == bonds->m_max)
        return;
}
;
        }
    }
    else
    {
            /*  box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            /*  calculate the bond force.  */
            dr[0] = r[i][0] - r[j][0];

            if(dr[1] >= 0.5 * geometry->l[1])
            {
                dr[1] -= geometry->l[1];
                dr[0] -= geometry->delta_l;
            }
            else if(dr[1] < - 0.5 * geometry->l[1])
            {
                dr[1] += geometry->l[1];
                dr[0] += geometry->delta_l;
            }
            while(dr[0] >= 0.5 * geometry->l[0])
            {
                dr[0] -= geometry->l[0];
            }
            while(dr[0] < - 0.5 * geometry->l[0])
            {
                dr[0] += geometry->l[0];
            }
            

            if(((dr[0] + r[j][0]) < 0.0) || ((dr[0] + r[j][0]) >= geometry->l[0])
   || ((dr[1] + r[j][1]) < 0.0) || ((dr[1] + r[j][1]) >= geometry->l[1])
  )
{
    /*  outside, split to two bonds  */
    q[0] = - dr[0];
    c = calculate_bond_scale_factor1(r[i],q,geometry->l);
    r_bonds[*m_eff * 2 * 1 + 0] = r[i][0];
    r_bonds[*m_eff * 2 * 1 + 1 + 0] = r[i][0] - c * dr[0];
        
    (*m_eff)++;
    if(*m_eff == bonds->m_max)
        return;

    c = calculate_bond_scale_factor1(r[j],dr,geometry->l);
    r_bonds[*m_eff * 2 * 1 + 0] = r[j][0] + c * dr[0];
    r_bonds[*m_eff * 2 * 1 + 1 + 0] = r[j][0];
        
    (*m_eff)++;
    if(*m_eff == bonds->m_max)
        return;
}
else
{
    /*  inside  */
    r_bonds[*m_eff * 2 * 1 + 0] = r[i][0];
    r_bonds[*m_eff * 2 * 1 + 1 + 0] = r[j][0];
        
    (*m_eff)++;
    if(*m_eff == bonds->m_max)
        return;
}
;
        }
    }
}

/*---------------------------------------------------------------------------*/
void extract_active_bonds_in_box2(vector2 *r,transient_bonds *bonds,box_geometry *geometry,
           float *r_bonds, int *m_eff)
{
    int i,j,k;
    vector2 dr;
    vector2 q;
double c;

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];

            if(dr[0] >= 0.5 * geometry->l[0])
                      {
                          dr[0] -= geometry->l[0];
                      }
                      else if(dr[0] < - 0.5 * geometry->l[0])
                      {
                          dr[0] += geometry->l[0];
                      }
if(dr[1] >= 0.5 * geometry->l[1])
                      {
                          dr[1] -= geometry->l[1];
                      }
                      else if(dr[1] < - 0.5 * geometry->l[1])
                      {
                          dr[1] += geometry->l[1];
                      }

            if(((dr[0] + r[j][0]) < 0.0) || ((dr[0] + r[j][0]) >= geometry->l[0])
   || ((dr[1] + r[j][1]) < 0.0) || ((dr[1] + r[j][1]) >= geometry->l[1])
  )
{
    /*  outside, split to two bonds  */
    q[0] = - dr[0];
q[1] = - dr[1];
    c = calculate_bond_scale_factor2(r[i],q,geometry->l);
    r_bonds[*m_eff * 2 * 2 + 0] = r[i][0];
r_bonds[*m_eff * 2 * 2 + 1] = r[i][1];
    r_bonds[*m_eff * 2 * 2 + 2 + 0] = r[i][0] - c * dr[0];
r_bonds[*m_eff * 2 * 2 + 2 + 1] = r[i][1] - c * dr[1];
        
    (*m_eff)++;
    if(*m_eff == bonds->m_max)
        return;

    c = calculate_bond_scale_factor2(r[j],dr,geometry->l);
    r_bonds[*m_eff * 2 * 2 + 0] = r[j][0] + c * dr[0];
r_bonds[*m_eff * 2 * 2 + 1] = r[j][1] + c * dr[1];
    r_bonds[*m_eff * 2 * 2 + 2 + 0] = r[j][0];
r_bonds[*m_eff * 2 * 2 + 2 + 1] = r[j][1];
        
    (*m_eff)++;
    if(*m_eff == bonds->m_max)
        return;
}
else
{
    /*  inside  */
    r_bonds[*m_eff * 2 * 2 + 0] = r[i][0];
r_bonds[*m_eff * 2 * 2 + 1] = r[i][1];
    r_bonds[*m_eff * 2 * 2 + 2 + 0] = r[j][0];
r_bonds[*m_eff * 2 * 2 + 2 + 1] = r[j][1];
        
    (*m_eff)++;
    if(*m_eff == bonds->m_max)
        return;
}
;
        }
    }
    else
    {
            /*  box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            /*  calculate the bond force.  */
            dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];

            if(dr[1] >= 0.5 * geometry->l[1])
            {
                dr[1] -= geometry->l[1];
                dr[0] -= geometry->delta_l;
            }
            else if(dr[1] < - 0.5 * geometry->l[1])
            {
                dr[1] += geometry->l[1];
                dr[0] += geometry->delta_l;
            }
            while(dr[0] >= 0.5 * geometry->l[0])
            {
                dr[0] -= geometry->l[0];
            }
            while(dr[0] < - 0.5 * geometry->l[0])
            {
                dr[0] += geometry->l[0];
            }
            

            if(((dr[0] + r[j][0]) < 0.0) || ((dr[0] + r[j][0]) >= geometry->l[0])
   || ((dr[1] + r[j][1]) < 0.0) || ((dr[1] + r[j][1]) >= geometry->l[1])
  )
{
    /*  outside, split to two bonds  */
    q[0] = - dr[0];
q[1] = - dr[1];
    c = calculate_bond_scale_factor2(r[i],q,geometry->l);
    r_bonds[*m_eff * 2 * 2 + 0] = r[i][0];
r_bonds[*m_eff * 2 * 2 + 1] = r[i][1];
    r_bonds[*m_eff * 2 * 2 + 2 + 0] = r[i][0] - c * dr[0];
r_bonds[*m_eff * 2 * 2 + 2 + 1] = r[i][1] - c * dr[1];
        
    (*m_eff)++;
    if(*m_eff == bonds->m_max)
        return;

    c = calculate_bond_scale_factor2(r[j],dr,geometry->l);
    r_bonds[*m_eff * 2 * 2 + 0] = r[j][0] + c * dr[0];
r_bonds[*m_eff * 2 * 2 + 1] = r[j][1] + c * dr[1];
    r_bonds[*m_eff * 2 * 2 + 2 + 0] = r[j][0];
r_bonds[*m_eff * 2 * 2 + 2 + 1] = r[j][1];
        
    (*m_eff)++;
    if(*m_eff == bonds->m_max)
        return;
}
else
{
    /*  inside  */
    r_bonds[*m_eff * 2 * 2 + 0] = r[i][0];
r_bonds[*m_eff * 2 * 2 + 1] = r[i][1];
    r_bonds[*m_eff * 2 * 2 + 2 + 0] = r[j][0];
r_bonds[*m_eff * 2 * 2 + 2 + 1] = r[j][1];
        
    (*m_eff)++;
    if(*m_eff == bonds->m_max)
        return;
}
;
        }
    }
}

/*---------------------------------------------------------------------------*/
void extract_active_bonds_in_box3(vector3 *r,transient_bonds *bonds,box_geometry *geometry,
           float *r_bonds, int *m_eff)
{
    int i,j,k;
    vector3 dr;
    vector3 q;
double c;

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];
dr[2] = r[i][2] - r[j][2];

            if(dr[0] >= 0.5 * geometry->l[0])
                      {
                          dr[0] -= geometry->l[0];
                      }
                      else if(dr[0] < - 0.5 * geometry->l[0])
                      {
                          dr[0] += geometry->l[0];
                      }
if(dr[1] >= 0.5 * geometry->l[1])
                      {
                          dr[1] -= geometry->l[1];
                      }
                      else if(dr[1] < - 0.5 * geometry->l[1])
                      {
                          dr[1] += geometry->l[1];
                      }
if(dr[2] >= 0.5 * geometry->l[2])
                      {
                          dr[2] -= geometry->l[2];
                      }
                      else if(dr[2] < - 0.5 * geometry->l[2])
                      {
                          dr[2] += geometry->l[2];
                      }

            if(((dr[0] + r[j][0]) < 0.0) || ((dr[0] + r[j][0]) >= geometry->l[0])
   || ((dr[1] + r[j][1]) < 0.0) || ((dr[1] + r[j][1]) >= geometry->l[1])
     || ((dr[2] + r[j][2]) < 0.0) || ((dr[2] + r[j][2]) >= geometry->l[2]))
{
    /*  outside, split to two bonds  */
    q[0] = - dr[0];
q[1] = - dr[1];
q[2] = - dr[2];
    c = calculate_bond_scale_factor3(r[i],q,geometry->l);
    r_bonds[*m_eff * 2 * 3 + 0] = r[i][0];
r_bonds[*m_eff * 2 * 3 + 1] = r[i][1];
r_bonds[*m_eff * 2 * 3 + 2] = r[i][2];
    r_bonds[*m_eff * 2 * 3 + 3 + 0] = r[i][0] - c * dr[0];
r_bonds[*m_eff * 2 * 3 + 3 + 1] = r[i][1] - c * dr[1];
r_bonds[*m_eff * 2 * 3 + 3 + 2] = r[i][2] - c * dr[2];
        
    (*m_eff)++;
    if(*m_eff == bonds->m_max)
        return;

    c = calculate_bond_scale_factor3(r[j],dr,geometry->l);
    r_bonds[*m_eff * 2 * 3 + 0] = r[j][0] + c * dr[0];
r_bonds[*m_eff * 2 * 3 + 1] = r[j][1] + c * dr[1];
r_bonds[*m_eff * 2 * 3 + 2] = r[j][2] + c * dr[2];
    r_bonds[*m_eff * 2 * 3 + 3 + 0] = r[j][0];
r_bonds[*m_eff * 2 * 3 + 3 + 1] = r[j][1];
r_bonds[*m_eff * 2 * 3 + 3 + 2] = r[j][2];
        
    (*m_eff)++;
    if(*m_eff == bonds->m_max)
        return;
}
else
{
    /*  inside  */
    r_bonds[*m_eff * 2 * 3 + 0] = r[i][0];
r_bonds[*m_eff * 2 * 3 + 1] = r[i][1];
r_bonds[*m_eff * 2 * 3 + 2] = r[i][2];
    r_bonds[*m_eff * 2 * 3 + 3 + 0] = r[j][0];
r_bonds[*m_eff * 2 * 3 + 3 + 1] = r[j][1];
r_bonds[*m_eff * 2 * 3 + 3 + 2] = r[j][2];
        
    (*m_eff)++;
    if(*m_eff == bonds->m_max)
        return;
}
;
        }
    }
    else
    {
            /*  box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            /*  calculate the bond force.  */
            dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];
dr[2] = r[i][2] - r[j][2];

            if(dr[1] >= 0.5 * geometry->l[1])
            {
                dr[1] -= geometry->l[1];
                dr[0] -= geometry->delta_l;
            }
            else if(dr[1] < - 0.5 * geometry->l[1])
            {
                dr[1] += geometry->l[1];
                dr[0] += geometry->delta_l;
            }
            while(dr[0] >= 0.5 * geometry->l[0])
            {
                dr[0] -= geometry->l[0];
            }
            while(dr[0] < - 0.5 * geometry->l[0])
            {
                dr[0] += geometry->l[0];
            }
            if(dr[2] >= 0.5 * geometry->l[2])
                   {
                       dr[2] -= geometry->l[2];
                   }
                   else if(dr[2] < - 0.5 * geometry->l[2])
                   {
                       dr[2] += geometry->l[2];
                   }

            if(((dr[0] + r[j][0]) < 0.0) || ((dr[0] + r[j][0]) >= geometry->l[0])
   || ((dr[1] + r[j][1]) < 0.0) || ((dr[1] + r[j][1]) >= geometry->l[1])
     || ((dr[2] + r[j][2]) < 0.0) || ((dr[2] + r[j][2]) >= geometry->l[2]))
{
    /*  outside, split to two bonds  */
    q[0] = - dr[0];
q[1] = - dr[1];
q[2] = - dr[2];
    c = calculate_bond_scale_factor3(r[i],q,geometry->l);
    r_bonds[*m_eff * 2 * 3 + 0] = r[i][0];
r_bonds[*m_eff * 2 * 3 + 1] = r[i][1];
r_bonds[*m_eff * 2 * 3 + 2] = r[i][2];
    r_bonds[*m_eff * 2 * 3 + 3 + 0] = r[i][0] - c * dr[0];
r_bonds[*m_eff * 2 * 3 + 3 + 1] = r[i][1] - c * dr[1];
r_bonds[*m_eff * 2 * 3 + 3 + 2] = r[i][2] - c * dr[2];
        
    (*m_eff)++;
    if(*m_eff == bonds->m_max)
        return;

    c = calculate_bond_scale_factor3(r[j],dr,geometry->l);
    r_bonds[*m_eff * 2 * 3 + 0] = r[j][0] + c * dr[0];
r_bonds[*m_eff * 2 * 3 + 1] = r[j][1] + c * dr[1];
r_bonds[*m_eff * 2 * 3 + 2] = r[j][2] + c * dr[2];
    r_bonds[*m_eff * 2 * 3 + 3 + 0] = r[j][0];
r_bonds[*m_eff * 2 * 3 + 3 + 1] = r[j][1];
r_bonds[*m_eff * 2 * 3 + 3 + 2] = r[j][2];
        
    (*m_eff)++;
    if(*m_eff == bonds->m_max)
        return;
}
else
{
    /*  inside  */
    r_bonds[*m_eff * 2 * 3 + 0] = r[i][0];
r_bonds[*m_eff * 2 * 3 + 1] = r[i][1];
r_bonds[*m_eff * 2 * 3 + 2] = r[i][2];
    r_bonds[*m_eff * 2 * 3 + 3 + 0] = r[j][0];
r_bonds[*m_eff * 2 * 3 + 3 + 1] = r[j][1];
r_bonds[*m_eff * 2 * 3 + 3 + 2] = r[j][2];
        
    (*m_eff)++;
    if(*m_eff == bonds->m_max)
        return;
}
;
        }
    }
}

/*---------------------------------------------------------------------------*/
