dnl ---------------------------------------------------------------------------
dnl
dnl  dxwrite.c.m4
dnl
dnl ---------------------------------------------------------------------------
dnl
dnl  Copyright (C) 2007,2018,2019 Takashi Uneyama
dnl
dnl  This program is free software; you can redistribute it and/or modify
dnl  it under the terms of the GNU General Public License as published by
dnl  the Free Software Foundation; either version 2, or (at your option)
dnl  any later version.
dnl
dnl  This program is distributed in the hope that it will be useful,
dnl  but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl  GNU General Public License for more details.
dnl
dnl  You should have received a copy of the GNU General Public License
dnl  along with this program; if not, write to the Free Software
dnl  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
dnl  USA.
dnl
dnl ---------------------------------------------------------------------------
include(`dimension.m4')dnl
include(`scan.m4')dnl
GENERATE_HEADER(dxwrite.c,
                output the position data as OpenDX format.)

#include "common.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include "xmalloc.h"
#include "dxwrite.h"

#define MAX(x,y) (((x) > (y)) ? (x) : (y))

/*---------------------------------------------------------------------------*/
PROTOTYPE_FUNCTIONS(`dnl
static void write_positions@D@_dx_binary(char *filename,
                                         bonded_particles@D@ *particles,
                                         transient_bonds *bonds,
                                         box_geometry *geometry);
static void write_box@D@_dx(FILE *fp,box_geometry *geometry);
static double calculate_bond_scale_factor@D@(vector@D@ r,vector@D@ q,double *l);'`')

PROTOTYPE_FOR_ALL_BONDS(dnl
`count_number_of_bonds_for_particles',dnl
`int *m')

PROTOTYPE_FOR_ALL_BONDS(dnl
`extract_active_bonds_in_box',dnl
`float *r_bonds, int *m_eff')

/*---------------------------------------------------------------------------*/
void write_positions_dx(char *filename,bonded_particles *particles,
                        box_geometry *geometry)
{
#if ((SIZEOF_FLOAT == 4) && (SIZEOF_INT == 4))
    int *m;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

        /*  do not generate data in 1d  */
    if(particles->dimension == 1)
        return;

        /*  IEEE floating number format (binary)  */
    SWITCH_FUNCTION(particles->dimension,
                    write_positions@D@_dx_binary(filename,&(particles->particles@D@),
                                                 &(particles->bonds),geometry);)
#endif
}

/*---------------------------------------------------------------------------*/
EXPAND_FUNCTIONS(`dnl
void write_positions@D@_dx_binary(char *filename,bonded_particles@D@ *particles,
                                  transient_bonds *bonds,box_geometry *geometry)
{
    FILE *fp;
    int i,j;
    int n;
    float *r_particles,*r_bonds;
    int *id,*m,*s_eff;
    int offset,m_eff;
    char *endian;

#ifdef WORDS_BIGENDIAN
    endian = "msb";
#else
    endian = "lsb";
#endif

    n = particles->n;
    r_particles = xmalloc(sizeof(float) * @D@ * n);
    r_bonds = xmalloc(sizeof(float) * @D@ * 2 * bonds->m_max);
    id = xmalloc(sizeof(int) * n);
    m = xmalloc(sizeof(int) * n);
    s_eff = xmalloc(sizeof(int) * 2 * bonds->m_max);


        /*  prepare the binary data  */
    for(i = 0;i < particles->n;i++)
    {
        id[i] = i;
        m[i] = 0;
        VECTOR@D@(r_particles[@D@ * i + @@] = particles->r[i][@@];)
    }
    count_number_of_bonds_for_particles@D@(particles->r,bonds,
                                           geometry,m);

    m_eff = 0;
    extract_active_bonds_in_box@D@(particles->r,bonds,
                                   geometry,r_bonds,&m_eff);
    if(m_eff == 0)
    {
        /*  no bond in this system, add a dummy bond  */
        m_eff = 1;
        VECTOR@D@(r_bonds[@D@ * 0 + @@] = 0.0f;)
        VECTOR@D@(r_bonds[@D@ * 1 + @@] = 0.0f;)
    }
    for(i = 0;i < 2 * m_eff;i++)
    {
        s_eff[i] = i;
    }


    fp = fopen(filename,"wb");


    write_box@D@_dx(fp,geometry);

        /*  write the id, position, and bond data of bonded particles.  */
    fprintf(fp,"object \"particle_id\" class array type int rank 0 items %d %s binary data 0\n",
            particles->n,endian);
    fprintf(fp,"attribute \"dep\" string \"positions\"\n");
    fprintf(fp,"\n");
    offset = sizeof(int) * particles->n;

    fprintf(fp,"object \"particle_bond_number\" class array type int rank 0 items %d %s binary data %d\n",
            particles->n,endian,offset);
    fprintf(fp,"attribute \"dep\" string \"positions\"\n");
    fprintf(fp,"\n");
    offset += sizeof(int) * particles->n;

    fprintf(fp,"object \"particle_positions\" class array type float rank 1 shape @D@ items %d %s ieee data %d\n",
            particles->n,endian,offset);
    fprintf(fp,"attribute \"dep\" string \"positions\"\n");
    fprintf(fp,"\n");
    offset += sizeof(float) * @D@ * particles->n;

    if(m_eff > 0)
    {
        fprintf(fp,"object \"bond_positions\" class array type float rank 1 shape @D@ items %d %s ieee data %d\n",
                2 * m_eff,endian,offset);
        fprintf(fp,"attribute \"dep\" string \"positions\"\n");
        fprintf(fp,"\n");
        offset += sizeof(float) * @D@ * 2 * m_eff;

        fprintf(fp,"object \"bond_connections\" class array type int rank 1 shape 2 items %d %s binary data %d\n",
                m_eff,endian,offset);
        fprintf(fp,"attribute \"element type\" string \"lines\"\n");
        fprintf(fp,"attribute \"dep\" string \"connections\"\n");
        fprintf(fp,"attribute \"ref\" string \"positions\"\n");
        fprintf(fp,"\n");
    }

    fprintf(fp,"object \"particles\" class field\n");
    fprintf(fp,"component \"id\" value \"particle_id\"\n");
    fprintf(fp,"component \"data\" value \"particle_bond_number\"\n");
    fprintf(fp,"component \"positions\" value \"particle_positions\"\n");
    fprintf(fp,"\n");
    if(m_eff > 0)
    {
        fprintf(fp,"object \"bonds\" class field\n");
        fprintf(fp,"component \"positions\" value \"bond_positions\"\n");
        fprintf(fp,"component \"connections\" value \"bond_connections\"\n");
        fprintf(fp,"\n");
    }

    fprintf(fp,"end\n");

        /*  write the binary data  */
    fwrite(id,sizeof(int),n,fp);
    fwrite(m,sizeof(int),n,fp);
    fwrite(r_particles,sizeof(float),@D@ * n,fp);
    if(m_eff > 0)
    {
        fwrite(r_bonds,sizeof(float),@D@ * 2 * m_eff,fp);
        fwrite(s_eff,sizeof(int),2 * m_eff,fp);
    }


    fclose(fp);

    free(r_particles);
    free(r_bonds);
    free(id);
    free(m);
    free(s_eff);
}

/*---------------------------------------------------------------------------*/
void write_box@D@_dx(FILE *fp,box_geometry *geometry)
{
                /*  write the box data  */
    ifelse(@D@,2,
           fprintf(fp,"object \"data_box\" class gridpositions counts 2 2\n");
           fprintf(fp,"origin 0.0 0.0\n");
           fprintf(fp,"delta %f 0.0\n",geometry->l[0]);
           fprintf(fp,"delta 0.0 %f\n",geometry->l[1]);,
           @D@,3,
           fprintf(fp,"object \"data_box\" class gridpositions counts 2 2 2\n");
           fprintf(fp,"origin 0.0 0.0 0.0\n");
           fprintf(fp,"delta %f 0.0 0.0\n",geometry->l[0]);
           fprintf(fp,"delta 0.0 %f 0.0\n",geometry->l[1]);
           fprintf(fp,"delta 0.0 0.0 %f\n",geometry->l[2]);)
    fprintf(fp,"attribute \"dep\" string \"positions\"\n");
    fprintf(fp,"\n");
    fprintf(fp,"object \"box\" class field\n");
    fprintf(fp,"component \"positions\" value \"data_box\"\n");
    fprintf(fp,"\n");

    fprintf(fp,"object \"delta_l\" class array type float rank 0 items 1 data follows\n");
    fprintf(fp,"%f\n",geometry->delta_l);
    fprintf(fp,"\n");
}

/*---------------------------------------------------------------------------*/
double calculate_bond_scale_factor@D@(vector@D@ r,vector@D@ q,double *l)
{
    double c;

    c = 1.0;

    if(q[0] + r[0] < 0.0)
    {
        c = - r[0] / q[0];
    }
    if(q[0] + r[0] >= l[0])
    {
        c = (l[0] - r[0]) / q[0];
    }

    if(q[1] + r[1] < 0.0)
    {
        if(c > - r[1] / q[1])
        {
            c = - r[1] / q[1];
        }
    }
    if(q[1] + r[1] >= l[1])
    {
        if(c > (l[1] - r[1]) / q[1])
        {
            c = (l[1] - r[1]) / q[1];
        }
    }

    ifelse(@D@,3,`dnl
    if(q[2] + r[2] < 0.0)
    {
        if(c > - r[2] / q[2])
        {
            c = - r[2] / q[2];
        }
    }
    if(q[2] + r[2] >= l[2])
    {
        if(c > (l[2] - r[2]) / q[2])
        {
            c = (l[2] - r[2]) / q[2];
        }
    }
    ')

    return c;
}')

/*---------------------------------------------------------------------------*/
FOR_ALL_BONDS(dnl
`count_number_of_bonds_for_particles',dnl
`int *m',dnl
`',dnl
`dnl
m[bonds->s[k][0]]++;
m[bonds->s[k][1]]++;')

/*---------------------------------------------------------------------------*/
FOR_ALL_BONDS(dnl
`extract_active_bonds_in_box',dnl
`float *r_bonds, int *m_eff',dnl
`vector@D@ q;
double c;',dnl
`dnl
if(((dr[0] + r[j][0]) < 0.0) || ((dr[0] + r[j][0]) >= geometry->l[0])
   || ((dr[1] + r[j][1]) < 0.0) || ((dr[1] + r[j][1]) >= geometry->l[1])
  ifelse(@D@,3,`dnl
   || ((dr[2] + r[j][2]) < 0.0) || ((dr[2] + r[j][2]) >= geometry->l[2])')`')
{
    /*  outside, split to two bonds  */
    VECTOR@D@(q[@@] = - dr[@@];)
    c = calculate_bond_scale_factor@D@(r[i],q,geometry->l);
    VECTOR@D@(r_bonds[*m_eff * 2 * @D@ + @@] = r[i][@@];)
    VECTOR@D@(r_bonds[*m_eff * 2 * @D@ + @D@ + @@] = r[i][@@] - c * dr[@@];)
        
    (*m_eff)++;
    if(*m_eff == bonds->m_max)
        return;

    c = calculate_bond_scale_factor@D@(r[j],dr,geometry->l);
    VECTOR@D@(r_bonds[*m_eff * 2 * @D@ + @@] = r[j][@@] + c * dr[@@];)
    VECTOR@D@(r_bonds[*m_eff * 2 * @D@ + @D@ + @@] = r[j][@@];)
        
    (*m_eff)++;
    if(*m_eff == bonds->m_max)
        return;
}
else
{
    /*  inside  */
    VECTOR@D@(r_bonds[*m_eff * 2 * @D@ + @@] = r[i][@@];)
    VECTOR@D@(r_bonds[*m_eff * 2 * @D@ + @D@ + @@] = r[j][@@];)
        
    (*m_eff)++;
    if(*m_eff == bonds->m_max)
        return;
}
;')

/*---------------------------------------------------------------------------*/
