/*---------------------------------------------------------------------------*/

/*!  \file  integrator.c
     \brief  integrate the Langevin equation numerically.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007,2008,2018,2019 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/

/*  DO NOT EDIT THIS FILE. This file is generated from integrator.c.m4.  */

#include "common.h"

#ifdef HAVE_MATH_H
#include <math.h>
#endif

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

#include "vector.h"
#include "particles.h"
#include "force.h"
#include "celllist.h"
#include "integrator.h"

/*---------------------------------------------------------------------------*/
static void integrate_langevin_equation1_srk_first(bonded_particles1 *particles,
                                                     box_geometry *geometry,
                                                     double gamma_dot,
                                                     double dt);
static void integrate_langevin_equation1_srk_second(bonded_particles1 *particles,
                                                      box_geometry *geometry,
                                                      double gamma_dot,
                                                      double dt);

static void integrate_langevin_equation2_srk_first(bonded_particles2 *particles,
                                                     box_geometry *geometry,
                                                     double gamma_dot,
                                                     double dt);
static void integrate_langevin_equation2_srk_second(bonded_particles2 *particles,
                                                      box_geometry *geometry,
                                                      double gamma_dot,
                                                      double dt);

static void integrate_langevin_equation3_srk_first(bonded_particles3 *particles,
                                                     box_geometry *geometry,
                                                     double gamma_dot,
                                                     double dt);
static void integrate_langevin_equation3_srk_second(bonded_particles3 *particles,
                                                      box_geometry *geometry,
                                                      double gamma_dot,
                                                      double dt);

static double calculate_gamma_dot(simulation_condition *condition,box_geometry *geometry);
static void integrate_dynamic_equation_geometry(simulation_condition *condition,
                                                box_geometry *geometry,
                                                double gamma_dot);

/*---------------------------------------------------------------------------*/
void integrate_langevin_equation(simulation_condition *condition,
                                 bonded_particles *particles,
                                 linked_cell_list *celllist,
                                 box_geometry *geometry,rng_state *rng)
{
    double xi,dt,gamma_dot;

    xi = condition->xi;
    dt = condition->dt;
    if(particles->dimension == 1)
        gamma_dot = 0.0;
    else
        gamma_dot = calculate_gamma_dot(condition,geometry);
   
    clear_interaction_force(particles);
    generate_random_force(particles,rng);


        /*  the first step of the stochastic Runge-Kutta method  */
    construct_linked_cell_list(celllist,particles,geometry);
    accumulate_site_force(particles,celllist,geometry,xi);
    accumulate_bond_force(particles,geometry);

        switch(particles->dimension)
    {
        case 1:
            integrate_langevin_equation1_srk_first(&(particles->particles1),geometry,gamma_dot,dt);
            break;

        case 2:
            integrate_langevin_equation2_srk_first(&(particles->particles2),geometry,gamma_dot,dt);
            break;

        case 3:
            integrate_langevin_equation3_srk_first(&(particles->particles3),geometry,gamma_dot,dt);
            break;

        default:
            break;
    }


    if(particles->dimension != 1)
        integrate_dynamic_equation_geometry(condition,geometry,gamma_dot);

    set_particle_positions_star_into_box(particles,geometry);


        /*  the second step of the stochastic Runge-Kutta method  */
    construct_linked_cell_list_star(celllist,particles,geometry);
    accumulate_site_force_star(particles,celllist,geometry,xi);
    accumulate_bond_force_star(particles,geometry);

        switch(particles->dimension)
    {
        case 1:
            integrate_langevin_equation1_srk_second(&(particles->particles1),geometry,gamma_dot,dt);
            break;

        case 2:
            integrate_langevin_equation2_srk_second(&(particles->particles2),geometry,gamma_dot,dt);
            break;

        case 3:
            integrate_langevin_equation3_srk_second(&(particles->particles3),geometry,gamma_dot,dt);
            break;

        default:
            break;
    }


    set_particle_positions_into_box(particles,geometry);
}

/*---------------------------------------------------------------------------*/
double calculate_gamma_dot(simulation_condition *condition,box_geometry *geometry)
{
    double gamma_dot;

    if(condition->omega != 0.0)
    {
            /*  calculate the effective strain rate  */
        gamma_dot = condition->kappa / condition->omega / condition->dt
                * (sin(geometry->theta + condition->omega * condition->dt)
                   - sin(geometry->theta));
    }
    else
    {
        gamma_dot = condition->kappa;
    }

    return gamma_dot;
}

/*---------------------------------------------------------------------------*/
void integrate_dynamic_equation_geometry(simulation_condition *condition,
                                         box_geometry *geometry,
                                         double gamma_dot)
{
    if(condition->kappa != 0.0)
    {
        geometry->delta_l += condition->dt * gamma_dot * geometry->l[1];
        geometry->theta += condition->omega * condition->dt;

        while(geometry->delta_l < 0.0)
        {
            geometry->delta_l += geometry->l[0];
            geometry->c--;
        }
        while(geometry->delta_l >= geometry->l[0])
        {
            geometry->delta_l -= geometry->l[0];
            geometry->c++;
        }

        while(geometry->theta >= 2.0 * M_PI)
            geometry->theta -= 2.0 * M_PI;
    }
}

/*---------------------------------------------------------------------------*/
void integrate_langevin_equation1_srk_first(bonded_particles1 *particles,
                                              box_geometry *geometry,
                                              double gamma_dot,double dt)
{
    int i;

    if(gamma_dot == 0.0)
    {
            /*  no shear flow  */
        for(i = 0;i < particles->n;i++)
        {
            particles->r_star[i][0] = particles->r[i][0]
                      + dt * particles->f[i][0]
                      + sqrt(2.0 * dt) * particles->w[i][0];
        }
    }
    else
    {
            /*  shear flow  */
        for(i = 0;i < particles->n;i++)
        {
            particles->r_star[i][0] = particles->r[i][0]
                      + dt * gamma_dot * particles->r[i][1]
                      + dt * particles->f[i][0]
                      + sqrt(2.0 * dt) * particles->w[i][0];
        }
    }
}

/*---------------------------------------------------------------------------*/
void integrate_langevin_equation1_srk_second(bonded_particles1 *particles,
                                               box_geometry *geometry,
                                               double gamma_dot,double dt)
{
    int i;

    if(gamma_dot == 0.0)
    {
            /*  no shear flow  */
        for(i = 0;i < particles->n;i++)
        {
            particles->r[i][0] +=
                      + 0.5 * dt * (particles->f[i][0] + particles->f_star[i][0])
                      + sqrt(2.0 * dt) * particles->w[i][0];
        }
    }
    else
    {
            /*  shear flow  */
        for(i = 0;i < particles->n;i++)
        {
            particles->r[i][0] +=
                      + dt * gamma_dot * particles->r[i][1]
                      + 0.5 * dt * (particles->f[i][0] + particles->f_star[i][0])
                      + sqrt(2.0 * dt) * particles->w[i][0];
        }
    }
}

/*---------------------------------------------------------------------------*/
void integrate_langevin_equation2_srk_first(bonded_particles2 *particles,
                                              box_geometry *geometry,
                                              double gamma_dot,double dt)
{
    int i;

    if(gamma_dot == 0.0)
    {
            /*  no shear flow  */
        for(i = 0;i < particles->n;i++)
        {
            particles->r_star[i][0] = particles->r[i][0]
                      + dt * particles->f[i][0]
                      + sqrt(2.0 * dt) * particles->w[i][0];
particles->r_star[i][1] = particles->r[i][1]
                      + dt * particles->f[i][1]
                      + sqrt(2.0 * dt) * particles->w[i][1];
        }
    }
    else
    {
            /*  shear flow  */
        for(i = 0;i < particles->n;i++)
        {
            particles->r_star[i][0] = particles->r[i][0]
                      + dt * gamma_dot * particles->r[i][1]
                      + dt * particles->f[i][0]
                      + sqrt(2.0 * dt) * particles->w[i][0];
particles->r_star[i][1] = particles->r[i][1]
                      
                      + dt * particles->f[i][1]
                      + sqrt(2.0 * dt) * particles->w[i][1];
        }
    }
}

/*---------------------------------------------------------------------------*/
void integrate_langevin_equation2_srk_second(bonded_particles2 *particles,
                                               box_geometry *geometry,
                                               double gamma_dot,double dt)
{
    int i;

    if(gamma_dot == 0.0)
    {
            /*  no shear flow  */
        for(i = 0;i < particles->n;i++)
        {
            particles->r[i][0] +=
                      + 0.5 * dt * (particles->f[i][0] + particles->f_star[i][0])
                      + sqrt(2.0 * dt) * particles->w[i][0];
particles->r[i][1] +=
                      + 0.5 * dt * (particles->f[i][1] + particles->f_star[i][1])
                      + sqrt(2.0 * dt) * particles->w[i][1];
        }
    }
    else
    {
            /*  shear flow  */
        for(i = 0;i < particles->n;i++)
        {
            particles->r[i][0] +=
                      + dt * gamma_dot * particles->r[i][1]
                      + 0.5 * dt * (particles->f[i][0] + particles->f_star[i][0])
                      + sqrt(2.0 * dt) * particles->w[i][0];
particles->r[i][1] +=
                      
                      + 0.5 * dt * (particles->f[i][1] + particles->f_star[i][1])
                      + sqrt(2.0 * dt) * particles->w[i][1];
        }
    }
}

/*---------------------------------------------------------------------------*/
void integrate_langevin_equation3_srk_first(bonded_particles3 *particles,
                                              box_geometry *geometry,
                                              double gamma_dot,double dt)
{
    int i;

    if(gamma_dot == 0.0)
    {
            /*  no shear flow  */
        for(i = 0;i < particles->n;i++)
        {
            particles->r_star[i][0] = particles->r[i][0]
                      + dt * particles->f[i][0]
                      + sqrt(2.0 * dt) * particles->w[i][0];
particles->r_star[i][1] = particles->r[i][1]
                      + dt * particles->f[i][1]
                      + sqrt(2.0 * dt) * particles->w[i][1];
particles->r_star[i][2] = particles->r[i][2]
                      + dt * particles->f[i][2]
                      + sqrt(2.0 * dt) * particles->w[i][2];
        }
    }
    else
    {
            /*  shear flow  */
        for(i = 0;i < particles->n;i++)
        {
            particles->r_star[i][0] = particles->r[i][0]
                      + dt * gamma_dot * particles->r[i][1]
                      + dt * particles->f[i][0]
                      + sqrt(2.0 * dt) * particles->w[i][0];
particles->r_star[i][1] = particles->r[i][1]
                      
                      + dt * particles->f[i][1]
                      + sqrt(2.0 * dt) * particles->w[i][1];
particles->r_star[i][2] = particles->r[i][2]
                      
                      + dt * particles->f[i][2]
                      + sqrt(2.0 * dt) * particles->w[i][2];
        }
    }
}

/*---------------------------------------------------------------------------*/
void integrate_langevin_equation3_srk_second(bonded_particles3 *particles,
                                               box_geometry *geometry,
                                               double gamma_dot,double dt)
{
    int i;

    if(gamma_dot == 0.0)
    {
            /*  no shear flow  */
        for(i = 0;i < particles->n;i++)
        {
            particles->r[i][0] +=
                      + 0.5 * dt * (particles->f[i][0] + particles->f_star[i][0])
                      + sqrt(2.0 * dt) * particles->w[i][0];
particles->r[i][1] +=
                      + 0.5 * dt * (particles->f[i][1] + particles->f_star[i][1])
                      + sqrt(2.0 * dt) * particles->w[i][1];
particles->r[i][2] +=
                      + 0.5 * dt * (particles->f[i][2] + particles->f_star[i][2])
                      + sqrt(2.0 * dt) * particles->w[i][2];
        }
    }
    else
    {
            /*  shear flow  */
        for(i = 0;i < particles->n;i++)
        {
            particles->r[i][0] +=
                      + dt * gamma_dot * particles->r[i][1]
                      + 0.5 * dt * (particles->f[i][0] + particles->f_star[i][0])
                      + sqrt(2.0 * dt) * particles->w[i][0];
particles->r[i][1] +=
                      
                      + 0.5 * dt * (particles->f[i][1] + particles->f_star[i][1])
                      + sqrt(2.0 * dt) * particles->w[i][1];
particles->r[i][2] +=
                      
                      + 0.5 * dt * (particles->f[i][2] + particles->f_star[i][2])
                      + sqrt(2.0 * dt) * particles->w[i][2];
        }
    }
}

/*---------------------------------------------------------------------------*/
