dnl ---------------------------------------------------------------------------
dnl
dnl  integrator.c.m4
dnl
dnl ---------------------------------------------------------------------------
dnl
dnl  Copyright (C) 2007,2019 Takashi Uneyama
dnl
dnl  This program is free software; you can redistribute it and/or modify
dnl  it under the terms of the GNU General Public License as published by
dnl  the Free Software Foundation; either version 2, or (at your option)
dnl  any later version.
dnl
dnl  This program is distributed in the hope that it will be useful,
dnl  but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl  GNU General Public License for more details.
dnl
dnl  You should have received a copy of the GNU General Public License
dnl  along with this program; if not, write to the Free Software
dnl  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
dnl  USA.
dnl
dnl ---------------------------------------------------------------------------
include(`dimension.m4')`'dnl
GENERATE_HEADER(integrator.c,
                integrate the Langevin equation numerically.)

#include "common.h"

#ifdef HAVE_MATH_H
#include <math.h>
#endif

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

#include "vector.h"
#include "particles.h"
#include "force.h"
#include "celllist.h"
#include "integrator.h"

/*---------------------------------------------------------------------------*/
PROTOTYPE_FUNCTIONS(`dnl
static void integrate_langevin_equation@D@_srk_first(bonded_particles@D@ *particles,
                                                     box_geometry *geometry,
                                                     double gamma_dot,
                                                     double dt);
static void integrate_langevin_equation@D@_srk_second(bonded_particles@D@ *particles,
                                                      box_geometry *geometry,
                                                      double gamma_dot,
                                                      double dt);'`')

static double calculate_gamma_dot(simulation_condition *condition,box_geometry *geometry);
static void integrate_dynamic_equation_geometry(simulation_condition *condition,
                                                box_geometry *geometry,
                                                double gamma_dot);

/*---------------------------------------------------------------------------*/
void integrate_langevin_equation(simulation_condition *condition,
                                 bonded_particles *particles,
                                 linked_cell_list *celllist,
                                 box_geometry *geometry,rng_state *rng)
{
    double xi,dt,gamma_dot;

    xi = condition->xi;
    dt = condition->dt;
    if(particles->dimension == 1)
        gamma_dot = 0.0;
    else
        gamma_dot = calculate_gamma_dot(condition,geometry);
   
    clear_interaction_force(particles);
    generate_random_force(particles,rng);


        /*  the first step of the stochastic Runge-Kutta method  */
    construct_linked_cell_list(celllist,particles,geometry);
    accumulate_site_force(particles,celllist,geometry,xi);
    accumulate_bond_force(particles,geometry);

    SWITCH_FUNCTION(particles->dimension,
                    integrate_langevin_equation@D@_srk_first(&(particles->particles@D@),geometry,gamma_dot,dt);)


    if(particles->dimension != 1)
        integrate_dynamic_equation_geometry(condition,geometry,gamma_dot);

    set_particle_positions_star_into_box(particles,geometry);


        /*  the second step of the stochastic Runge-Kutta method  */
    construct_linked_cell_list_star(celllist,particles,geometry);
    accumulate_site_force_star(particles,celllist,geometry,xi);
    accumulate_bond_force_star(particles,geometry);

    SWITCH_FUNCTION(particles->dimension,
                    integrate_langevin_equation@D@_srk_second(&(particles->particles@D@),geometry,gamma_dot,dt);)


    set_particle_positions_into_box(particles,geometry);
}

/*---------------------------------------------------------------------------*/
double calculate_gamma_dot(simulation_condition *condition,box_geometry *geometry)
{
    double gamma_dot;

    if(condition->omega != 0.0)
    {
            /*  calculate the effective strain rate  */
        gamma_dot = condition->kappa / condition->omega / condition->dt
                * (sin(geometry->theta + condition->omega * condition->dt)
                   - sin(geometry->theta));
    }
    else
    {
        gamma_dot = condition->kappa;
    }

    return gamma_dot;
}

/*---------------------------------------------------------------------------*/
void integrate_dynamic_equation_geometry(simulation_condition *condition,
                                         box_geometry *geometry,
                                         double gamma_dot)
{
    if(condition->kappa != 0.0)
    {
        geometry->delta_l += condition->dt * gamma_dot * geometry->l[1];
        geometry->theta += condition->omega * condition->dt;

        while(geometry->delta_l < 0.0)
        {
            geometry->delta_l += geometry->l[0];
            geometry->c--;
        }
        while(geometry->delta_l >= geometry->l[0])
        {
            geometry->delta_l -= geometry->l[0];
            geometry->c++;
        }

        while(geometry->theta >= 2.0 * M_PI)
            geometry->theta -= 2.0 * M_PI;
    }
}

/*---------------------------------------------------------------------------*/
EXPAND_FUNCTIONS(`dnl
void integrate_langevin_equation@D@_srk_first(bonded_particles@D@ *particles,
                                              box_geometry *geometry,
                                              double gamma_dot,double dt)
{
    int i;

    if(gamma_dot == 0.0)
    {
            /*  no shear flow  */
        for(i = 0;i < particles->n;i++)
        {
            VECTOR@D@(particles->r_star[i][@@] = particles->r[i][@@]
                      + dt * particles->f[i][@@]
                      + sqrt(2.0 * dt) * particles->w[i][@@];)
        }
    }
    else
    {
            /*  shear flow  */
        for(i = 0;i < particles->n;i++)
        {
            VECTOR@D@(particles->r_star[i][@@] = particles->r[i][@@]
                      `ifelse(@@,0,+ dt * gamma_dot * particles->r[i][1])'`'
                      + dt * particles->f[i][@@]
                      + sqrt(2.0 * dt) * particles->w[i][@@];)
        }
    }
}

/*---------------------------------------------------------------------------*/
void integrate_langevin_equation@D@_srk_second(bonded_particles@D@ *particles,
                                               box_geometry *geometry,
                                               double gamma_dot,double dt)
{
    int i;

    if(gamma_dot == 0.0)
    {
            /*  no shear flow  */
        for(i = 0;i < particles->n;i++)
        {
            VECTOR@D@(particles->r[i][@@] +=
                      + 0.5 * dt * (particles->f[i][@@] + particles->f_star[i][@@])
                      + sqrt(2.0 * dt) * particles->w[i][@@];)
        }
    }
    else
    {
            /*  shear flow  */
        for(i = 0;i < particles->n;i++)
        {
            VECTOR@D@(particles->r[i][@@] +=
                      `ifelse(@@,0,+ dt * gamma_dot * particles->r[i][1])'`'
                      + 0.5 * dt * (particles->f[i][@@] + particles->f_star[i][@@])
                      + sqrt(2.0 * dt) * particles->w[i][@@];)
        }
    }
}'`')

/*---------------------------------------------------------------------------*/
