/*---------------------------------------------------------------------------*/

/*!  \file  load.c
     \brief  load the state from a HDF5 file  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2009-2011,2018,2019 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#include "load.h"
#include <stdio.h>
#include <stdlib.h>

/*---------------------------------------------------------------------------*/
#ifdef HAVE_HDF5
#include <hdf5.h>
#include <hdf5_hl.h>

/*---------------------------------------------------------------------------*/
static rng_state *load_rng_from_hdf5(hid_t file);
static void load_box_geometry_from_hdf5(hid_t file,box_geometry *geometry);
static bonded_particles *load_bonded_particles_from_hdf5(hid_t file,
                                                         simulation_condition *condition,
                                                         box_geometry *geometry);
static void read_dataset_unsignedlong(hid_t location,char *path,
                                      unsigned long *dest);

/*---------------------------------------------------------------------------*/
rng_state *load_rng_from_hdf5(hid_t file)
{
    hid_t group1,group2;
    int next_offset;
    rng_state *rng;

    rng = allocate_rng_state();

    group1 = H5Gopen2(file,"/rng_state",H5P_DEFAULT);
    group2 = H5Gopen2(file,"/rng_state/mt_state",H5P_DEFAULT);

    read_dataset_unsignedlong(file,"/rng_state/mt_state/state",rng->mt->state);

    H5LTread_dataset_int(file,"/rng_state/mt_state/n",&(rng->mt->n));
    H5LTread_dataset_int(file,"/rng_state/mt_state/left",&(rng->mt->left));
    H5LTread_dataset_int(file,"/rng_state/mt_state/initf",&(rng->mt->initf));
    H5LTread_dataset_int(file,"/rng_state/mt_state/next",&next_offset);
    rng->mt->next = rng->mt->state + next_offset;

    H5LTread_dataset_int(file,"/rng_state/nrand_has_saved_number",
                         &(rng->nrand_has_saved_number));
    H5LTread_dataset_double(file,"/rng_state/nrand_w",&(rng->nrand_w));

    H5Gclose(group2);
    H5Gclose(group1);

    return rng;
}

/*---------------------------------------------------------------------------*/
void load_box_geometry_from_hdf5(hid_t file,box_geometry *geometry)
{
    hid_t group;
    int dimension;

    group = H5Gopen2(file,"/box_geometry",H5P_DEFAULT);

    H5LTread_dataset_int(file,"/box_geometry/dimension",
                         &dimension);
    if(geometry->dimension != dimension)
    {
        fprintf(stderr,"error: the dimension of box geometry is not consistent (%d and %d)\n",
                geometry->dimension,dimension);
        exit(EXIT_FAILURE);
    }
    H5LTread_dataset_double(file,"/box_geometry/theta",
                            &(geometry->theta));
    H5LTread_dataset_double(file,"/box_geometry/delta_l",
                            &(geometry->delta_l));
    H5LTread_dataset_int(file,"/box_geometry/c",
                         &(geometry->c));

    H5LTread_dataset_double(file,"/box_geometry/l",geometry->l);

    H5Gclose(group);
}

/*---------------------------------------------------------------------------*/
bonded_particles *load_bonded_particles_from_hdf5(hid_t file,
                                                  simulation_condition *condition,
                                                  box_geometry *geometry)
{
    hid_t group;
    int i,n,m;
    bonded_particles *particles;
    double *r;
    int *c;

    particles = allocate_bonded_particles(condition->n,
                                          estimate_maximum_number_of_bonds(condition,geometry),
                                          geometry);

    group = H5Gopen2(file,"/bonded_particles",H5P_DEFAULT);

    H5LTread_dataset_int(file,"/bonded_particles/n",
                         &n);
    H5LTread_dataset_int(file,"/bonded_particles/m",
                         &m);

    if(n != condition->n)
    {
        fprintf(stderr,"error: the number of particles is not consistent (%d and %d)\n",
                condition->n,n);
        exit(EXIT_FAILURE);
    }
    if(m > particles->bonds.m_max)
    {
        fprintf(stderr,"error: the number of bonds is too large (%d < %d)\n",
                particles->bonds.m_max,m);
        exit(EXIT_FAILURE);
    }

    switch(particles->dimension)
    {
        case 1:
            r = (double *)(particles->particles1.r);
            c = (int *)(particles->particles1.c);

        case 2:
            r = (double *)(particles->particles2.r);
            c = (int *)(particles->particles2.c);
            break;

        case 3:
            r = (double *)(particles->particles3.r);
            c = (int *)(particles->particles3.c);
            break;

        default:
            break;
    }


    H5LTread_dataset_double(file,"/bonded_particles/r",
                            r);
    H5LTread_dataset_int(file,"/bonded_particles/c",
                         c);

    H5LTread_dataset_int(file,"/bonded_particles/s",
                         (int *)(particles->bonds.s));

    if(m == particles->bonds.m_max)
    {
        particles->bonds.head = INACTIVE_BOND;
    }
    else
    {
        particles->bonds.head = m;
        for(i = m;i < particles->bonds.m_max;i++)
        {
            particles->bonds.s[i][0] = INACTIVE_BOND;
            particles->bonds.s[i][1] = i + 1;
        }
        particles->bonds.s[particles->bonds.m_max - 1][1] = INACTIVE_BOND;
    }


    H5Gclose(group);
    
    return particles;
}

/*---------------------------------------------------------------------------*/
void read_dataset_unsignedlong(hid_t location,char *path,unsigned long *dest)
{
    hid_t data;

    data = H5Dopen2(location,path,H5P_DEFAULT);

    H5Dread(data,H5T_NATIVE_ULONG,H5S_ALL,H5S_ALL,H5P_DEFAULT,dest);

    H5Dclose(data);
}

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
void h5_load_state_from_file(char *h5filename,
                             simulation_condition *condition,
                             box_geometry *geometry,
                             bonded_particles **particles,
                             rng_state **rng)
{
#ifdef HAVE_HDF5
    hid_t file;

    file = H5Fopen(h5filename,H5F_ACC_RDONLY,H5P_DEFAULT);
    if(file < 0)
    {
        fprintf(stderr,"error: cannot open the input HDF5 file `%s'.\n",
                h5filename);
        exit(EXIT_FAILURE);
    }

    load_box_geometry_from_hdf5(file,geometry);
    *rng = load_rng_from_hdf5(file);
    *particles =load_bonded_particles_from_hdf5(file,condition,geometry);

    H5Fclose(file);
#else
    return;
#endif
}

/*---------------------------------------------------------------------------*/
