/*---------------------------------------------------------------------------*/

/*!  \file  luaread.c
     \brief  read input Lua file for the simulator  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007,2008,2018 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifdef HAVE_LUA_H
#include <lua.h>
#endif
#ifdef HAVE_LUALIB_H
#include <lualib.h>
#endif
#ifdef HAVE_LAUXLIB_H
#include <lauxlib.h>
#endif

#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif

#include "xmalloc.h"
#include "luautil.h"
#include "luaread.h"

#include "default.c"

/*---------------------------------------------------------------------------*/
static simulation_condition *read_simulation_condition_lua(lua_State *L);
static data_files *read_data_files_lua(lua_State *L,
                                       simulation_condition *condition);
static box_geometry *read_box_geometry_lua(lua_State *L);

/*---------------------------------------------------------------------------*/
void read_input_lua(char *input_lua,simulation_condition **condition,
                    data_files **files,box_geometry **geometry)
{
    lua_State *L;

    L = luaL_newstate();
    luaL_openlibs(L);
        /*  load the default parameters  */
    luaL_loadstring(L,default_lua);
    lua_pcall(L,0,LUA_MULTRET,0);

    /* L = luau_open_file(input_lua); */
    if(luaL_loadfile(L,input_lua) || lua_pcall(L,0,LUA_MULTRET,0))
    {
        fprintf(stderr,"error: %s\n",lua_tostring(L,-1));
        exit(EXIT_FAILURE);
    }

    *condition = read_simulation_condition_lua(L);
    *files = read_data_files_lua(L,*condition);
    *geometry = read_box_geometry_lua(L);

    lua_close(L);
}

/*---------------------------------------------------------------------------*/
simulation_condition *read_simulation_condition_lua(lua_State *L)
{
    simulation_condition *condition;

    condition = allocate_simulation_condition();

    /* luau_push_table_or_exit(L,"condition"); */


    luau_get_bool_or_exit(L,"save_position_sequential",
                          &condition->save_position_sequential);
    luau_get_bool_or_exit(L,"save_bond_sequential",
                          &condition->save_bond_sequential);
    luau_get_bool_or_exit(L,"save_stress_sequential",
                          &condition->save_stress_sequential);
    luau_get_bool_or_exit(L,"save_energy_sequential",
                          &condition->save_energy_sequential);
    luau_get_bool_or_exit(L,"save_geometry_sequential",
                          &condition->save_geometry_sequential);
    luau_get_bool_or_exit(L,"save_dx_sequential",
                          &condition->save_dx_sequential);
    luau_get_bool_or_exit(L,"apply_shear",&condition->apply_shear);
    luau_get_int_or_exit(L,"iteration_step",&condition->iteration_step);

    luau_get_unsigned_long_or_exit(L,"seed",&condition->seed);
    luau_get_int_or_exit(L,"iteration_max",&condition->iteration_max);
    luau_get_int_or_exit(L,"interval",&condition->interval);

    luau_get_int_or_exit(L,"n",&condition->n);
    luau_get_double_or_exit(L,"dt",&condition->dt);
    luau_get_double_or_exit(L,"xi",&condition->xi);
    luau_get_double_or_exit(L,"tau",&condition->tau);
    if(condition->apply_shear)
    {
        luau_get_double_or_exit(L,"kappa",&condition->kappa);
        luau_get_double_or_exit(L,"omega",&condition->omega);
    }
    else
    {
        condition->kappa = 0.0;
        condition->omega = 0.0;
    }


    /* luau_pop_table(L); */

    return condition;
}

/*---------------------------------------------------------------------------*/
data_files *read_data_files_lua(lua_State *L,simulation_condition *condition)
{
    data_files *files;

    files = allocate_data_files();

    /* luau_push_table_or_exit(L,"file"); */


    if(condition->save_position_sequential)
        luau_get_string_or_exit(L,"position_template",&files->position_template);
    else
        luau_get_string_or_exit(L,"position_output",&files->position_output);
    if(condition->save_bond_sequential)
        luau_get_string_or_exit(L,"bond_template",&files->bond_template);
    else
        luau_get_string_or_exit(L,"bond_output",&files->bond_output);
    luau_get_string_or_exit(L,"stress_output",&files->stress_output);
    luau_get_string_or_exit(L,"energy_output",&files->energy_output);
    luau_get_string_or_exit(L,"geometry_output",&files->geometry_output);
    luau_get_string_or_exit(L,"h5_output",&files->h5_output);
    if(condition->save_dx_sequential)
        luau_get_string_or_exit(L,"dx_template",&files->dx_template);
    else
        luau_get_string_or_exit(L,"dx_output",&files->dx_output);


    /* luau_pop_table_or_exit(L); */

    return files;
}

/*---------------------------------------------------------------------------*/
box_geometry *read_box_geometry_lua(lua_State *L)
{
    box_geometry *geometry;
    double *l;
    int n;

    geometry = allocate_box_geometry();

    /* luau_push_table_or_exit(L,"geometry"); */


    luau_get_int_or_exit(L,"dimension",&geometry->dimension);

    n = luau_get_double_array_or_exit(L,"l",&l);
    if(n < geometry->dimension)
    {
        fprintf(stderr,
                "`l' has only %d values while the dimension is %d.",
                n,geometry->dimension);
        exit(EXIT_FAILURE);
    }
    geometry->l[0] = l[0];
    geometry->l[1] = ((geometry->dimension >= 2) ? (l[1]) : (1.0));
    geometry->l[2] = ((geometry->dimension >= 3) ? (l[2]) : (1.0));

    luau_free_double_array(l);

    /* luau_pop_table_or_exit(L); */

    return geometry;
}

/*---------------------------------------------------------------------------*/
