/*---------------------------------------------------------------------------*/

/*!  \file  particles.c
     \brief  the particles and transient bonds.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007,2008,2018,2019 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/

/*  DO NOT EDIT THIS FILE. This file is generated from particles.c.m4.  */

#include "common.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_MATH_H
#include <math.h>
#endif

#include "xmalloc.h"
#include "vector.h"
#include "rng.h"
#include "particles.h"

#define BOND_NUMBER_WIDTH_FACTOR (10.0)
#define MAXIMUM_BOND_NUMBER_LOWER_LIMIT (0x100)

/*---------------------------------------------------------------------------*/
static void allocate_bonded_particles1(int n,bonded_particles1 *particles);
static void free_bonded_particles1(bonded_particles1 *particles);
static void set_particle_positions1_random(bonded_particles1 *particles,
                                             box_geometry *geometry,
                                             rng_state *rng);
static void set_particle_positions1_into_box(bonded_particles1 *particles,
                                               box_geometry *geometry);
static void set_particle_positions1_star_into_box(bonded_particles1 *particles,
                                                    box_geometry *geometry);

static void allocate_bonded_particles2(int n,bonded_particles2 *particles);
static void free_bonded_particles2(bonded_particles2 *particles);
static void set_particle_positions2_random(bonded_particles2 *particles,
                                             box_geometry *geometry,
                                             rng_state *rng);
static void set_particle_positions2_into_box(bonded_particles2 *particles,
                                               box_geometry *geometry);
static void set_particle_positions2_star_into_box(bonded_particles2 *particles,
                                                    box_geometry *geometry);

static void allocate_bonded_particles3(int n,bonded_particles3 *particles);
static void free_bonded_particles3(bonded_particles3 *particles);
static void set_particle_positions3_random(bonded_particles3 *particles,
                                             box_geometry *geometry,
                                             rng_state *rng);
static void set_particle_positions3_into_box(bonded_particles3 *particles,
                                               box_geometry *geometry);
static void set_particle_positions3_star_into_box(bonded_particles3 *particles,
                                                    box_geometry *geometry);

/*---------------------------------------------------------------------------*/
bonded_particles *allocate_bonded_particles(int n,int m_max,
                                            box_geometry *geometry)
{
    bonded_particles *particles;
    int i;

    particles = xmalloc(sizeof(bonded_particles));
    particles->dimension = geometry->dimension;

        switch(geometry->dimension)
    {
        case 1:
            allocate_bonded_particles1(n,&(particles->particles1));
            break;

        case 2:
            allocate_bonded_particles2(n,&(particles->particles2));
            break;

        case 3:
            allocate_bonded_particles3(n,&(particles->particles3));
            break;

        default:
            free(particles); particles = NULL;
            break;
    }

    particles->bonds.m_max = m_max;
    particles->bonds.head = 0;
    particles->bonds.s = xmalloc(sizeof(index2) * m_max);

    for(i = 0;i < m_max;i++)
    {
        particles->bonds.s[i][0] = INACTIVE_BOND;
        particles->bonds.s[i][1] = i + 1;
    }
    particles->bonds.s[m_max - 1][1] = INACTIVE_BOND;

    return particles;
}

/*---------------------------------------------------------------------------*/
void free_bonded_particles(bonded_particles *particles)
{
    if(particles == NULL)
        return;

        switch(particles->dimension)
    {
        case 1:
            free_bonded_particles1(&(particles->particles1));
            break;

        case 2:
            free_bonded_particles2(&(particles->particles2));
            break;

        case 3:
            free_bonded_particles3(&(particles->particles3));
            break;

        default:
            break;
    }

    free(particles->bonds.s);
}

/*---------------------------------------------------------------------------*/
int estimate_maximum_number_of_bonds(simulation_condition *condition,
                                     box_geometry *geometry)
{
    int n,m_max;
    double m_av;

    n = condition->n;
    m_av = 0.5 * n * (n - 1) * condition->xi / calculate_volume_of_box(geometry);
    switch(geometry->dimension)
    {
        case 1:
            m_av *= sqrt(2.0 * M_PI);
            break;

        case 2:
            m_av *= M_PI;
            break;

        case 3:
            m_av *= (2.0 * M_PI / 3.0) * sqrt(2.0 * M_PI / 3.0);
            break;

        default:
            m_av = 0.0;
            break;
    }

    m_max = (int)(m_av + BOND_NUMBER_WIDTH_FACTOR * sqrt(m_av));
    if(m_max < MAXIMUM_BOND_NUMBER_LOWER_LIMIT)
    {
        m_max = MAXIMUM_BOND_NUMBER_LOWER_LIMIT;
    }

    return m_max;
}

/*---------------------------------------------------------------------------*/
void set_particle_positions_random(bonded_particles *particles,
                                   box_geometry *geometry,rng_state *rng)
{
        switch(particles->dimension)
    {
        case 1:
            set_particle_positions1_random(&(particles->particles1),geometry,rng);
            break;

        case 2:
            set_particle_positions2_random(&(particles->particles2),geometry,rng);
            break;

        case 3:
            set_particle_positions3_random(&(particles->particles3),geometry,rng);
            break;

        default:
            break;
    }
}

/*---------------------------------------------------------------------------*/
void set_particle_positions_into_box(bonded_particles *particles,
                                     box_geometry *geometry)
{
        switch(particles->dimension)
    {
        case 1:
            set_particle_positions1_into_box(&(particles->particles1),geometry);
            break;

        case 2:
            set_particle_positions2_into_box(&(particles->particles2),geometry);
            break;

        case 3:
            set_particle_positions3_into_box(&(particles->particles3),geometry);
            break;

        default:
            break;
    }
}

/*---------------------------------------------------------------------------*/
void set_particle_positions_star_into_box(bonded_particles *particles,
                                          box_geometry *geometry)
{
        switch(particles->dimension)
    {
        case 1:
            set_particle_positions1_star_into_box(&(particles->particles1),geometry);
            break;

        case 2:
            set_particle_positions2_star_into_box(&(particles->particles2),geometry);
            break;

        case 3:
            set_particle_positions3_star_into_box(&(particles->particles3),geometry);
            break;

        default:
            break;
    }
}

/*---------------------------------------------------------------------------*/
void allocate_bonded_particles1(int n,bonded_particles1 *particles)
{
    particles->n = n;
    particles->r = xmalloc(sizeof(vector1) * n);
    particles->f = xmalloc(sizeof(vector1) * n);
    particles->w = xmalloc(sizeof(vector1) * n);
    particles->r_star = xmalloc(sizeof(vector1) * n);
    particles->f_star = xmalloc(sizeof(vector1) * n);
    particles->c = xmalloc(sizeof(index1) * n);
}

/*---------------------------------------------------------------------------*/
void free_bonded_particles1(bonded_particles1 *particles)
{
    free(particles->r);
    free(particles->f);
    free(particles->w);
    free(particles->r_star);
    free(particles->f_star);
    free(particles->c);
}

/*---------------------------------------------------------------------------*/
void set_particle_positions1_random(bonded_particles1 *particles,
                                      box_geometry *geometry,rng_state *rng)
{
    int i;

    for(i = 0;i < particles->n;i++)
    {
        particles->r[i][0] = geometry->l[0] * urand(rng);
        particles->c[i][0] = 0;
    }
}

/*---------------------------------------------------------------------------*/


void set_particle_positions1_star_into_box(bonded_particles1 *particles,
                                        box_geometry *geometry)
{
    int i;
    vector1 *r;
    

    r = particles->r_star;
    

/*  no box deformation in 1d  */
    for(i = 0;i < particles->n;i++)
    {
        while(r[i][0] < 0.0)
        {
            r[i][0] += geometry->l[0];
            
        }
        while(r[i][0] >= geometry->l[0])
        {
            r[i][0] -= geometry->l[0];
            
        }
    }
}

/*---------------------------------------------------------------------------*/

void set_particle_positions1_into_box(bonded_particles1 *particles,
                                        box_geometry *geometry)
{
    int i;
    vector1 *r;
    index1 *c;

    r = particles->r;
    c = particles->c;

/*  no box deformation in 1d  */
    for(i = 0;i < particles->n;i++)
    {
        while(r[i][0] < 0.0)
        {
            r[i][0] += geometry->l[0];
            c[i][0]--;
        }
        while(r[i][0] >= geometry->l[0])
        {
            r[i][0] -= geometry->l[0];
            c[i][0]++;
        }
    }
}


/*---------------------------------------------------------------------------*/
void allocate_bonded_particles2(int n,bonded_particles2 *particles)
{
    particles->n = n;
    particles->r = xmalloc(sizeof(vector2) * n);
    particles->f = xmalloc(sizeof(vector2) * n);
    particles->w = xmalloc(sizeof(vector2) * n);
    particles->r_star = xmalloc(sizeof(vector2) * n);
    particles->f_star = xmalloc(sizeof(vector2) * n);
    particles->c = xmalloc(sizeof(index2) * n);
}

/*---------------------------------------------------------------------------*/
void free_bonded_particles2(bonded_particles2 *particles)
{
    free(particles->r);
    free(particles->f);
    free(particles->w);
    free(particles->r_star);
    free(particles->f_star);
    free(particles->c);
}

/*---------------------------------------------------------------------------*/
void set_particle_positions2_random(bonded_particles2 *particles,
                                      box_geometry *geometry,rng_state *rng)
{
    int i;

    for(i = 0;i < particles->n;i++)
    {
        particles->r[i][0] = geometry->l[0] * urand(rng);
particles->r[i][1] = geometry->l[1] * urand(rng);
        particles->c[i][0] = 0;
particles->c[i][1] = 0;
    }
}

/*---------------------------------------------------------------------------*/


void set_particle_positions2_star_into_box(bonded_particles2 *particles,
                                        box_geometry *geometry)
{
    int i;
    vector2 *r;
    

    r = particles->r_star;
    

if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(i = 0;i < particles->n;i++)
        {
            while(r[i][0] < 0.0)
                      {
                          r[i][0] += geometry->l[0];
                          
                      }
                      while(r[i][0] >= geometry->l[0])
                      {
                          r[i][0] -= geometry->l[0];
                          
                      }
while(r[i][1] < 0.0)
                      {
                          r[i][1] += geometry->l[1];
                          
                      }
                      while(r[i][1] >= geometry->l[1])
                      {
                          r[i][1] -= geometry->l[1];
                          
                      }
        }
    }
    else
    {
            /*  box deformation  */
        for(i = 0;i < particles->n;i++)
        {
            while(r[i][1] < 0.0)
            {
                r[i][1] += geometry->l[1];
                r[i][0] += geometry->delta_l;
                
            }
            while(r[i][1] >= geometry->l[1])
            {
                r[i][1] -= geometry->l[1];
                r[i][0] -= geometry->delta_l;
                
            }
            while(r[i][0] < 0.0)
            {
                r[i][0] += geometry->l[0];
                
            }
            while(r[i][0] >= geometry->l[0])
            {
                r[i][0] -= geometry->l[0];
                
            }
            
        }
    }

}

/*---------------------------------------------------------------------------*/

void set_particle_positions2_into_box(bonded_particles2 *particles,
                                        box_geometry *geometry)
{
    int i;
    vector2 *r;
    index2 *c;

    r = particles->r;
    c = particles->c;

if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(i = 0;i < particles->n;i++)
        {
            while(r[i][0] < 0.0)
                      {
                          r[i][0] += geometry->l[0];
                          c[i][0]--;
                      }
                      while(r[i][0] >= geometry->l[0])
                      {
                          r[i][0] -= geometry->l[0];
                          c[i][0]++;
                      }
while(r[i][1] < 0.0)
                      {
                          r[i][1] += geometry->l[1];
                          c[i][1]--;
                      }
                      while(r[i][1] >= geometry->l[1])
                      {
                          r[i][1] -= geometry->l[1];
                          c[i][1]++;
                      }
        }
    }
    else
    {
            /*  box deformation  */
        for(i = 0;i < particles->n;i++)
        {
            while(r[i][1] < 0.0)
            {
                r[i][1] += geometry->l[1];
                r[i][0] += geometry->delta_l;
                c[i][1]--;
            }
            while(r[i][1] >= geometry->l[1])
            {
                r[i][1] -= geometry->l[1];
                r[i][0] -= geometry->delta_l;
                c[i][1]++;
            }
            while(r[i][0] < 0.0)
            {
                r[i][0] += geometry->l[0];
                c[i][0]--;
            }
            while(r[i][0] >= geometry->l[0])
            {
                r[i][0] -= geometry->l[0];
                c[i][0]++;
            }
            
        }
    }

}


/*---------------------------------------------------------------------------*/
void allocate_bonded_particles3(int n,bonded_particles3 *particles)
{
    particles->n = n;
    particles->r = xmalloc(sizeof(vector3) * n);
    particles->f = xmalloc(sizeof(vector3) * n);
    particles->w = xmalloc(sizeof(vector3) * n);
    particles->r_star = xmalloc(sizeof(vector3) * n);
    particles->f_star = xmalloc(sizeof(vector3) * n);
    particles->c = xmalloc(sizeof(index3) * n);
}

/*---------------------------------------------------------------------------*/
void free_bonded_particles3(bonded_particles3 *particles)
{
    free(particles->r);
    free(particles->f);
    free(particles->w);
    free(particles->r_star);
    free(particles->f_star);
    free(particles->c);
}

/*---------------------------------------------------------------------------*/
void set_particle_positions3_random(bonded_particles3 *particles,
                                      box_geometry *geometry,rng_state *rng)
{
    int i;

    for(i = 0;i < particles->n;i++)
    {
        particles->r[i][0] = geometry->l[0] * urand(rng);
particles->r[i][1] = geometry->l[1] * urand(rng);
particles->r[i][2] = geometry->l[2] * urand(rng);
        particles->c[i][0] = 0;
particles->c[i][1] = 0;
particles->c[i][2] = 0;
    }
}

/*---------------------------------------------------------------------------*/


void set_particle_positions3_star_into_box(bonded_particles3 *particles,
                                        box_geometry *geometry)
{
    int i;
    vector3 *r;
    

    r = particles->r_star;
    

if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(i = 0;i < particles->n;i++)
        {
            while(r[i][0] < 0.0)
                      {
                          r[i][0] += geometry->l[0];
                          
                      }
                      while(r[i][0] >= geometry->l[0])
                      {
                          r[i][0] -= geometry->l[0];
                          
                      }
while(r[i][1] < 0.0)
                      {
                          r[i][1] += geometry->l[1];
                          
                      }
                      while(r[i][1] >= geometry->l[1])
                      {
                          r[i][1] -= geometry->l[1];
                          
                      }
while(r[i][2] < 0.0)
                      {
                          r[i][2] += geometry->l[2];
                          
                      }
                      while(r[i][2] >= geometry->l[2])
                      {
                          r[i][2] -= geometry->l[2];
                          
                      }
        }
    }
    else
    {
            /*  box deformation  */
        for(i = 0;i < particles->n;i++)
        {
            while(r[i][1] < 0.0)
            {
                r[i][1] += geometry->l[1];
                r[i][0] += geometry->delta_l;
                
            }
            while(r[i][1] >= geometry->l[1])
            {
                r[i][1] -= geometry->l[1];
                r[i][0] -= geometry->delta_l;
                
            }
            while(r[i][0] < 0.0)
            {
                r[i][0] += geometry->l[0];
                
            }
            while(r[i][0] >= geometry->l[0])
            {
                r[i][0] -= geometry->l[0];
                
            }
            while(r[i][2] < 0.0)
                   {
                       r[i][2] += geometry->l[2];
                       
                   }
                   while(r[i][2] >= geometry->l[2])
                   {
                       r[i][2] -= geometry->l[2];
                       
                   }
        }
    }

}

/*---------------------------------------------------------------------------*/

void set_particle_positions3_into_box(bonded_particles3 *particles,
                                        box_geometry *geometry)
{
    int i;
    vector3 *r;
    index3 *c;

    r = particles->r;
    c = particles->c;

if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(i = 0;i < particles->n;i++)
        {
            while(r[i][0] < 0.0)
                      {
                          r[i][0] += geometry->l[0];
                          c[i][0]--;
                      }
                      while(r[i][0] >= geometry->l[0])
                      {
                          r[i][0] -= geometry->l[0];
                          c[i][0]++;
                      }
while(r[i][1] < 0.0)
                      {
                          r[i][1] += geometry->l[1];
                          c[i][1]--;
                      }
                      while(r[i][1] >= geometry->l[1])
                      {
                          r[i][1] -= geometry->l[1];
                          c[i][1]++;
                      }
while(r[i][2] < 0.0)
                      {
                          r[i][2] += geometry->l[2];
                          c[i][2]--;
                      }
                      while(r[i][2] >= geometry->l[2])
                      {
                          r[i][2] -= geometry->l[2];
                          c[i][2]++;
                      }
        }
    }
    else
    {
            /*  box deformation  */
        for(i = 0;i < particles->n;i++)
        {
            while(r[i][1] < 0.0)
            {
                r[i][1] += geometry->l[1];
                r[i][0] += geometry->delta_l;
                c[i][1]--;
            }
            while(r[i][1] >= geometry->l[1])
            {
                r[i][1] -= geometry->l[1];
                r[i][0] -= geometry->delta_l;
                c[i][1]++;
            }
            while(r[i][0] < 0.0)
            {
                r[i][0] += geometry->l[0];
                c[i][0]--;
            }
            while(r[i][0] >= geometry->l[0])
            {
                r[i][0] -= geometry->l[0];
                c[i][0]++;
            }
            while(r[i][2] < 0.0)
                   {
                       r[i][2] += geometry->l[2];
                       c[i][2]--;
                   }
                   while(r[i][2] >= geometry->l[2])
                   {
                       r[i][2] -= geometry->l[2];
                       c[i][2]++;
                   }
        }
    }

}


/*---------------------------------------------------------------------------*/
