dnl ---------------------------------------------------------------------------
dnl
dnl  particles.c.m4
dnl
dnl ---------------------------------------------------------------------------
dnl
dnl  Copyright (C) 2007,2018 Takashi Uneyama
dnl
dnl  This program is free software; you can redistribute it and/or modify
dnl  it under the terms of the GNU General Public License as published by
dnl  the Free Software Foundation; either version 2, or (at your option)
dnl  any later version.
dnl
dnl  This program is distributed in the hope that it will be useful,
dnl  but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl  GNU General Public License for more details.
dnl
dnl  You should have received a copy of the GNU General Public License
dnl  along with this program; if not, write to the Free Software
dnl  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
dnl  USA.
dnl
dnl ---------------------------------------------------------------------------
include(`dimension.m4')`'dnl
GENERATE_HEADER(particles.c,
                the particles and transient bonds.)

#include "common.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_MATH_H
#include <math.h>
#endif

#include "xmalloc.h"
#include "vector.h"
#include "rng.h"
#include "particles.h"

#define BOND_NUMBER_WIDTH_FACTOR (10.0)
#define MAXIMUM_BOND_NUMBER_LOWER_LIMIT (0x100)

/*---------------------------------------------------------------------------*/
PROTOTYPE_FUNCTIONS(`dnl
static void allocate_bonded_particles@D@(int n,bonded_particles@D@ *particles);
static void free_bonded_particles@D@(bonded_particles@D@ *particles);
static void set_particle_positions@D@_random(bonded_particles@D@ *particles,
                                             box_geometry *geometry,
                                             rng_state *rng);
static void set_particle_positions@D@_into_box(bonded_particles@D@ *particles,
                                               box_geometry *geometry);
static void set_particle_positions@D@_star_into_box(bonded_particles@D@ *particles,
                                                    box_geometry *geometry);'`')

/*---------------------------------------------------------------------------*/
bonded_particles *allocate_bonded_particles(int n,int m_max,
                                            box_geometry *geometry)
{
    bonded_particles *particles;
    int i;

    particles = xmalloc(sizeof(bonded_particles));
    particles->dimension = geometry->dimension;

    SWITCH_FUNCTION(geometry->dimension,
                    allocate_bonded_particles@D@(n,&(particles->particles@D@));,
                    free(particles); particles = NULL;)

    particles->bonds.m_max = m_max;
    particles->bonds.head = 0;
    particles->bonds.s = xmalloc(sizeof(index2) * m_max);

    for(i = 0;i < m_max;i++)
    {
        particles->bonds.s[i][0] = INACTIVE_BOND;
        particles->bonds.s[i][1] = i + 1;
    }
    particles->bonds.s[m_max - 1][1] = INACTIVE_BOND;

    return particles;
}

/*---------------------------------------------------------------------------*/
void free_bonded_particles(bonded_particles *particles)
{
    if(particles == NULL)
        return;

    SWITCH_FUNCTION(particles->dimension,
                    free_bonded_particles@D@(&(particles->particles@D@));)

    free(particles->bonds.s);
}

/*---------------------------------------------------------------------------*/
int estimate_maximum_number_of_bonds(simulation_condition *condition,
                                     box_geometry *geometry)
{
    int n,m_max;
    double m_av;

    n = condition->n;
    m_av = 0.5 * n * (n - 1) * condition->xi / calculate_volume_of_box(geometry);
    switch(geometry->dimension)
    {
        case 1:
            m_av *= sqrt(2.0 * M_PI);
            break;

        case 2:
            m_av *= M_PI;
            break;

        case 3:
            m_av *= (2.0 * M_PI / 3.0) * sqrt(2.0 * M_PI / 3.0);
            break;

        default:
            m_av = 0.0;
            break;
    }

    m_max = (int)(m_av + BOND_NUMBER_WIDTH_FACTOR * sqrt(m_av));
    if(m_max < MAXIMUM_BOND_NUMBER_LOWER_LIMIT)
    {
        m_max = MAXIMUM_BOND_NUMBER_LOWER_LIMIT;
    }

    return m_max;
}

/*---------------------------------------------------------------------------*/
void set_particle_positions_random(bonded_particles *particles,
                                   box_geometry *geometry,rng_state *rng)
{
    SWITCH_FUNCTION(particles->dimension,
                    set_particle_positions@D@_random(&(particles->particles@D@),geometry,rng);)
}

/*---------------------------------------------------------------------------*/
void set_particle_positions_into_box(bonded_particles *particles,
                                     box_geometry *geometry)
{
    SWITCH_FUNCTION(particles->dimension,
                    set_particle_positions@D@_into_box(&(particles->particles@D@),geometry);)
}

/*---------------------------------------------------------------------------*/
void set_particle_positions_star_into_box(bonded_particles *particles,
                                          box_geometry *geometry)
{
    SWITCH_FUNCTION(particles->dimension,
                    set_particle_positions@D@_star_into_box(&(particles->particles@D@),geometry);)
}

/*---------------------------------------------------------------------------*/
EXPAND_FUNCTIONS(`dnl
void allocate_bonded_particles@D@(int n,bonded_particles@D@ *particles)
{
    particles->n = n;
    particles->r = xmalloc(sizeof(vector@D@) * n);
    particles->f = xmalloc(sizeof(vector@D@) * n);
    particles->w = xmalloc(sizeof(vector@D@) * n);
    particles->r_star = xmalloc(sizeof(vector@D@) * n);
    particles->f_star = xmalloc(sizeof(vector@D@) * n);
    particles->c = xmalloc(sizeof(index@D@) * n);
}

/*---------------------------------------------------------------------------*/
void free_bonded_particles@D@(bonded_particles@D@ *particles)
{
    free(particles->r);
    free(particles->f);
    free(particles->w);
    free(particles->r_star);
    free(particles->f_star);
    free(particles->c);
}

/*---------------------------------------------------------------------------*/
void set_particle_positions@D@_random(bonded_particles@D@ *particles,
                                      box_geometry *geometry,rng_state *rng)
{
    int i;

    for(i = 0;i < particles->n;i++)
    {
        VECTOR@D@(particles->r[i][@@] = geometry->l[@@] * urand(rng);)
        VECTOR@D@(particles->c[i][@@] = 0;)
    }
}

/*---------------------------------------------------------------------------*/
dnl  GENERATE_SET_PARTICLE_POSITIONS_INTO_BOX
dnl  $1  0, 1. For 0, r_star is set into the box. For 1, r is set into the
dnl      box and the cell index is updated simultaneously.
define(`GENERATE_SET_PARTICLE_POSITIONS_INTO_BOX',`dnl

void set_particle_positions@D@_`'ifelse($1,`0',star_)into_box(bonded_particles@D@ *particles,
                                        box_geometry *geometry)
{
    int i;
    vector@D@ *r;
    ifelse($1,`1',index@D@ *c;)

    r = particles->ifelse($1,`0',r_star,r);
    ifelse($1,`1',c = particles->c;)

ifelse(@D@,`1',
        /*  no box deformation in 1d  */
    for(i = 0;i < particles->n;i++)
    {
        while(r[i][0] < 0.0)
        {
            r[i][0] += geometry->l[0];
            ifelse($1,`1',c[i][0]--;)
        }
        while(r[i][0] >= geometry->l[0])
        {
            r[i][0] -= geometry->l[0];
            ifelse($1,`1',c[i][0]++;)
        }
    },

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(i = 0;i < particles->n;i++)
        {
            VECTOR@D@(while(r[i][@@] < 0.0)
                      {
                          r[i][@@] += geometry->l[@@];
                          ifelse($1,`1',c[i][@@]--;)
                      }
                      while(r[i][@@] >= geometry->l[@@])
                      {
                          r[i][@@] -= geometry->l[@@];
                          ifelse($1,`1',c[i][@@]++;)
                      })
        }
    }
    else
    {
            /*  box deformation  */
        for(i = 0;i < particles->n;i++)
        {
            while(r[i][1] < 0.0)
            {
                r[i][1] += geometry->l[1];
                r[i][0] += geometry->delta_l;
                ifelse($1,`1',c[i][1]--;)
            }
            while(r[i][1] >= geometry->l[1])
            {
                r[i][1] -= geometry->l[1];
                r[i][0] -= geometry->delta_l;
                ifelse($1,`1',c[i][1]++;)
            }
            while(r[i][0] < 0.0)
            {
                r[i][0] += geometry->l[0];
                ifelse($1,`1',c[i][0]--;)
            }
            while(r[i][0] >= geometry->l[0])
            {
                r[i][0] -= geometry->l[0];
                ifelse($1,`1',c[i][0]++;)
            }
            ifelse(@D@,3,
                   while(r[i][2] < 0.0)
                   {
                       r[i][2] += geometry->l[2];
                       ifelse($1,`1',c[i][2]--;)
                   }
                   while(r[i][2] >= geometry->l[2])
                   {
                       r[i][2] -= geometry->l[2];
                       ifelse($1,`1',c[i][2]++;)
                   })
        }
    }
)
}')dnl

GENERATE_SET_PARTICLE_POSITIONS_INTO_BOX(0)

/*---------------------------------------------------------------------------*/
GENERATE_SET_PARTICLE_POSITIONS_INTO_BOX(1)
'`')

/*---------------------------------------------------------------------------*/
