/*---------------------------------------------------------------------------*/

/*!  \file  particles.h
     \brief  the particles and transient bonds.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007,2008,2018,2019 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_PARTICLES_H
#define INCLUDE_PARTICLES_H

#include "rng.h"
#include "vector.h"
#include "tensor.h"
#include "condition.h"
#include "geometry.h"

#define INACTIVE_BOND (-1)
#define IS_ACTIVE_BOND(x) ((x[0]) != (-1))
#define IS_INACTIVE_BOND(x) ((x[0]) == (-1))

/*---------------------------------------------------------------------------*/

/*!  \brief  positions of particles and foreces acting on particles in 1d.  */
typedef struct bonded_particles1
{
        /*!  \brief  number of particles.  */
    int n;

        /*!  \brief  positions of particles.  */
    vector1 *r;
        /*!  \brief  thermodynamic and potential forces acting on particles.  */
    vector1 *f;
        /*!  \brief  random forces acting on particles.  */
    vector1 *w;

        /*!  \brief  positions of particles at the middle point.  */
    vector1 *r_star;
        /*!  \brief  forces acting on particles at the middle point.  */
    vector1 *f_star;

        /*!  \brief  cell index for particles.  */
    index1 *c;

        /*!  \brief  stress tensor.  */
    symmetric_tensor1 sigma;

        /*!  \brief  potential energy.  */
    double e[2];
} bonded_particles1;

/*!  \brief  positions of particles and foreces acting on particles in 2d.  */
typedef struct bonded_particles2
{
        /*!  \brief  number of particles.  */
    int n;

        /*!  \brief  positions of particles.  */
    vector2 *r;
        /*!  \brief  thermodynamic and potential forces acting on particles.  */
    vector2 *f;
        /*!  \brief  random forces acting on particles.  */
    vector2 *w;

        /*!  \brief  positions of particles at the middle point.  */
    vector2 *r_star;
        /*!  \brief  forces acting on particles at the middle point.  */
    vector2 *f_star;

        /*!  \brief  cell index for particles.  */
    index2 *c;

        /*!  \brief  stress tensor.  */
    symmetric_tensor2 sigma;

        /*!  \brief  potential energy.  */
    double e[2];
} bonded_particles2;

/*!  \brief  positions of particles and foreces acting on particles in 3d.  */
typedef struct bonded_particles3
{
        /*!  \brief  number of particles.  */
    int n;

        /*!  \brief  positions of particles.  */
    vector3 *r;
        /*!  \brief  thermodynamic and potential forces acting on particles.  */
    vector3 *f;
        /*!  \brief  random forces acting on particles.  */
    vector3 *w;

        /*!  \brief  positions of particles at the middle point.  */
    vector3 *r_star;
        /*!  \brief  forces acting on particles at the middle point.  */
    vector3 *f_star;

        /*!  \brief  cell index for particles.  */
    index3 *c;

        /*!  \brief  stress tensor.  */
    symmetric_tensor3 sigma;

        /*!  \brief  potential energy.  */
    double e[2];
} bonded_particles3;

typedef struct transient_bonds
{
        /*!  \brief  maximum numer of bonds  */
    int m_max;

        /*!  \brief  head for unused bond index  */
    int head;

        /*!  \brief  indices for transient bonds  */
    index2 *s;

        /*   `s' contains both active and inactive bonds.
             If the k-th bond is active, two indicies s[k][0], s[k][1]
             means the indices of two bonded particles. If the k-th
             bond is inactive, s[k][0] is INACTIVE_BOND and s[k][1] means
             the next inactive bond (s[k][1] is INACTIVE_BOND if there
             is no next inactive bond). The index of the first inactive
             bond is `head'.  */
} transient_bonds;

/*!  \brief  positions of particles and foreces acting on particles.  */
typedef struct bonded_particles
{
        /*  \brief  dimension of the system.  */
    int dimension;

        /*!  \brief  particles in 1d.  */
    bonded_particles1 particles1;
        /*!  \brief  particles in 2d.  */
    bonded_particles2 particles2;
        /*!  \brief  particles in 3d.  */
    bonded_particles3 particles3;

    transient_bonds bonds;
} bonded_particles;

/*---------------------------------------------------------------------------*/
/*!  \brief  allocate new particles and bonds.
     \param  n  number of particles
     \param  m_max  maximum number of bonds
     \param  geometry  geometry of the simulation box  */
bonded_particles *allocate_bonded_particles(int n,int m_max,
                                            box_geometry *geometry);
/*!  \brief  free particles and bonds.
     \param  particles  bonded particles to free  */
void free_bonded_particles(bonded_particles *particles);
/*   \brief  set particle positions randomly.
     \param  particles  bonded particles
     \param  blend  information of the polymer blend
     \param  geometry  geometry of the simulation box
     \param  rng  random number generator  */
void set_particle_positions_random(bonded_particles *particles,
                                   box_geometry *geometry,rng_state *rng);
/*   \brief  set particles which positions are out of the box into the box.
     \param  particles  bonded particles
     \param  geometry  geometry of the simulation box  */
void set_particle_positions_into_box(bonded_particles *particles,
                                     box_geometry *geometry);
void set_particle_positions_star_into_box(bonded_particles *particles,
                                          box_geometry *geometry);
/*   \brief  estimate the maximum number of bonds.
     \param  condition  simulation condition
     \param  geometry  box geometry  */
int estimate_maximum_number_of_bonds(simulation_condition *condition,
                                     box_geometry *geometry);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
