/*---------------------------------------------------------------------------*/

/*!  \file  simulator.c
     \brief  a simple transient bond model simulator.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007,2008,2018,2019 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_MATH_H
#include <math.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS 0
#endif
#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif

#include "condition.h"
#include "files.h"
#include "geometry.h"
#include "particles.h"
#include "textwrite.h"
#include "force.h"
#include "integrator.h"
#include "celllist.h"
#include "bonds.h"
#include "dxwrite.h"

/*---------------------------------------------------------------------------*/
static int filename_is_valid(char *filename);
static void initialize_files(simulation_condition *condition,
                             data_files *files);
static void write_data_to_files(simulation_condition *condition,
                                data_files *files,int n,
                                bonded_particles *particles,
                                linked_cell_list *celllist,
                                box_geometry *geometry);

/*---------------------------------------------------------------------------*/
int simulate(simulation_condition *condition,
             data_files *files,
             box_geometry *geometry,
             bonded_particles *particles,
             linked_cell_list *celllist,
             rng_state *rng)
{
    int i,n;

    if((geometry->dimension != 1) && (geometry->dimension != 2)
       && (geometry->dimension != 3))
    {
        fprintf(stderr,
                "%d dimensional simulations are not supported (only 2 and 3 are supported).",
                geometry->dimension);
        return EXIT_FAILURE;
    }

    fprintf(stderr,"dimensions: %d\n",geometry->dimension);
    fprintf(stderr,"number of particles: %d\n",condition->n);
    fprintf(stderr,"maximum number of bonds: %d\n",particles->bonds.m_max);
    fprintf(stderr,"average particle density (%dD): %g\n",
            geometry->dimension,
            condition->n / (geometry->l[0] * geometry->l[1] * geometry->l[2]));
    fprintf(stderr,"bond fugacity: %g\n",condition->xi);
    fprintf(stderr,"average bond life time: %g\n",condition->tau);
    fprintf(stderr,"cutoff radius: %f\n",sqrt(celllist->rc2));
    if(geometry->dimension == 1)
    {
        fprintf(stderr,"cells: %d\n",celllist->nx);
    }
    else if(geometry->dimension == 2)
    {
        fprintf(stderr,"cells: %d %d\n",celllist->nx,celllist->ny);
    }
    else
    {
        fprintf(stderr,"cells: %d %d %d\n",celllist->nx,celllist->ny,
                celllist->nz);
    }
    if(condition->apply_shear)
    {
        if(condition->omega > 0.0)
            fprintf(stderr,"shear flow: oscillatory (kappa: %g, omega: %g)\n",
                    condition->kappa,condition->omega);
        else
            fprintf(stderr,"shear flow: steady (kappa: %g)\n",
                    condition->kappa);
    }
    fprintf(stderr,"time step: %g\n",condition->dt);


    if(condition->interval <= 0)
        condition->interval = condition->iteration_max;

    if(condition->apply_shear)
    {
        if(condition->omega < 0.0)
            condition->omega = 0.0;
    }
    else
    {
        condition->kappa = 0.0;
        condition->omega = 0.0;
    }

    geometry->delta_l = 0.0;
    geometry->c = 0;
    geometry->theta = 0.0;

    initialize_files(condition,files);


        /*  main simulation loop.  */
    n = condition->iteration_step;

    for(i = 0;i < condition->iteration_max;i++)
    {
        if((i % condition->interval) == 0)
        {
            write_data_to_files(condition,files,n,
                                particles,celllist,geometry);
            n++;
        }

        integrate_langevin_equation(condition,particles,celllist,geometry,rng);
        reconstruct_bonds_by_transition(condition,particles,celllist,geometry,rng);
    }

    write_data_to_files(condition,files,n,
                        particles,celllist,geometry);


    fprintf(stderr,"number of total iterations: %d\n",i);

    return EXIT_SUCCESS;
}

/*---------------------------------------------------------------------------*/
void initialize_files(simulation_condition *condition,
                      data_files *files)
{
    FILE *fp;

    if(filename_is_valid(files->stress_output) &&
       condition->save_stress_sequential)
    {
        fp = fopen(files->stress_output,"wt");
        fclose(fp);
    }

    if(filename_is_valid(files->energy_output) &&
       condition->save_energy_sequential)
    {
        fp = fopen(files->energy_output,"wt");
        fclose(fp);
    }

    if(filename_is_valid(files->geometry_output) &&
       condition->save_geometry_sequential)
    {
        fp = fopen(files->geometry_output,"wt");
        fclose(fp);
    }
}

/*---------------------------------------------------------------------------*/
void write_data_to_files(simulation_condition *condition,
                         data_files *files,int n,
                         bonded_particles *particles,
                         linked_cell_list *celllist,
                         box_geometry *geometry)
{
    static char filename[FILENAME_BUFFERSIZE];

    if(condition->save_position_sequential)
    {
        sprintf(filename,files->position_template,n);
        if(filename_is_valid(filename))
           write_positions_text(filename,particles);
    }
    else if(filename_is_valid(files->position_output))
    {
        sprintf(filename,"%s~",files->position_output);
        write_positions_text(filename,particles);
        rename(filename,files->position_output);
    }

    if(condition->save_bond_sequential)
    {
        sprintf(filename,files->bond_template,n);
        if(filename_is_valid(filename))
           write_bonds_text(filename,particles);
    }
    else if(filename_is_valid(files->bond_output))
    {
        sprintf(filename,"%s~",files->bond_output);
        write_bonds_text(filename,particles);
        rename(filename,files->bond_output);
    }

    if(condition->save_dx_sequential)
    {
        sprintf(filename,files->dx_template,n);
        if(filename_is_valid(filename))
            write_positions_dx(filename,particles,geometry);
    }
    else if(filename_is_valid(files->dx_output))
    {
        sprintf(filename,"%s~",files->dx_output);
        write_positions_dx(filename,particles,geometry);
        rename(filename,files->dx_output);
    }

    if(filename_is_valid(files->stress_output))
    {
        calculate_stress_tensor(particles,celllist,geometry,condition->xi);

        if(condition->save_stress_sequential)
        {
            write_stress_text(files->stress_output,particles,geometry,TRUE);
        }
        else
        {
            sprintf(filename,"%s~",files->stress_output);
            write_stress_text(filename,particles,geometry,FALSE);
            rename(filename,files->stress_output);
        }
    }

    if(filename_is_valid(files->energy_output))
    {
        calculate_potential_energy(particles,celllist,geometry,condition->xi);

        if(condition->save_energy_sequential)
        {
            write_energy_density_text(files->energy_output,particles,geometry,TRUE);
        }
        else
        {
            sprintf(filename,"%s~",files->energy_output);
            write_energy_density_text(filename,particles,geometry,FALSE);
            rename(filename,files->energy_output);
        }
    }

    if(filename_is_valid(files->geometry_output))
    {
        if(condition->save_geometry_sequential)
        {
            write_box_geometry_text(files->geometry_output,geometry,TRUE);
        }
        else 
        {
            sprintf(filename,"%s~",files->geometry_output);
            write_box_geometry_text(filename,geometry,FALSE);
            rename(filename,files->geometry_output);
        }
    }
}

/*---------------------------------------------------------------------------*/
int filename_is_valid(char *filename)
{
    if((filename == NULL) || (strlen(filename) == 0))
    {
        return FALSE;
    }
    else
    {
        return TRUE;
    }
}

/*---------------------------------------------------------------------------*/
