/*---------------------------------------------------------------------------*/

/*!  \file  store.c
     \brief  store the state to a HDF5 file  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2009,2010,2018,2019 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#include "store.h"
#include "xmalloc.h"
#include <stdlib.h>

/*---------------------------------------------------------------------------*/
#ifdef HAVE_HDF5
#include <hdf5.h>
#include <hdf5_hl.h>

/*---------------------------------------------------------------------------*/
static void store_rng_to_hdf5(hid_t file,rng_state *rng);
static void store_bonded_particles_to_hdf5(hid_t file,
                                           bonded_particles *particles);
static void store_box_geometry_to_hdf5(hid_t file,box_geometry *geometry);
static void make_dataset_unsignedlong(hid_t localtion,char *path,int rank,
                                      hsize_t *dims,unsigned long *src);

/*---------------------------------------------------------------------------*/
void store_rng_to_hdf5(hid_t file,rng_state *rng)
{
    hid_t group1,group2;
    hsize_t dims[1];
    int next_offset;

    group1 = H5Gcreate2(file,"/rng_state",H5P_DEFAULT,H5P_DEFAULT,H5P_DEFAULT);
    group2 = H5Gcreate2(file,"/rng_state/mt_state",H5P_DEFAULT,H5P_DEFAULT,H5P_DEFAULT);

    dims[0] = rng->mt->n;
    make_dataset_unsignedlong(file,"/rng_state/mt_state/state",
                              1,dims,rng->mt->state);

    dims[0] = 1;
    H5LTmake_dataset_int(file,"/rng_state/mt_state/n",
                         1,dims,&(rng->mt->n));
    H5LTmake_dataset_int(file,"/rng_state/mt_state/left",
                         1,dims,&(rng->mt->left));
    H5LTmake_dataset_int(file,"/rng_state/mt_state/initf",
                         1,dims,&(rng->mt->initf));
    next_offset = rng->mt->next - rng->mt->state;
    H5LTmake_dataset_int(file,"/rng_state/mt_state/next",
                         1,dims,&next_offset);

    dims[0] = 1;
    H5LTmake_dataset_int(file,"/rng_state/nrand_has_saved_number",
                         1,dims,&(rng->nrand_has_saved_number));
    H5LTmake_dataset_double(file,"/rng_state/nrand_w",
                            1,dims,&(rng->nrand_w));

    H5Gclose(group2);
    H5Gclose(group1);
}

/*---------------------------------------------------------------------------*/
void store_bonded_particles_to_hdf5(hid_t file,bonded_particles *particles)
{
    int i,j,n,m;
    hid_t group;
    hsize_t dims[2];
    double *r;
    int *c;
    index2 *s;

    group = H5Gcreate2(file,"/bonded_particles",H5P_DEFAULT,H5P_DEFAULT,
                       H5P_DEFAULT);

    switch(particles->dimension)
    {
        case 1:
            n = particles->particles1.n;
            r = (double *)(particles->particles1.r);
            c = (int *)(particles->particles1.c);
            break;

        case 2:
            n = particles->particles2.n;
            r = (double *)(particles->particles2.r);
            c = (int *)(particles->particles2.c);
            break;

        case 3:
            n = particles->particles3.n;
            r = (double *)(particles->particles3.r);
            c = (int *)(particles->particles3.c);
            break;

        default:
            break;
    }
    m = total_number_of_bonds(&(particles->bonds));

    dims[0] = 1;
    H5LTmake_dataset_int(file,"/bonded_particles/dimension",1,dims,
                         &(particles->dimension));
    H5LTmake_dataset_int(file,"/bonded_particles/n",1,dims,
                         &n);
    H5LTmake_dataset_int(file,"/bonded_particles/m",1,dims,
                         &m);

    dims[0] = n;
    dims[1] = particles->dimension;
    H5LTmake_dataset_double(file,"/bonded_particles/r",
                            2,dims,r);
    H5LTmake_dataset_int(file,"/bonded_particles/c",
                         2,dims,c);


        /*  pick-up the active bonds  */
    s = xmalloc(sizeof(index2) * m);
    j = 0;
    for(i = 0;i < particles->bonds.m_max;i++)
    {
        if(IS_ACTIVE_BOND(particles->bonds.s[i]))
        {
            s[j][0] = particles->bonds.s[i][0];
            s[j][1] = particles->bonds.s[i][1];
            j++;
        }
    }

    dims[0] = m;
    dims[1] = 2;
    H5LTmake_dataset_int(file,"/bonded_particles/s",
                         2,dims,(int *)s);

    free(s);

    H5Gclose(group);
}

/*---------------------------------------------------------------------------*/
void store_box_geometry_to_hdf5(hid_t file,box_geometry *geometry)
{
    hid_t group;
    hsize_t dims[1];

    group = H5Gcreate2(file,"/box_geometry",H5P_DEFAULT,H5P_DEFAULT,
                       H5P_DEFAULT);

    dims[0] = 1;
    H5LTmake_dataset_int(file,"/box_geometry/dimension",1,dims,
                         &(geometry->dimension));
    H5LTmake_dataset_double(file,"/box_geometry/theta",1,dims,
                            &(geometry->theta));
    H5LTmake_dataset_double(file,"/box_geometry/delta_l",1,dims,
                            &(geometry->delta_l));
    H5LTmake_dataset_int(file,"/box_geometry/c",1,dims,
                         &(geometry->c));

    dims[0] = geometry->dimension;
    H5LTmake_dataset_double(file,"/box_geometry/l",1,dims,
                            geometry->l);

    H5Gclose(group);
}

/*---------------------------------------------------------------------------*/
void make_dataset_unsignedlong(hid_t location,char *path,int rank,
                               hsize_t *dims,unsigned long *src)
{
    hid_t data,space;

    space = H5Screate_simple(rank,dims,NULL);
    data = H5Dcreate2(location,path,H5T_NATIVE_ULONG,space,H5P_DEFAULT,
                      H5P_DEFAULT,H5P_DEFAULT);

    H5Dwrite(data,H5T_NATIVE_ULONG,H5S_ALL,H5S_ALL,H5P_DEFAULT,src);

    H5Dclose(data);
    H5Sclose(space);
}

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
void h5_store_state_to_file(char *h5filename,
                            bonded_particles *particles,
                            box_geometry *geometry,
                            rng_state *rng)
{
#ifdef HAVE_HDF5
    hid_t file;
 
    file = H5Fcreate(h5filename,H5F_ACC_TRUNC,H5P_DEFAULT,H5P_DEFAULT);

    store_rng_to_hdf5(file,rng);
    store_box_geometry_to_hdf5(file,geometry);
    store_bonded_particles_to_hdf5(file,particles);

    H5Fclose(file);
#else
    return;
#endif
}

/*---------------------------------------------------------------------------*/
