/*---------------------------------------------------------------------------*/

/*!  \file  teaberry.c
     \brief  a simple transient bond model simulator.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007,2008,2018 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#ifdef HAVE_MATH_H
#include <math.h>
#endif

#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS 0
#endif
#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif

#include "getopt.h"
#include "condition.h"
#include "files.h"
#include "geometry.h"
#include "luaread.h"
#include "mt.h"
#include "particles.h"
#include "bonds.h"
#include "simulator.h"
#include "store.h"
#include "load.h"

/*---------------------------------------------------------------------------*/
static char *programstring = "teaberry";

static struct option longopts[] =
{
    {"input", required_argument, NULL, 'i'},
    {"state", required_argument, NULL, 's'},
    {"version", no_argument, NULL, 'v'},
    {"help", no_argument, NULL, 'h'},
    {0, 0, NULL, 0},
};

static char *optstring = "i:s:vh";

static char *helpstring =
"\
Usage: %s [options] [input]\n\
Options:\n\
  -i, --input <file>       use <file> for the input file for the simulator\n\
  -s, --state <file>       read the state from <file>\n\
  -v, --version            output version information and exit\n\
  -h, --help               display this help and exit\n\
";

static char *copyingstring = COPYRIGHT_STRING;

/*---------------------------------------------------------------------------*/
static int process_option(int argc,char **argv,char **input_lua,
                          char **h5_input);

/*---------------------------------------------------------------------------*/
int main(int argc,char **argv)
{
    char *input_lua = DEFAULT_INPUT_FILE;
    char *h5_input = NULL;
    data_files *files;
    simulation_condition *condition;
    box_geometry *geometry;
    bonded_particles *particles;
    linked_cell_list *celllist;
    rng_state *rng;

    if(process_option(argc,argv,&input_lua,&h5_input) != 0)
    {
        return EXIT_FAILURE;
    }

    read_input_lua(input_lua,&condition,&files,&geometry);

#ifndef HAVE_HDF5
    h5_input = NULL;
#endif
    if(h5_input == NULL)
    {
        rng = allocate_rng_state();
        initialize_rng_state(rng,condition->seed);

        particles = allocate_bonded_particles(condition->n,
                                              estimate_maximum_number_of_bonds(condition,geometry),
                                              geometry);
        set_particle_positions_random(particles,geometry,rng);

        celllist = allocate_linked_cell_list(particles,
                                             estimate_squared_cutoff_radius(geometry->dimension,condition->xi),
                                             geometry);

        create_bonds_equilibrium(particles,celllist,geometry,rng,
                                 condition->xi);
    }
    else
    {
        h5_load_state_from_file(h5_input,condition,geometry,&particles,&rng);
        set_particle_positions_into_box(particles,geometry);

        celllist = allocate_linked_cell_list(particles,
                                             estimate_squared_cutoff_radius(geometry->dimension,condition->xi),
                                             geometry);
    }

    simulate(condition,files,geometry,particles,celllist,rng);

    h5_store_state_to_file(files->h5_output,particles,geometry,rng);


    free_bonded_particles(particles);
    free_simulation_condition(condition);
    free_data_files(files);
    free_box_geometry(geometry);
    free_rng_state(rng);
    free_linked_cell_list(celllist);

    return EXIT_SUCCESS;
}

/*---------------------------------------------------------------------------*/
int process_option(int argc,char **argv,char **input_lua,char **h5_input)
{
    int c;
    int option_index;
    char *program_name;

    program_name = argv[0];

    do
    {
        option_index = 0;
        c = getopt_long(argc,argv,optstring,longopts,&option_index);

        switch(c)
        {
            case 'i':
                    /*  input file  */
                *input_lua = optarg;
                break;

            case 's':
                    /*  state input file  */
                *h5_input = optarg;
                break;

            case 'v':
                    /*  version  */
                fprintf(stderr,copyingstring,programstring,PACKAGE,VERSION);
                exit(EXIT_SUCCESS);
                break;

            case 'h':
                    /*  help  */
                fprintf(stderr,helpstring,program_name);
                exit(EXIT_SUCCESS);
                break;

            case '?':
                    /*  invalid option  */
                exit(EXIT_FAILURE);
                break;

            case -1:
                break;

            default:
                fprintf(stderr,
                        "error: getopt returned character code 0%o\n", c);
                return 1;
        }
    }
    while(c != -1);


    if(optind < argc)
    {
            /*  input file  */
        *input_lua = argv[optind++];
    }

    return 0;
}

/*---------------------------------------------------------------------------*/
