/*---------------------------------------------------------------------------*/

/*!  \file  stress2modulus.c
     \brief  calculate the shear relaxation modulus from the stress data.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2018,2019 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#ifdef HAVE_MATH_H
#include <math.h>
#endif

#define BUFFER_SIZE (0x400)

/*---------------------------------------------------------------------------*/
static char *helpstring =
"\
Usage: %s input\n\
  input                    time-series stress data file (text format)\n\
";

/*---------------------------------------------------------------------------*/
void get_dimension_and_number_of_data(FILE *fp,int *d,int *n);
void scan_stress(FILE *fp,double *s,int d,int n);
void calculate_correlation_function(double *s,double *g,int d,int n);
void print_correlation_function(double *g,int d,int n);

/*---------------------------------------------------------------------------*/
int main(int argc,char **argv)
{
    FILE *fp;
    int d,n;
    double *s,*g;

    if(argc != 2)
    {
        fprintf(stderr,helpstring,argv[0]);
        return 0;
    }

    fp = fopen(argv[1],"rt");
    if(fp == NULL)
    {
        fprintf(stderr,"error: cannot open the file.\n");
        return 0;
    }

    get_dimension_and_number_of_data(fp,&d,&n);
    if((d == 0) || (n == 0))
    {
        fprintf(stderr,"error: `%s' does not contain correct stress data.\n");
        return 0;
    }
    /* if(d == 1) */
    /* { */
    /*     fprintf(stderr,"error: 1 dimensional data is not supported.\n"); */
    /*     return 0; */
    /* } */

    if(d == 1)
    {
        s = malloc(sizeof(double) * n);
        g = malloc(sizeof(double) * n);
    }
    else if(d == 2)
    {
        s = malloc(sizeof(double) * n * 2);
        g = malloc(sizeof(double) * n * 2);
    }
    else
    {
        s = malloc(sizeof(double) * n * 6);
        g = malloc(sizeof(double) * n * 6);
    }

    scan_stress(fp,s,d,n);
    calculate_correlation_function(s,g,d,n);
    print_correlation_function(g,d,n);

    free(s);
    free(g);

    return 0;
}

/*---------------------------------------------------------------------------*/
void get_dimension_and_number_of_data(FILE *fp,int *d,int *n)
{
    static char buf[BUFFER_SIZE];
    double s[6];
    int m,i;

    if(fgets(buf,BUFFER_SIZE - 1,fp) == NULL)
    {
        *d = 0;
        *n = 0;
        return;
    }

    m = sscanf(buf,"%lg %lg %lg %lg %lg %lg\n",
               s,s + 1,s + 2,s + 3,s + 4,s + 5);

    switch(m)
    {
        case 1:
            *d = 1;
            break;

        case 3:
            *d = 2;
            break;

        case 6:
            *d = 3;
            break;

        default:
            *d = 0;
            *n = 0;
            return;
    }

    i = 1;
    while(fgets(buf,BUFFER_SIZE - 1,fp) != NULL)
    {
        i++;
    }

    *n = i;

    rewind(fp);
}

/*---------------------------------------------------------------------------*/
void scan_stress(FILE *fp,double *s,int d,int n)
{
    int i;
    double x[6];

    if(d == 1)
    {
        for(i = 0;i < n;i++)
        {
            fscanf(fp,"%lg",x);

                /*  sigma_xx  */
            s[i] = x[0];
        }
    }
    else if(d == 2)
    {
        for(i = 0;i < n;i++)
        {
            fscanf(fp,"%lg %lg %lg",x,x + 1,x + 2);

                /*  sigma_xy, N_xy  */
            s[2 * i + 0] = x[2];
            s[2 * i + 1] = x[0] - x[1];
        }
    }
    else
    {
        for(i = 0;i < n;i++)
        {
            fscanf(fp,"%lg %lg %lg %lg %lg %lg",
                   x,x + 1,x + 2,x + 3,x + 4,x + 5);

                /*  sigma_xy, sigma_yz, sigma_zx, N_xy, N_yz, N_zx  */
            s[6 * i + 0] = x[3];
            s[6 * i + 1] = x[4];
            s[6 * i + 2] = x[5];
            s[6 * i + 3] = x[0] - x[1];
            s[6 * i + 4] = x[1] - x[2];
            s[6 * i + 5] = x[2] - x[0];
        }
    }
}

/*---------------------------------------------------------------------------*/
void calculate_correlation_function(double *s,double *g,int d,int n)
{
    int i,j;

    if(d == 1)
    {
        for(i = 0;i < n;i++)
        {
            g[i] = 0.0;

            for(j = 0;j < (n - i);j++)
            {
                g[i] += s[j] * s[j + i];
            }

            g[i] /= (n - i);
        }
    }
    else if(d == 2)
    {
        for(i = 0;i < n;i++)
        {
            g[2 * i + 0] = 0.0;
            g[2 * i + 1] = 0.0;

            for(j = 0;j < (n - i);j++)
            {
                g[2 * i + 0] += s[2 * j + 0] * s[2 * (j + i) + 0];
                g[2 * i + 1] += s[2 * j + 1] * s[2 * (j + i) + 1];
            }

            g[2 * i + 0] /= (n - i);
            g[2 * i + 1] /= (n - i);
        }
    }
    else
    {
        for(i = 0;i < n;i++)
        {
            g[6 * i + 0] = 0.0;
            g[6 * i + 1] = 0.0;
            g[6 * i + 2] = 0.0;
            g[6 * i + 3] = 0.0;
            g[6 * i + 4] = 0.0;
            g[6 * i + 5] = 0.0;

            for(j = 0;j < (n - i);j++)
            {
                g[6 * i + 0] += s[6 * j + 0] * s[6 * (j + i) + 0];
                g[6 * i + 1] += s[6 * j + 1] * s[6 * (j + i) + 1];
                g[6 * i + 2] += s[6 * j + 2] * s[6 * (j + i) + 2];
                g[6 * i + 3] += s[6 * j + 3] * s[6 * (j + i) + 3];
                g[6 * i + 4] += s[6 * j + 4] * s[6 * (j + i) + 4];
                g[6 * i + 5] += s[6 * j + 5] * s[6 * (j + i) + 5];
            }

            g[6 * i + 0] /= (n - i);
            g[6 * i + 1] /= (n - i);
            g[6 * i + 2] /= (n - i);
            g[6 * i + 3] /= (n - i);
            g[6 * i + 4] /= (n - i);
            g[6 * i + 5] /= (n - i);
        }
    }
}

/*---------------------------------------------------------------------------*/
void print_correlation_function(double *g,int d,int n)
{
    int i;

    if(d == 1)
    {
            /*  in 1D, only the bulk modulus can be defined:
               k_B T K(t) / V
               = <sigma_xx(t) sigma_xx(0)> - <sigma_xx>^2
               this program outputs <sigma_xx(t) sigma_xx(0)>  */
        for(i = 0;i < n;i++)
        {
            printf("%g\n",g[i]);
        }
    }
    else if(d == 2)
    {
        for(i = 0;i < n;i++)
        {
                /*  2D version of Likhtman's formula:
                    k_B T G(t) / V
                    = <sigma_xy(t) sigma_xy(0)> / 2
                      + <N_xy(t) N_xy(0)> / 8  */
            printf("%g\n",g[2 * i + 0] / 2.0 + g[2 * i + 1] / 8.0);
        }
    }
    else
    {
        for(i = 0;i < n;i++)
        {
                /*  Likhtman's formula:
                    k_B T G(t) / V
                    = (<sigma_xy(t) sigma_xy(0)> + <sigma_yz(t) sigma_yz(0)>
                       + <sigma_xy(t) sigma_xy(0)>) / 5
                      + (<N_xy(t) N_xy(0)> + <N_yz(t) N_yz(0)>
                         + <N_zx(t) N_zx(0)>) / 30  */
            printf("%g\n",
                   (g[6 * i + 0] + g[6 * i + 1] + g[6 * i + 2]) / 5.0
                   + (g[6 * i + 3] + g[6 * i + 4] + g[6 * i + 5]) / 30.0);
        }
    }
}

/*---------------------------------------------------------------------------*/
